//---------------------------------------------------------------------------
#ifdef _WIN32
#define _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES 1
#define _CRT_SECURE_NO_WARNINGS
#endif

#include <cassert>
#include <cstdlib>
#include <ctime>
#include <cerrno>

#include <sstream>
using std::stringstream;

#ifdef _WIN32
#include <windows.h>
#include <sdl.h>
/*#include <fmod.h>
#include <fmod_errors.h>*/
#endif

#ifdef __linux
#include <dirent.h>
#include <string.h>
#include <SDL/SDL.h>
/*#include <fmod/fmod.h>
#include <fmod/fmod_errors.h>*/
#endif

#include "game.h"
#include "utils.h"
#include "sound.h"
#include "image.h"
#include "sector.h"
#include "screen.h"
#include "gamestate.h"
#include "panel.h"
#include "gui.h"
#include "player.h"
#include "resource.h"

//---------------------------------------------------------------------------

const int majorVersion = 0;
const int minorVersion = 16;

int lastmouseclick_time = 0;

//int default_height_c = 256;
float scale_width = 0.0f;
float scale_height = 0.0f;
/*int screen_width = -1;
int screen_height = -1;*/

int offset_flag_x_c = 22;
int offset_flag_y_c = 6;

bool use_amigadata = true;

GameType gameType = GAMETYPE_SINGLEISLAND;
//GameType gameType = GAMETYPE_ALLISLANDS;

Image *player_select = NULL;
Image *background = NULL;
Image *land[MAP_N_COLOURS];
Image *fortress[n_epochs_c];
Image *mine[n_epochs_c];
Image *factory[n_epochs_c];
Image *lab[n_epochs_c];
Image *men[n_epochs_c];
Image *unarmed_man = NULL;
Image *flags[n_players_c][n_flag_frames_c];
Image *panel_design = NULL;
//Image *panel_design_dark = NULL;
Image *panel_lab = NULL;
Image *panel_factory = NULL;
Image *panel_shield = NULL;
Image *panel_defence = NULL;
Image *panel_attack = NULL;
Image *panel_bloody_attack = NULL;
Image *panel_twoattack = NULL;
Image *panel_build[Building::N_BUILDINGS];
Image *panel_building[Building::N_BUILDINGS];
Image *panel_knowndesigns = NULL;
Image *panel_bigdesign = NULL;
Image *panel_biglab = NULL;
Image *panel_bigfactory = NULL;
Image *panel_bigshield = NULL;
Image *panel_bigdefence = NULL;
Image *panel_bigattack = NULL;
Image *panel_bigbuild = NULL;
Image *panel_bigknowndesigns = NULL;
Image *numbers_blue[10];
Image *numbers_grey[10];
Image *numbers_white[10];
Image *numbers_orange[10];
Image *numbers_yellow[10];
Image *numbers_largegrey[10];
Image *numbers_largeshiny[10];
Image *numbers_small[n_players_c][10];
Image *numbers_half = NULL;
Image *letters_large[26];
Image *letters_small[26];
Image *mouse_pointers[n_players_c];
Image *playershields[n_playershields_c];
Image *building_health = NULL;
Image *dash_grey = NULL;
Image *icon_shield = NULL;
Image *icon_defence = NULL;
Image *icon_weapon = NULL;
Image *icon_shields[n_shields_c];
Image *icon_defences[n_epochs_c];
Image *icon_weapons[n_epochs_c];
Image *numbered_defences[n_epochs_c];
Image *numbered_weapons[n_epochs_c];
Image *icon_elements[Element::N_ID];
Image *icon_clocks[13];
Image *icon_infinity = NULL;
Image *icon_bc = NULL;
Image *icon_ad = NULL;
Image *icon_ad_shiny = NULL;
Image *icon_towers[n_players_c];
Image *icon_armies[n_players_c];
Image *icon_nuke_hole = NULL;
Image *mine_gatherable_small = NULL;
Image *mine_gatherable_large = NULL;
Image *icon_ergo = NULL;
Image *icon_trash = NULL;
Image *coast_icons[n_coast_c];
Image *map_sq[MAP_N_COLOURS][n_map_sq_c];
Image *defenders[n_players_c][n_epochs_c][n_defender_frames_c];
Image *nuke_defences[n_players_c];
Image *attackers_walking[n_players_c][n_epochs_c+1][n_attacker_frames_c];
Image *planes[n_players_c][n_epochs_c];
Image *nukes[n_players_c][n_nuke_frames_c];
Image *saucers[n_players_c][n_saucer_frames_c];
Image *attackers_ammo[n_epochs_c][N_ATTACKER_AMMO_DIRS];
Image *icon_openpitmine = NULL;
Image *icon_trees[4];
Image *flashingmapsquare = NULL;
Image *mapsquare = NULL;
Image *death_flashes[n_death_flashes_c];
Image *blue_flashes[n_blue_flashes_c];
Image *icon_mice[3];
Image *icon_speeds[3];

Image *background_islands = NULL;

// speech
Sample *s_design_is_ready = NULL;
Sample *s_ergo = NULL;
Sample *s_advanced_tech = NULL;
Sample *s_fcompleted = NULL;
Sample *s_on_hold = NULL;
Sample *s_running_out_of_elements = NULL;
Sample *s_tower_critical = NULL;
Sample *s_sector_destroyed = NULL;
Sample *s_mine_destroyed = NULL;
Sample *s_factory_destroyed = NULL;
Sample *s_lab_destroyed = NULL;
Sample *s_itis_all_over = NULL;
Sample *s_conquered = NULL;
Sample *s_won = NULL;
Sample *s_weve_nuked_them = NULL;
Sample *s_weve_been_nuked = NULL;
Sample *s_alliance_yes[n_players_c] = {NULL, NULL, NULL, NULL};
Sample *s_alliance_no[n_players_c] = {NULL, NULL, NULL, NULL};
Sample *s_alliance_ask[n_players_c] = {NULL, NULL, NULL, NULL};
Sample *s_quit[n_players_c] = {NULL, NULL, NULL, NULL};
Sample *s_cant_nuke_ally = NULL;

// effects
Sample *s_explosion = NULL;
Sample *s_scream = NULL;
Sample *s_buildingdestroyed = NULL;
Sample *s_guiclick = NULL;

const unsigned char shadow_alpha_c = (unsigned char)160;

const int epoch_dates[n_epochs_c] = {-10000, -2000, 1, 900, 1400, 1850, 1914, 1944, 1980, 2100};
const char *epoch_names[n_epochs_c] = { "FIRST", "SECOND", "THIRD", "FOURTH", "FIFTH", "SIXTH", "SEVENTH", "EIGHTH", "NINTH", "TENTH" };

Invention *invention_shields[n_epochs_c];
Invention *invention_defences[n_epochs_c];
Weapon *invention_weapons[n_epochs_c];

Element *elements[Element::N_ID];

Player *players[n_players_c];

//bool player_won = false;
GameResult gameResult = GAMERESULT_UNDEFINED;

GameStateID gameStateID = GAMESTATEID_UNDEFINED;
int human_player = 0;
//int enemy_player = 1;
//GameWon gameWon = GAME_PLAYING;
bool state_changed = false;

//const int n_men_per_epoch_c = 33;
int n_men_store = 0;
int n_suspended[n_players_c];
//int n_men_for_this_island = 0;

DifficultyLevel difficulty_level = DIFFICULTY_EASY;

int getMenPerEpoch() {
	ASSERT( gameType == GAMETYPE_ALLISLANDS );
	if( difficulty_level == DIFFICULTY_EASY )
		return 150;
	else if( difficulty_level == DIFFICULTY_MEDIUM )
		return 100;
	else if( difficulty_level == DIFFICULTY_HARD )
		return 75;
	ASSERT(false);
	return 0;
}

int getMenAvailable() {
	if( start_epoch == end_epoch_c && gameType == GAMETYPE_ALLISLANDS )
		return n_suspended[human_player];
	return n_men_store;
}

int getNSuspended() {
	return n_suspended[human_player];
}

int start_epoch = 0;
int n_sub_epochs = 4;
int selected_island = 0;
bool completed_island[max_islands_per_epoch_c];
//const int n_islands_c = 10;
Map *maps[n_epochs_c][max_islands_per_epoch_c];
Map *map = NULL;

Screen *screen = NULL;
GameState *gamestate = NULL;
Sector *current_sector = NULL;

bool play_music = true;
/*FMUSIC_MODULE *module = NULL;
FSOUND_STREAM *str_music = NULL;*/
Sample *music = NULL;

Map::Map(MapColour colour,int n_opponents,const char *name) {
	//*this->filename = '\0';
	this->colour = colour;
	this->n_opponents = n_opponents;
	for(int x=0;x<map_width_c;x++) {
		for(int y=0;y<map_height_c;y++) {
			sector_at[x][y] = false;
			sectors[x][y] = NULL;
			//panels[x][y] = NULL;
		}
	}
	/*for(int i=0;i<Element::N_ID;i++) {
	this->elements[i] = 0;
	}*/
	//clearTemp();
	/*strncpy(this->name,name,MAP_MAX_NAME);
	this->name[MAP_MAX_NAME] = '\0';*/
	this->name = name;
}

Map::~Map() {
	freeSectors();
}

/*void Map::clearTemp() {
	for(int x=0;x<map_width_c;x++) {
		for(int y=0;y<map_height_c;y++) {
			temp[x][y] = false;
		}
	}
}*/

const Sector *Map::getSector(int x, int y) const {
	ASSERT(x >= 0 && x < map_width_c && y >= 0 && y < map_height_c);
	Sector *sector = this->sectors[x][y];
	return sector;
}

Sector *Map::getSector(int x, int y) {
	ASSERT(x >= 0 && x < map_width_c && y >= 0 && y < map_height_c);
	Sector *sector = this->sectors[x][y];
	return sector;
}

bool Map::isSectorAt(int x, int y) const {
	ASSERT(x >= 0 && x < map_width_c && y >= 0 && y < map_height_c);
	return this->sector_at[x][y];
}

void Map::newSquareAt(int x,int y) {
	ASSERT(x >= 0 && x < map_width_c && y >= 0 && y < map_height_c);
	this->sector_at[x][y] = true;
}

void Map::createSectors(PlayingGameState *gamestate, int epoch) {
	ASSERT_EPOCH(epoch);
	for(int x=0;x<map_width_c;x++) {
		for(int y=0;y<map_height_c;y++) {
			if( sector_at[x][y] ) {
				this->sectors[x][y] = new Sector(gamestate, epoch, x, y);
				/*for(int i=0;i<Element::N_ID;i++) {
				this->sectors[x][y]->setElements(i, this->elements[i]);
				}*/
			}
		}
	}
}

#if 0
void Map::checkSectors() const {
	//LOG("Map::checkSectors()\n");
#ifdef _WIN32
	for(int x=0;x<map_width_c;x++) {
		for(int y=0;y<map_height_c;y++) {
			if( sectors[x][y] != NULL ) {
				//LOG("check sector at %d, %d : %d\n", x, y, sectors[x][y]);
				if( !_CrtIsValidHeapPointer( sectors[x][y] ) ) {
					LOG("invalid sector at %d, %d\n", x, y);
					LOG("    ptr %d\n", sectors[x][y]);
					ASSERT( false );
				}
			}
		}
	}
#endif
	//LOG("Map::checkSectors exit\n");
}
#endif

void Map::freeSectors() {
	LOG("Map::freeSectors()\n");
	for(int x=0;x<map_width_c;x++) {
		for(int y=0;y<map_height_c;y++) {
			if( sectors[x][y] != NULL ) {
				LOG("free sector at %d, %d\n", x, y);
				LOG("    ptr %d\n", sectors[x][y]);
#ifdef _WIN32
				ASSERT( _CrtIsValidHeapPointer( sectors[x][y] ) == TRUE );
#endif
				delete sectors[x][y];
				sectors[x][y] = NULL;
			}
		}
	}
	LOG("Map::freeSectors exit\n");
}

/*void Map::setElements(int id,int n_elements) {
ASSERT_ELEMENT_ID(id);
this->elements[id] = n_elements;
}*/

void Map::findRandomSector(int *rx,int *ry) const {
	while(true) {
		int x = rand() % map_width_c;
		int y = rand() % map_height_c;
		if( sector_at[x][y] ) {
			*rx = x;
			*ry = y;
			break;
		}
	}
}

void Map::canMoveTo(bool temp[map_width_c][map_height_c], int sx,int sy,int player) const {
	ASSERT(player != -1 );
	ASSERT(this->sector_at[sx][sy]);
	//clearTemp();
	//bool temp[map_width_c][map_height_c];

	for(int x=0;x<map_width_c;x++) {
		for(int y=0;y<map_height_c;y++) {
			temp[x][y] = false;
		}
	}

	temp[sx][sy] = true;
	bool changed = false;
	do {
		changed = false;

		for(int x=0;x<map_width_c;x++) {
			for(int y=0;y<map_height_c;y++) {
				if( temp[x][y] ) {
					// can we move through this square?
					if( ( x == sx && y == sy ) ||
						sectors[x][y]->getPlayer() == player ||
						( sectors[x][y]->getPlayer() == -1 &&
						!sectors[x][y]->enemiesPresent(player) ) ) {
							for(int c=0;c<4;c++) {
								int cx = x, cy = y;
								if( c == 0 )
									cy--;
								else if( c == 1 )
									cx++;
								else if( c == 2 )
									cy++;
								else if( c == 3 )
									cx--;
								if( cx >= 0 && cy >= 0 && cx < map_width_c && cy < map_height_c
									&& sector_at[cx][cy]
								&& !sectors[x][y]->isNuked()
									&& !temp[cx][cy] ) {
										temp[cx][cy] = true;
										changed = true;
								}
							}
					}
				}
			}
		}
	} while( changed );
}

void Map::calculateStats() const {
	// deaths = start + births - remaining
	for(int i=0;i<n_players_c;i++) {
		if( players[i] != NULL ) {
			players[i]->setNDeaths( players[i]->getNMenForThisIsland() + players[i]->getNBirths() );
			players[i]->setNSuspended(0);
		}
	}
	for(int x=0;x<map_width_c;x++) {
		for(int y=0;y<map_height_c;y++) {
			if( this->sector_at[x][y] ) {
				Sector *sector = this->sectors[x][y];
				if( sector->getPlayer() != -1 ) {
					players[sector->getPlayer()]->addNDeaths( - sector->getPopulation() );
					if( sector->isShutdown() && gameResult == GAMERESULT_WON ) {
						//players[sector->getPlayer()]->n_suspended += sector->getPopulation();
						players[sector->getPlayer()]->addNSuspended(sector->getPopulation());
					}
				}
				for(int i=0;i<n_players_c;i++) {
					if( players[i] != NULL ) {
						players[i]->addNDeaths( - sector->getArmy(i)->getTotalMen() );
					}
				}
			}
		}
	}
	/*for(int i=0;i<n_players_c;i++) {
	if( players[i] != NULL )
	ASSERT( players[i]->n_deaths >= 0 );
	}*/
}

/*bool Map::mapIs(char *that_name) {
//return strcmp( this->name, that_name ) == 0;
return this->name == that_name;
}*/

int Map::getNSquares() const {
	int n_squares = 0;
	for(int y=0;y<map_height_c;y++) {
		for(int x=0;x<map_width_c;x++) {
			if( map->sector_at[x][y] ) {
				n_squares++;
			}
		}
	}
	return n_squares;
}

void Map::draw(int offset_x, int offset_y) const {
	for(int y=0;y<map_height_c;y++) {
		for(int x=0;x<map_width_c;x++) {
			if( map->sector_at[x][y] ) {
				int icon = 0;
				if( y > 0 && map->sector_at[x][y-1] )
					icon += 1;
				if( x < map_width_c-1 && map->sector_at[x+1][y] )
					icon += 2;
				if( y < map_height_c-1 && map->sector_at[x][y+1] )
					icon += 4;
				if( x > 0 && map->sector_at[x-1][y] )
					icon += 8;
				ASSERT( icon >= 0 && icon < 16 );
				if( map_sq[colour][icon] == NULL ) {
					LOG("map name: %s\n", map->getName());
					LOG("ERROR map icon not available [%d,%d]: %d, %d\n", x, y, colour, icon);
					ASSERT( map_sq[colour][icon] != NULL );
				}
				int map_x = offset_x - 3 + 16 * x;
				int map_y = offset_y - 3 + 16 * y;
				map_sq[colour][icon]->draw(map_x, map_y, true);
				/*int coast = 15 - icon;
				if( coast > 0 && coast_icons[coast-1] != NULL ) {
				coast_icons[coast-1]->draw(map_x, map_y, true);
				}*/
				//LOG(">>> %d %d %d\n", icon, icon & 1, 4 & 1);
				if( (icon & 1) == 0 && coast_icons[0] != NULL )
					coast_icons[0]->draw(map_x, map_y, true);
				if( (icon & 2) == 0 && coast_icons[1] != NULL )
					coast_icons[1]->draw(map_x, map_y, true);
				if( (icon & 4) == 0 && coast_icons[3] != NULL )
					coast_icons[3]->draw(map_x, map_y, true);
				if( (icon & 8) == 0 && coast_icons[7] != NULL )
					coast_icons[7]->draw(map_x, map_y, true);
			}
		}
	}
}

void setEpoch(int epoch) {
	LOG("set epoch %d\n", epoch);
	ASSERT( gameStateID == GAMESTATEID_PLACEMEN );
	ASSERT( epoch >= 0 && epoch < n_epochs_c );
	start_epoch = epoch;

	n_sub_epochs = 4;
	//if( start_epoch == n_epochs_c-1 )
	if( start_epoch == end_epoch_c )
		n_sub_epochs = 0;
	else if( start_epoch + n_sub_epochs > n_epochs_c ) {
		n_sub_epochs = n_epochs_c - start_epoch;
	}

	selected_island = 0;

	if( gameType == GAMETYPE_ALLISLANDS ) {
		// skip islands we've completed
		while( completed_island[selected_island] ) {
			selected_island++;
			if( selected_island == max_islands_per_epoch_c || maps[start_epoch][selected_island] == NULL ) {
				LOG("error, should be at least one island on this epoch that isn't completed\n");
				ASSERT( false );
			}
		}
	}

	map = maps[start_epoch][selected_island];
	gamestate->reset();
}

void newGame() {
	LOG("newGame()\n");
	ASSERT( gameStateID == GAMESTATEID_PLACEMEN );
	if( gameType == GAMETYPE_SINGLEISLAND )
		n_men_store = 1000;
	else
		n_men_store = getMenPerEpoch();

	for(int i=0;i<max_islands_per_epoch_c;i++)
		completed_island[i] = false;
	//completed_island[0] = true;

	for(int i=0;i<n_players_c;i++)
		n_suspended[i] = 0;

	setEpoch(0);

	//n_men_store = 1000;
	//setEpoch(9);
}

void nextEpoch() {
	LOG("nextEpoch()\n");
	ASSERT( gameStateID == GAMESTATEID_PLACEMEN );
	start_epoch++;
	if( start_epoch == n_epochs_c ) {
		ASSERT( gameType == GAMETYPE_SINGLEISLAND );
		start_epoch = 0;
	}
	for(int i=0;i<max_islands_per_epoch_c;i++)
		completed_island[i] = false;
	setEpoch(start_epoch);
}

void nextIsland() {
	if( gameType == GAMETYPE_SINGLEISLAND ) {
		selected_island++;
		if( selected_island == max_islands_per_epoch_c || maps[start_epoch][selected_island] == NULL )
			selected_island = 0;
	}
	else {
		// skip islands we've completed
		do {
			selected_island++;
			if( selected_island == max_islands_per_epoch_c || maps[start_epoch][selected_island] == NULL )
				selected_island = 0;
		} while( completed_island[selected_island] );
	}
	LOG("next island: %d\n", selected_island);

	map = maps[start_epoch][selected_island];
	LOG("map name: %s\n", map->getName());
	gamestate->reset();
}

char *getFilename(int slot) {
	char name[300] = "";
	sprintf(name, "game_%d.SAV", slot);
	char *filename = getApplicationFilename(name);
	LOG("filename: %s\n", filename);
	return filename;
}

bool loadGameInfo(DifficultyLevel *difficulty, int *player, int *n_men, int suspended[n_players_c], int *epoch, bool completed[max_islands_per_epoch_c], int slot) {
	LOG("loadGameInfo(%d)\n",slot);
	ASSERT( gameStateID == GAMESTATEID_PLACEMEN );
	ASSERT( slot >= 0 && slot < n_slots_c );

	char *filename = getFilename(slot);
	FILE *file = fopen(filename, "rb+");
	delete [] filename;
	if( file == NULL ) {
		LOG("FAILED to open file\n");
		return false;
	}

	const int bufsize = 1024;
	char buffer[bufsize+1] = "";
	fgets(buffer, bufsize, file); // header line

	// data line
	/*for(int i=0;i<8;i++) {
	buffer[i] = fgetc(file);
	}*/
	for(int i=0;;) {
		ASSERT( i <= bufsize );
		buffer[i] = fgetc(file);
		if( buffer[i] == EOF ) {
			buffer[i] = '\0';
			break;
		}
		i++;
	}

	char *ptr = buffer;

	*difficulty = (DifficultyLevel)*((int *)ptr);
	ptr += sizeof(int);
	if( !validDifficulty( *difficulty ) )
		return false;

	*player = *((int *)ptr);
	ptr += sizeof(int);
	if( !validPlayer( *player ) )
		return false;

	*n_men = *((int *)ptr);
	ptr += sizeof(int);
	if( *n_men < 0 )
		return false;

	for(int i=0;i<n_players_c;i++) {
		suspended[i] = *((int *)ptr);
		ptr += sizeof(int);
		if( suspended[i] < 0 )
			return false;
	}

	*epoch = *((int *)ptr);
	ptr += sizeof(int);
	if( *epoch < 0 || *epoch >= n_epochs_c )
		return false;

	for(int i=0;i<max_islands_per_epoch_c;i++) {
		int val = *((int *)ptr);
		if( val != 0 && val != 1 )
			return false;
		completed[i] = (val==1);
		ptr += sizeof(int);
	}

	fclose(file);
	return true;
}

bool loadGame(int slot) {
	LOG("loadGame(%d)\n",slot);
	ASSERT( gameType == GAMETYPE_ALLISLANDS );
	ASSERT( gameStateID == GAMESTATEID_PLACEMEN );
	ASSERT( slot >= 0 && slot < n_slots_c );

	DifficultyLevel temp_difficulty = DIFFICULTY_EASY;
	int temp_player = 0;
	int temp_n_men_store = 0;
	int temp_start_epoch = 0;
	int temp_suspended[n_players_c];
	bool temp_completed[max_islands_per_epoch_c];
	if( loadGameInfo(&temp_difficulty, &temp_player, &temp_n_men_store, temp_suspended, &temp_start_epoch, temp_completed, slot) ) {
		difficulty_level = temp_difficulty;
		human_player = temp_player;
		n_men_store = temp_n_men_store;
		for(int i=0;i<n_players_c;i++)
			n_suspended[i] = temp_suspended[i];
		for(int i=0;i<max_islands_per_epoch_c;i++)
			completed_island[i] = temp_completed[i];
		setEpoch(temp_start_epoch);
		return true;
	}
	return false;
}

void saveGame(int slot) {
	LOG("saveGame(%d)\n",slot);
	ASSERT( gameType == GAMETYPE_ALLISLANDS );
	ASSERT( gameStateID == GAMESTATEID_PLACEMEN );
	ASSERT( slot >= 0 && slot < n_slots_c );

	char *filename = getFilename(slot);
	FILE *file = fopen(filename, "wb+");
	delete [] filename;
	if( file == NULL ) {
		LOG("FAILED to open file\n");
		return;
	}

	const int savVersion = 1;
	fprintf(file, "GLMSAV%d.%d.%d\n", majorVersion, minorVersion, savVersion);

	char buffer[256] = "";
	char *ptr = buffer;

	*((int *)ptr) = difficulty_level;
	ptr += sizeof(int);

	*((int *)ptr) = human_player;
	ptr += sizeof(int);

	*((int *)ptr) = n_men_store;
	ptr += sizeof(int);

	for(int i=0;i<n_players_c;i++) {
		*((int *)ptr) = n_suspended[i];
		ptr += sizeof(int);
	}

	*((int *)ptr) = start_epoch;
	ptr += sizeof(int);

	for(int i=0;i<max_islands_per_epoch_c;i++) {
		int val = completed_island[i] ? 1 : 0;
		*((int *)ptr) = val;
		ptr += sizeof(int);
	}

	ptrdiff_t n_bytes = ptr - buffer;
	int sum = 0;
	for(int i=0;i<n_bytes;i++) {
		sum += buffer[i];
	}

	LOG("checksum %d\n", sum);

	*((int *)ptr) = sum;
	ptr += sizeof(int);

	*ptr = '\0';

	//fprintf(file, "%s\n", buffer);
	//fwrite(buffer, (int)ptr - (int)buffer, 1, file);
	ptrdiff_t diff = ptr - buffer;
	fwrite(buffer, diff, 1, file);

	fclose(file);
}

bool validPlayer(int player) {
	bool valid = player >= 0 && player < n_players_c;
	if( !valid ) {
		LOG("ERROR invalid player %d\n", player);
	}
	return valid;
}

void stopMusic() {
	/*if( module != NULL ) {
		FMUSIC_FreeSong(module);
		module = NULL;
	}
	if( str_music != NULL ) {
		FSOUND_Stream_Close(str_music);
		str_music = NULL;
	}*/
	if( music != NULL ) {
		delete music;
		music = NULL;
	}
}

void playMusic() {
	stopMusic();
	/*if( !play_music )
	return;*/

	if( gameStateID == GAMESTATEID_CHOOSEPLAYER ) {
		/*module = FMUSIC_LoadSong("data/mod.warintro");
		if( module == NULL ) {
			LOG("Failed to load module\n");
			return;
		}
		playModule(module);*/
		music = loadMusic("data/mod.warintro");
		if( music == NULL ) {
			LOG("Failed to load music\n");
		}
		else {
			music->play(SOUND_CHANNEL_MUSIC);
		}
	}
	else if( gameStateID == GAMESTATEID_PLACEMEN ) {
		/*module = FMUSIC_LoadSong("data/mod.music");
		if( module == NULL ) {
			LOG("Failed to load module\n");
			return;
		}
		playModule(module);*/
		music = loadMusic("data/mod.music");
		if( music == NULL ) {
			LOG("Failed to load music\n");
		}
		else {
			music->play(SOUND_CHANNEL_MUSIC);
		}
	}
	else if( play_music && gameStateID == GAMESTATEID_PLAYING ) {
		//FSOUND_SAMPLE *fsound_sample = FSOUND_Sample_Load(FSOUND_FREE, "data/desready", FSOUND_LOADRAW, 0, 0);
		//str_music = FSOUND_Stream_OpenFile("gamemusic.ogg", FSOUND_LOOP_NORMAL, 0);
		/*str_music = FSOUND_Stream_Open("gamemusic.ogg", FSOUND_LOOP_NORMAL, 0, 0);
		//str_music = FSOUND_Stream_Open("gamemusic.ogg", FSOUND_LOOP_OFF, 0, 0);
		//str_music = FSOUND_Stream_Open("gamemusic.ogg", 0, 0, 0);
		if( str_music == NULL ) {
			LOG("Failed to load streamed music\n");
			return;
		}
		FSOUND_Stream_SetLoopCount(str_music, -1); // loop forever
		playStream(str_music);*/
		music = loadMusic("gamemusic.ogg");
		if( music == NULL ) {
			LOG("Failed to load music\n");
		}
		else {
			music->play(SOUND_CHANNEL_MUSIC);
		}
	}
}

bool loadSamples() {
	if( use_amigadata )
	{
		s_design_is_ready = loadSample("data/desready", true);
		s_design_is_ready->setText("the design is completed");
		s_ergo = loadSample("data/ergo", true);
		s_ergo->setText("ergonomically terrific");
		s_fcompleted = loadSamplesChained("data/fthe", "data/-prodrun2", NULL);
		s_fcompleted->setText("the production run is completed");
		s_advanced_tech = loadSample("data/teklev2", true);
		s_advanced_tech->setText("we have advanced a tech level");
		s_on_hold = loadSample("data/hold", true);
		//s_on_hold->setText("putting you on hold");
		s_running_out_of_elements = loadSample("data/nelem", true);
		s_running_out_of_elements->setText("running out of elements");
		s_tower_critical = loadSample("data/da-tower", true);
		s_tower_critical->setText("the tower is almost destroyed");
		s_sector_destroyed = loadSample("data/arsect", true);
		s_sector_destroyed->setText("the sector has been destroyed");
		s_mine_destroyed = loadSample("data/ndest", true);
		s_mine_destroyed->setText("the mine is destroyed");
		s_factory_destroyed = loadSamplesChained("data/fthe", "data/ffact", "data/fdest");
		s_factory_destroyed->setText("the factory is destroyed");
		s_lab_destroyed = loadSample("data/labdest", true);
		s_lab_destroyed->setText("the lab is destroyed");
		s_itis_all_over = loadSample("data/da-allover", true);
		s_itis_all_over->setText("game over");
		s_conquered = loadSamplesChained("data/arweve", "data/arconq", NULL);
		s_conquered->setText("we have conquered the sector");
		s_won = loadSamplesChained("data/arweve", "data/arwon", NULL);
		s_won->setText("we have won");
		s_weve_nuked_them = loadSamplesChained("data/arweve", "data/arnuke", NULL);
		s_weve_nuked_them->setText("we nuked them");
		s_weve_been_nuked = loadSample("data/da-nuked", true);
		s_weve_been_nuked->setText("we have been nuked");
		s_alliance_yes[0] = loadSamplesChained("data/dwhy", "data/dall", NULL);
		s_alliance_yes[0]->setText("red team says okay");
		s_alliance_yes[1] = loadSample("data/asisi", true);
		s_alliance_yes[1]->setText("green team says okay");
		s_alliance_yes[2] = loadSample("data/PVERYWELL", true);
		s_alliance_yes[2]->setText("yellow team says okay");
		s_alliance_yes[3] = loadSample("data/yougotit", true);
		s_alliance_yes[3]->setText("blue team says okay");
		s_alliance_no[0] = loadSamplesChained("data/dha", "data/NO", NULL);
		s_alliance_no[0]->setText("red team says no");
		s_alliance_no[1] = loadSample("data/anowork", true);
		s_alliance_no[1]->setText("green team says no");
		s_alliance_no[2] = loadSample("data/pno", true);
		s_alliance_no[2]->setText("yellow team says no");
		s_alliance_no[3] = loadSample("data/noway", true);
		s_alliance_no[3]->setText("blue team says no");
		s_alliance_ask[0] = loadSample("data/djoin", true);
		s_alliance_ask[0]->setText("red team requests alliance");
		s_alliance_ask[1] = loadSample("data/awanna", true);
		s_alliance_ask[1]->setText("green team requests alliance");
		s_alliance_ask[2] = loadSample("data/pmyteam", true);
		s_alliance_ask[2]->setText("yellow team requests alliance");
		s_alliance_ask[3] = loadSample("data/myside2", true);
		s_alliance_ask[3]->setText("blue team requests alliance");
		s_quit[0] = loadSamplesChained("data/dha", "data/dpath", NULL);
		s_quit[1] = loadSample("data/alaff", true);
		s_quit[2] = loadSample("data/pmmm2", true);
		s_quit[3] = loadSample("data/laff", true);
	}
	else
	{
		/*s_design_is_ready = loadSample("Mega Lo Mania/desready");
		s_ergo = loadSample("Mega Lo Mania/ergo");
		s_fcompleted = loadSamplesChained("Mega Lo Mania/fthe", "Mega Lo Mania/prodrun2", NULL);
		s_advanced_tech = loadSample("Mega Lo Mania/teklev2");
		s_on_hold = loadSample("Mega Lo Mania/hold");
		s_running_out_of_elements = loadSample("Mega Lo Mania/nelem");
		s_tower_critical = loadSample("Mega Lo Mania/da-tower");
		s_sector_destroyed = loadSample("Mega Lo Mania/arsect");
		s_mine_destroyed = loadSample("Mega Lo Mania/ndest");
		s_factory_destroyed = loadSamplesChained("Mega Lo Mania/fthe", "Mega Lo Mania/ffact", "Mega Lo Mania/fdest");
		s_lab_destroyed = loadSample("Mega Lo Mania/labdest");
		s_itis_all_over = loadSample("Mega Lo Mania/da_over");
		s_conquered = loadSamplesChained("Mega Lo Mania/arweve", "Mega Lo Mania/arconq", NULL);
		s_won = loadSamplesChained("Mega Lo Mania/arweve", "Mega Lo Mania/arwon", NULL);
		s_weve_nuked_them = loadSamplesChained("Mega Lo Mania/arweve", "Mega Lo Mania/arnuke", NULL);
		s_weve_been_nuked = loadSample("Mega Lo Mania/da-nuked");
		s_alliance_yes[0] = loadSamplesChained("Mega Lo Mania/dwhy", "Mega Lo Mania/dall", NULL);
		s_alliance_yes[1] = loadSample("Mega Lo Mania/asisi");
		s_alliance_yes[2] = loadSample("Mega Lo Mania/PVERYW");
		s_alliance_yes[3] = loadSample("Mega Lo Mania/yougotit");
		s_alliance_no[0] = loadSamplesChained("Mega Lo Mania/dha", "Mega Lo Mania/NO", NULL);
		s_alliance_no[1] = loadSample("Mega Lo Mania/anowork");
		s_alliance_no[2] = loadSample("Mega Lo Mania/pno");
		s_alliance_no[3] = loadSample("Mega Lo Mania/noway");*/
	}

	// text messages without corresponding samples
	s_cant_nuke_ally = new Sample(NULL);
	s_cant_nuke_ally->setText("cannot nuke our ally");

	// sound effects
	s_explosion = loadSample("sound/explosion_6.wav");
	s_scream = loadSample("sound/plagasul_jah.wav");
	s_buildingdestroyed = loadSample("sound/woodbrk.wav");
	s_guiclick = loadSample("sound/pulsed808bd.wav");

	bool ok = !errorSound();
	return ok;
}

bool remapLand(Image *image,MapColour colour) {
	for(int y=0;y<image->getHeight();y++) {
		for(int x=0;x<image->getWidth();x++) {
			unsigned char c = image->getPixelIndex(x,y);
			if( c == 0 ) {
				// leave as 0
			}
			else if( c == 1 ) {
				// light
				int c2 = 0;
				if( colour == MAP_ORANGE )
					c2 = 15;
				else if( colour == MAP_GREEN )
					c2 = 1;
				else if( colour == MAP_BROWN )
					c2 = 14;
				else if( colour == MAP_WHITE )
					c2 = 2;
				else if( colour == MAP_DBROWN )
					c2 = 6;
				else if( colour == MAP_DGREEN )
					c2 = 12;
				else if( colour == MAP_GREY )
					c2 = 1;
				else {
					ASSERT(false);
				}
				image->setPixelIndex(x, y, c2);
			}
			else if( c == 2 ) {
				// medium
				int c2 = 0;
				if( colour == MAP_ORANGE )
					c2 = 14;
				else if( colour == MAP_GREEN )
					c2 = 12;
				else if( colour == MAP_BROWN )
					c2 = 6;
				else if( colour == MAP_WHITE )
					c2 = 1;
				else if( colour == MAP_DBROWN )
					c2 = 8;
				else if( colour == MAP_DGREEN )
					c2 = 7;
				else if( colour == MAP_GREY )
					c2 = 5;
				else {
					ASSERT(false);
				}
				image->setPixelIndex(x, y, c2);
			}
			else if( c == 3 ) {
				// dark
				int c2 = 0;
				if( colour == MAP_ORANGE )
					c2 = 6;
				else if( colour == MAP_GREEN )
					c2 = 7;
				else if( colour == MAP_BROWN )
					c2 = 8;
				else if( colour == MAP_WHITE )
					c2 = 5;
				else if( colour == MAP_DBROWN )
					c2 = 3;
				else if( colour == MAP_DGREEN )
					c2 = 8;
				else if( colour == MAP_GREY )
					c2 = 4;
				else {
					ASSERT(false);
				}
				image->setPixelIndex(x, y, c2);
			}
			else {
				LOG("land slabs should only have colour indices 0-3\n");
				return false;
			}
		}
	}
	return true;
}

void createAlphaShadows(Image *image,int sx, int sy) {
	// hack to mark some shadows when using the original data files
	//Uint32 amask = image->getSDLSurface()->format->Amask;
	//if( image->getScaleX() != 2 || image->getScaleY() != 2 ) {
	/*if( !image->isPaletted() ) {
	return;
	}*/
	image->convertToHiColor(true);
	SDL_Surface *surface = image->getSDLSurface();
	SDL_LockSurface(surface);
	int w = image->getWidth();
	int h = image->getHeight();
	int bytespp = surface->format->BytesPerPixel;
	SDL_Color maskcol = image->getMaskColor();
	Uint32 mask_index = SDL_MapRGB(surface->format, maskcol.r, maskcol.g, maskcol.b);
	Uint32 shadow_index = SDL_MapRGB(surface->format, 0, 0, 0);
	//Uint32 amask2 = surface->format->Amask;
	for(int x=0;x<w;x++) {
		for(int y=0;y<h;y++) {
			unsigned char *ptr = &((unsigned char *)surface->pixels)[ surface->pitch * y + bytespp * x ];
			Uint32 pixel = *((Uint32 *)ptr);
			if( pixel == mask_index )
				ptr[3] = 0; // mask color
			else if( pixel == shadow_index && ( x >= sx || y >= sy ) )
				ptr[3] = shadow_alpha_c; // shadow
			else
				ptr[3] = 255; // opaque
		}
	}
	SDL_UnlockSurface(surface);
	SDL_SetAlpha(surface, SDL_SRCALPHA, 255);
	/*
	int amask2 = surface->format->Amask;
	image->convertToDisplayFormat();*/
}

void createAlphaShadows(int epoch,Building::Type type,Image *image) {
	ASSERT_ANY_EPOCH(epoch);
	int sx = 100, sy = 100;
	if( type == Building::BUILDING_TOWER ) {
		if( epoch == 0 ) {
			sx = 46;
			sy = 41;
		}
		else if( epoch == 1 ) {
			sx = 42;
			sy = 51;
		}
		else if( epoch == 2 ) {
			sx = 46;
			sy = 53;
		}
		else if( epoch == 3 ) {
			sx = 46;
			sy = 50;
		}
		else if( epoch == 4 ) {
			sx = 45;
			sy = 51;
		}
	}
	else if( type == Building::BUILDING_MINE ) {
		if( epoch == 3 ) {
			sx = 45;
			sy = 44;
		}
	}
	/*sx *= scale_width;
	sy *= scale_height;*/
	/*sx *= image->getScaleX();
	sy *= image->getScaleY();*/
	sx = (int)(sx * image->getScaleX());
	sy = (int)(sy * image->getScaleY());
	createAlphaShadows(image, sx, sy);
}

bool original_background = false;

bool loadImages() {
	bool using_new_graphics = true;
	player_select = Image::loadImage("gfx/player_select.jpg");
	if( player_select == NULL ) {
		using_new_graphics = false;
		player_select = Image::loadImage(use_amigadata ? "data/mlm_select" : "Mega Lo Mania/");
	}
	if( using_new_graphics ) {
		LOG("using new graphics\n");
	}
	else {
		LOG("using old graphics\n");
	}
	if( player_select == NULL )
		return false;
	/*player_select->scale(scale_width,scale_height);
	player_select->setScale(2, 2);*/
	float scale_factor = ((float)(scale_width*default_width_c))/(float)player_select->getWidth();
	LOG("scale factor for images = %f\n", scale_factor);
	// nb, still scale if scale_factor==1, as this is a way of converting to 8bit
	/*if( !player_select->scaleTo(scale_width*default_width_c) )
	return false;*/
	player_select->scale(scale_factor, scale_factor);
	player_select->setScale(scale_width, scale_height);

	if( using_new_graphics )
		background = Image::loadImage("gfx/starfield.jpg");
	else {
		background = Image::loadImage("data/mlm_starfield");
		original_background = true;
	}
	if( background == NULL )
		return false;
	/*background->scale(scale_width,scale_height);
	background->setScale(2, 2);*/
	/*if( !background->scaleTo(scale_width*default_width_c) )
	return false;*/
	background->scale(scale_factor, scale_factor);
	background->setScale(scale_width, scale_height);

	Image *image_slabs = NULL;
	if( using_new_graphics )
		image_slabs = Image::loadImage("gfx/slabs.png");
	else {
		image_slabs = Image::loadImage("data/mlm_slabs");
		//copy_palette = true;
	}
	if( image_slabs == NULL )
		return false;
	for(int i=0;i<MAP_N_COLOURS;i++) {
		//bool copy_palette = false;
		/*land[i] = Image::loadImage("gfx/slabs.png");
		if( land[i] == NULL ) {
			land[i] = Image::loadImage("data/mlm_slabs");
			//copy_palette = true;
		}
		if( land[i] == NULL )
			return false;*/
		land[i] = image_slabs->copy();
		/*land[i]->scale(scale_width,scale_height);
		land[i]->setScale(2, 2);*/
		/*if( !land[i]->scaleTo(scale_width*default_width_c) )
		return false;*/
		land[i]->scale(scale_factor, scale_factor); // also forces to 8bit (since should be paletted)
		//land[i]->scale(1, 1); // hacky way of forcing to 8bit
		land[i]->setScale(scale_width, scale_height);
		/*if( copy_palette ) {
		//land[i]->convertToFormat(background);
		if( !land[i]->copyPalette(background) ) {
		LOG("failed to copy palette for land\n");
		return false;
		}
		}*/
		if( !land[i]->isPaletted() ) {
			LOG("land slabs should be paletted\n");
			return false;
		}
		else if( land[i]->getNColors() < 16 ) { // should be 256 by now anyway, due to scaling
			LOG("land slabs should have at least 16 colours\n");
			return false;
		}
		else {
			// 1, 2, 3, 4, 5, 6, 7, 8, 12, 14, 15
			land[i]->setColor(1, 176, 176, 176);
			land[i]->setColor(2, 240, 240, 240);
			land[i]->setColor(3, 0, 0, 0);
			land[i]->setColor(4, 64, 64, 64);
			land[i]->setColor(5, 112, 112, 112);
			land[i]->setColor(6, 160, 64, 0);
			land[i]->setColor(7, 0, 96, 0);
			land[i]->setColor(8, 112, 36, 16);
			land[i]->setColor(12, 0, 192, 0);
			land[i]->setColor(14, 240, 112, 16);
			land[i]->setColor(15, 240, 240, 0);
		}
		if( !remapLand(land[i], (MapColour)i) ) {
			LOG("failed to remap land\n");
			return false;
		}
	}
	delete image_slabs;
	image_slabs = NULL;

	bool buildings_shadow = false; // shadow hack

	/*Image *buildings = Image::loadImage("gfx/buildings.png");
	if( buildings != NULL ) {
		buildings_shadow = false; // done using alpha channel
		buildings->scale(scale_factor, scale_factor);
		buildings->setScale(scale_width, scale_height);
		for(int i=0;i<n_epochs_c;i++) {
			fortress[i] = buildings->copy( 0, 60*i, 58, 60);
			mine[i] = buildings->copy( 58, 60*i, 58, 60);
			factory[i] = buildings->copy( 174, 60*i, 58, 60);
			lab[i] = buildings->copy( 116, 60*i, 58, 60);
		}
	}*/
	for(int i=0;i<n_epochs_c;i++) {
		fortress[i] = NULL;
		mine[i] = NULL;
		factory[i] = NULL;
	}
	if( using_new_graphics ) {
		for(int i=0;i<n_epochs_c;i++) {
			stringstream filename;
			filename << "gfx/building_tower_" << i << ".png";
			Image *temp = Image::loadImage(filename.str().c_str());
			if( temp == NULL ) {
				return false;
			}
			temp->scale(scale_factor, scale_factor);
			temp->setScale(scale_width, scale_height);
			//delete fortress[i];
			//fortress[i] = temp->copy(29, 9, 62, 51);
			fortress[i] = temp->copy(27, 9, 64, 51);
			delete temp;
		}
		for(int i=mine_epoch_c;i<n_epochs_c-1;i++) {
			stringstream filename;
			filename << "gfx/building_mine_" << i << ".png";
			Image *temp = Image::loadImage(filename.str().c_str());
			if( temp == NULL ) {
				return false;
			}
			temp->scale(scale_factor, scale_factor);
			temp->setScale(scale_width, scale_height);
			mine[i] = temp->copy(28, 12, 66, 51);
			delete temp;
		}
		for(int i=factory_epoch_c;i<n_epochs_c-1;i++) {
			stringstream filename;
			filename << "gfx/building_factory_" << i << ".png";
			Image *temp = Image::loadImage(filename.str().c_str());
			if( temp == NULL ) {
				return false;
			}
			temp->scale(scale_factor, scale_factor);
			temp->setScale(scale_width, scale_height);
			//factory[i] = temp->copy(24, 1, 70, 62);
			factory[i] = temp->copy(25, 1, 68, 62);
			delete temp;
		}
		for(int i=lab_epoch_c;i<n_epochs_c-1;i++) {
			stringstream filename;
			filename << "gfx/building_lab_" << i << ".png";
			Image *temp = Image::loadImage(filename.str().c_str());
			if( temp == NULL ) {
				return false;
			}
			temp->scale(scale_factor, scale_factor);
			temp->setScale(scale_width, scale_height);
			//lab[i] = temp->copy(28, 12, 66, 51);
			lab[i] = temp->copy(31, 12, 52, 51);
			delete temp;
		}

		buildings_shadow = false; // done using alpha channel
		offset_flag_y_c = 3; // correct placement of flag on tower for new graphics
	}
	else {
		Image *buildings = Image::loadImage("data/mlm_buildings");
		if( buildings == NULL )
			return false;
		buildings_shadow = buildings->isPaletted() && buildings->getNColors() == 16;
		/*buildings->scale(scale_width,scale_height);
		buildings->setScale(2, 2);*/
		/*if( !buildings->scaleTo(scale_width*default_width_c) )
		return false;*/
		buildings->scale(scale_factor, scale_factor);
		buildings->setScale(scale_width, scale_height);

		for(int i=0;i<n_epochs_c;i++) {
			fortress[i] = buildings->copy( 0, 60*i, 58, 60);
			mine[i] = buildings->copy( 64, 60*i, 58, 60);
			factory[i] = buildings->copy( 192, 60*i, 58, 60);
			lab[i] = buildings->copy( 128, 60*i, 58, 60);
		}
		delete buildings;
	}

	if( using_new_graphics ) {
		Image *icons = Image::loadImage("gfx/icons.png");
		if( icons == NULL )
			return false;
		/*if( !icons->scaleTo(scale_width*default_width_c) )
		return false;*/
		icons->scale(scale_factor, scale_factor);
		icons->setScale(scale_width, scale_height);

		for(int i=0;i<n_epochs_c;i++)
			men[i] = icons->copy(16*i, 0, 16, 16);

		unarmed_man = icons->copy(80, 32, 16, 16);

		panel_design = icons->copy(304, 0, 16, 16);
		//panel_design_dark = panel_design->copy();
		panel_lab = icons->copy(16, 33, 16, 16);
		panel_shield = icons->copy(240, 0, 16, 16);
		panel_defence = icons->copy(256, 0, 16, 16);
		panel_attack = icons->copy(272, 0, 16, 16);
		panel_knowndesigns = icons->copy(240, 16, 16, 16);
		panel_factory = icons->copy(48, 33, 16, 16);
		panel_bloody_attack = icons->copy(240, 32, 16, 16);

		mine_gatherable_small = icons->copy(160, 0, 16, 16);

		//panel_build[Building::BUILDING_TOWER] = icons->copy(224, 63, 16, 16); // different size // not yet used
		panel_build[Building::BUILDING_MINE] = icons->copy(256, 63, 16, 16); // different size
		panel_build[Building::BUILDING_FACTORY] = icons->copy(288, 63, 16, 16); // different size
		panel_build[Building::BUILDING_LAB] = icons->copy(192, 63, 16, 16); // different size

		panel_building[Building::BUILDING_TOWER] = icons->copy(0, 33, 16, 16); // different size
		panel_building[Building::BUILDING_MINE] = icons->copy(32, 33, 16, 16); // different size
		panel_building[Building::BUILDING_FACTORY] = panel_factory;
		panel_building[Building::BUILDING_LAB] = panel_lab;

		mine_gatherable_large = mine_gatherable_small;
		panel_bigdesign = panel_design;
		panel_biglab = panel_lab;
		panel_bigfactory = panel_factory;
		panel_bigshield = panel_shield;
		panel_bigdefence = panel_defence;
		panel_bigattack = panel_attack;
		panel_bigbuild = icons->copy(32, 48, 32, 16); // different size
		panel_bigknowndesigns = panel_knowndesigns;
		panel_twoattack = panel_attack;

		for(int i=0;i<n_players_c;i++)
			mouse_pointers[i] = icons->copy(176 + 16*i, 0, 16, 16);

		for(int i=0;i<13;i++)
			icon_clocks[i] = icons->copy(16*i, 16, 16, 16);

		for(int i=0;i<3;i++)
			icon_mice[i] = icons->copy(256 + 16*i, 16, 16, 16); // smaller size

		for(int i=0;i<n_death_flashes_c;i++)
			death_flashes[i] = icons->copy(256 + 16*i, 32, 16, 16); // different location and smaller size

		for(int i=0;i<7;i++)
			blue_flashes[i] = icons->copy(208 + 16*i, 144, 16, 16); // different location (and size for i = 5, 6)

		for(int i=0;i<n_players_c;i++) {
			icon_towers[i] = icons->copy(160 + 16*i, 81, 6, 6);
			icon_armies[i] = icons->copy(160 + 16*i, 87, 4, 4);
		}

		for(int i=0;i<Element::N_ID;i++)
			icon_elements[i] = icons->copy(16*i, 128, 16, 16); // different location

		for(int i=0;i<n_players_c;i++) { // different locations
			for(int j=0;j<3;j++)
				flags[i][j] = icons->copy(160 + 16*j, 144, 16, 16);
			flags[i][3] = flags[i][1];
			for(int j=0;j<n_flag_frames_c;j++) {
				int r = 0, g = 0, b = 0, col = 0;
				PlayerType::getColour(&r, &g, &b, (PlayerType::PlayerTypeID)i);
				flags[i][j]->remap(240, 0, 0, r, g, b);
			}
		}

		for(int i=0;i<n_playershields_c;i++) {
			//playershields[i] = icons->copy(16*i, 176, 16, 14); // different location // original version for my gfx
			playershields[i] = icons->copy(16*i, 176, 16, 16); // different location
			//playershields[i] = icons->copy(16*i, 177, 16, 14); // different location
		}

		for(int i=0;i<n_shields_c;i++)
			icon_shields[i] = icons->copy(112 + 16*i, 32, 16, 16);
		for(int i=0;i<n_epochs_c;i++)
			icon_defences[i] = icons->copy(16*i, 240, 16, 16); // different location
		for(int i=0;i<n_epochs_c;i++)
			icon_weapons[i] = icons->copy(16*i, 224, 16, 16); // different location
		for(int i=0;i<n_epochs_c;i++)
			//numbered_defences[i] = icons->copy(16*i, 173, 16, 16);
			numbered_defences[i] = icon_defences[i]; // todo:
		for(int i=0;i<n_epochs_c;i++)
			//numbered_weapons[i] = icons->copy(16*i, 157, 16, 16);
			numbered_weapons[i] = icon_weapons[i]; // todo:

		for(int i=0;i<3;i++)
			icon_speeds[i] = icons->copy(272 + 16*i, 192, 16, 16); // different location and size

		//building_health = icons->copy(0, 192, 41, 5); // original version for my gfx
		building_health = icons->copy(0, 192, 48, 16);
		dash_grey = icons->copy(144, 112, 5, 2); // original version for my gfx
		dash_grey = icons->copy(144, 112, 16, 16);

		icon_shield = icons->copy(0, 64, 16, 16);
		icon_defence = icons->copy(16, 64, 16, 16);
		icon_weapon = icons->copy(32, 64, 16, 16);

		/*icon_infinity = icons->copy(128, 112, 12, 8); // original versions for my gfx
		icon_bc = icons->copy(224, 15, 12, 8);
		icon_ad = icons->copy(208, 16, 12, 8);
		icon_ad_shiny = icons->copy(208, 16, 12, 8);*/
		icon_infinity = icons->copy(128, 112, 16, 16);
		icon_bc = icons->copy(224, 15, 16, 16);
		icon_ad = icons->copy(208, 16, 16, 16);
		icon_ad_shiny = icons->copy(208, 16, 16, 16);

		icon_nuke_hole = icons->copy(288, 96, 16, 16);

		icon_ergo = icons->copy(176, 112, 16, 16);
		icon_trash = icons->copy(192, 112, 16, 16);

		icons = Image::loadImage("gfx/icons64.png");
		if( icons == NULL )
			return false;
		// replace with new large icons
		/*if( !icons->scaleTo(scale_width*128) ) // may need to update width as more icons added!
		return false;*/
		icons->scale(scale_factor, scale_factor);
		icons->setScale(scale_width, scale_height);

		mapsquare = icons->copy(0, 0, 17, 17);
		flashingmapsquare = icons->copy(32, 0, 17, 17);

		icons = Image::loadImage("gfx/font.png");
		if( icons == NULL )
			return false;
		icons->scale(scale_factor, scale_factor);
		icons->setScale(scale_width, scale_height);
		for(int i=0;i<10;i++) {
			numbers_blue[i] = icons->copy(8*i, 0, 6, 8);
			numbers_grey[i] = icons->copy(8*i, 0, 6, 8);
			numbers_white[i] = icons->copy(8*i, 0, 6, 8);
			numbers_orange[i] = icons->copy(8*i, 0, 6, 8);
			numbers_yellow[i] = icons->copy(8*i, 0, 6, 8);
			numbers_largegrey[i] = icons->copy(8*i, 0, 6, 8); // should be 6x15
			numbers_largeshiny[i] = icons->copy(8*i, 0, 6, 8); // should be 6x15
			for(int j=0;j<4;j++) {
				numbers_small[j][i] = icons->copy(8*i, 32, 5, 7);
				// todo: remap to dark player colours too
				/*int r=0, g=0, b=0;
				PlayerType::getColour(&r, &g, &b, (PlayerType::PlayerTypeID)j);
				numbers_small[j][i]->remap(240, 0, 0, r, g, b);
				numbers_small[j][i]->remap(112, 32, 16, r, g, b);*/
			}
			numbers_small[1][i]->reshadeRGB(0, false, true, false);
			numbers_small[2][i]->reshadeRGB(0, true, true, false);
			numbers_small[3][i]->reshadeRGB(0, false, false, true);
		}
		for(int i=0;i<26;i++) {
			letters_small[i] = icons->copy(80 + 8*i, 16, 6, 8);
			letters_large[i] = icons->copy(80 + 8*i, 16, 6, 8); // should be 6x15
		}
		numbers_half = icons->copy(288, 16, 6, 8);

		delete icons;
	}
	else {
		Image *icons = Image::loadImage("data/mlm_icons");
		if( icons == NULL )
			return false;
		if( !icons->scaleTo((int)(scale_width*default_width_c)) ) // must use this method, as still using alongside the new gfx images
			return false;
		//icons->scale(scale_factor, scale_factor);
		icons->setScale(scale_width, scale_height);

		for(int i=0;i<n_epochs_c;i++)
			men[i] = icons->copy(16*i, 0, 16, 16);

		unarmed_man = icons->copy(80, 32, 16, 16);

		panel_design = icons->copy(304, 0, 16, 16);
		//panel_design_dark = panel_design->copy();
		panel_lab = icons->copy(16, 33, 16, 16);
		panel_shield = icons->copy(240, 0, 16, 16);
		panel_defence = icons->copy(256, 0, 16, 16);
		panel_attack = icons->copy(272, 0, 16, 16);
		panel_knowndesigns = icons->copy(240, 16, 16, 16);
		panel_factory = icons->copy(48, 33, 16, 16);
		panel_bloody_attack = icons->copy(240, 32, 16, 16);

		mine_gatherable_small = icons->copy(160, 0, 16, 16);

		//panel_build[Building::BUILDING_TOWER] = icons->copy(224, 63, 19, 16); // not yet used
		panel_build[Building::BUILDING_MINE] = icons->copy(256, 63, 19, 16);
		panel_build[Building::BUILDING_FACTORY] = icons->copy(288, 63, 19, 16);
		panel_build[Building::BUILDING_LAB] = icons->copy(192, 63, 19, 16);

		panel_building[Building::BUILDING_TOWER] = icons->copy(0, 33, 16, 14);
		panel_building[Building::BUILDING_MINE] = icons->copy(32, 33, 16, 14);
		//panel_building[Building::BUILDING_FACTORY] = icons->copy(48, 33, 16, 14);
		//panel_building[Building::BUILDING_LAB] = icons->copy(16, 33, 16, 14);
		panel_building[Building::BUILDING_FACTORY] = panel_factory;
		panel_building[Building::BUILDING_LAB] = panel_lab;

		mine_gatherable_large = icons->copy(160, 64, 32, 16);

		panel_bigdesign = icons->copy(256, 48, 32, 15);
		panel_biglab = icons->copy(96, 48, 32, 16);
		panel_bigfactory = icons->copy(128, 48, 32, 16);
		panel_bigshield = icons->copy(160, 48, 32, 15);
		panel_bigdefence = icons->copy(192, 48, 32, 15);
		panel_bigattack = icons->copy(224, 48, 32, 15);
		panel_bigbuild = icons->copy(32, 49, 32, 15);
		panel_bigknowndesigns = icons->copy(288, 48, 32, 15);
		panel_twoattack = icons->copy(64, 33, 15, 15);

		for(int i=0;i<10;i++)
			numbers_blue[i] = icons->copy(16*i, 64, 6, 8);
		for(int i=0;i<10;i++)
			numbers_grey[i] = icons->copy(16*i, 72, 6, 8);
		for(int i=0;i<10;i++)
			numbers_white[i] = icons->copy(16*i, 80, 6, 8);
		for(int i=0;i<10;i++)
			numbers_orange[i] = icons->copy(16*i, 88, 6, 8);
		for(int i=0;i<10;i++)
			numbers_yellow[i] = icons->copy(16*i, 119, 6, 8);
		for(int i=0;i<3;i++)
			numbers_largeshiny[i] = icons->copy(16*i, 269, 6, 15);
		for(int i=3;i<10;i++)
			numbers_largeshiny[i] = NULL; // not used
		for(int i=0;i<10;i++)
			numbers_largegrey[i] = icons->copy(64 + 16*i, 269, 6, 15);
		for(int i=0;i<10;i++)
			numbers_small[0][i] = icons->copy(16*i, 126, 5, 7);
		for(int i=0;i<10;i++)
			numbers_small[1][i] = icons->copy(16*i, 134, 5, 7);
		for(int i=0;i<10;i++)
			numbers_small[2][i] = icons->copy(160 + 16*i, 126, 5, 7);
		for(int i=0;i<10;i++)
			numbers_small[3][i] = icons->copy(160 + 16*i, 134, 5, 7);
		numbers_half = icons->copy(160, 119, 6, 8);

		for(int i=0;i<4;i++)
			letters_large[i] = icons->copy(256 + 16*i, 269, 6, 15);
		for(int i=0;i<20;i++)
			letters_large[4+i] = icons->copy(16*i, 284, 6, 15);
		for(int i=0;i<2;i++)
			letters_large[24+i] = icons->copy(16*i, 299, 6, 15);

		for(int i=0;i<15;i++)
			letters_small[i] = icons->copy(80 + 16*i, 299, 6, 8);
		for(int i=0;i<11;i++)
			letters_small[15+i] = icons->copy(80 + 16*i, 307, 6, 8);

		for(int i=0;i<n_players_c;i++)
			mouse_pointers[i] = icons->copy(176 + 16*i, 0, 16, 16);

		for(int i=0;i<13;i++)
			icon_clocks[i] = icons->copy(16*i, 16, 16, 16);

		for(int i=0;i<3;i++)
			icon_mice[i] = icons->copy(256 + 16*i, 16, 16, 19);

		for(int i=0;i<n_death_flashes_c;i++)
			death_flashes[i] = icons->copy(272 + 16*i, 346, 16, 19);

		for(int i=0;i<5;i++)
			blue_flashes[i] = icons->copy(225 + 16*i, 173, 16, 16);
		blue_flashes[5] = icons->copy(305, 165, 15, 19);
		blue_flashes[6] = icons->copy(305, 184, 15, 17);

		for(int i=0;i<n_players_c;i++) {
			icon_towers[i] = icons->copy(160 + 16*i, 81, 6, 6);
			icon_armies[i] = icons->copy(160 + 16*i, 87, 4, 4);
		}

		for(int i=0;i<Element::N_ID;i++)
			icon_elements[i] = icons->copy(16*i, 141, 16, 16);

		for(int i=0;i<n_players_c;i++)
			flags[i][0] = icons->copy(160 + 16*i, 157, 16, 15);
		for(int i=0;i<n_players_c;i++)
			flags[i][1] = icons->copy(224 + 16*i, 157, 16, 15);
		for(int i=0;i<n_players_c;i++)
			flags[i][2] = icons->copy(160 + 16*i, 172, 16, 15);
		for(int i=0;i<n_players_c;i++)
			flags[i][3] = flags[i][1];

		for(int i=0;i<n_playershields_c;i++)
			playershields[i] = icons->copy(16*i, 189, 16, 14);

		for(int i=0;i<n_shields_c;i++)
			icon_shields[i] = icons->copy(112 + 16*i, 32, 16, 16);
		for(int i=0;i<n_epochs_c;i++)
			icon_defences[i] = icons->copy(16*i, 253, 16, 16);
		for(int i=0;i<n_epochs_c;i++)
			icon_weapons[i] = icons->copy(16*i, 237, 16, 16);
		for(int i=0;i<n_epochs_c;i++)
			numbered_defences[i] = icons->copy(16*i, 173, 16, 16);
		for(int i=0;i<n_epochs_c;i++)
			numbered_weapons[i] = icons->copy(16*i, 157, 16, 16);

		for(int i=0;i<3;i++)
			icon_speeds[i] = icons->copy(272 + 16*i, 204, 16, 18);

		building_health = icons->copy(0, 203, 41, 5);
		dash_grey = icons->copy(144, 114, 5, 2);
		icon_shield = icons->copy(272, 315, 16, 16);
		icon_defence = icons->copy(288, 315, 16, 16);
		icon_weapon = icons->copy(304, 315, 16, 16);

		icon_infinity = icons->copy(112, 104, 12, 8);

		icon_bc = icons->copy(240, 276, 12, 8);
		icon_ad = icons->copy(224, 276, 12, 8);
		icon_ad_shiny = icons->copy(48, 276, 12, 8);

		icon_nuke_hole = icons->copy(288, 99, 9, 9);

		icon_ergo = icons->copy(176, 111, 16, 16);
		icon_trash = icons->copy(192, 111, 16, 16);

		mapsquare = icons->copy(288, 81, 17, 17);
		flashingmapsquare = icons->copy(192, 92, 17, 17);

		delete icons;
	}

	for(int i=0;i<n_coast_c;i++)
		coast_icons[i] = NULL;
	if( using_new_graphics ) {
		Image *smallmap = Image::loadImage("gfx/smallmap.png");
		if( smallmap == NULL )
			return false;
		/*if( !smallmap->scaleTo(scale_width*308) )
		return false;*/
		smallmap->scale(scale_factor, scale_factor);
		smallmap->setScale(scale_width, scale_height);
		for(int i=0;i<MAP_N_COLOURS;i++) {
			for(int j=0;j<n_map_sq_c;j++) {
				map_sq[i][j] = smallmap->copy(22*i, 0, 22, 22);
			}
		}
		Image *smallmap_coast = Image::loadImage("gfx/smallmap_coast.png");
		if( smallmap_coast != NULL ) {
			/*if( !smallmap_coast->scaleTo(scale_width*616) )
			return false;*/
			smallmap_coast->scale(scale_factor, scale_factor);
			smallmap_coast->setScale(scale_width, scale_height);
			for(int i=0;i<n_coast_c;i++)
				coast_icons[i] = smallmap_coast->copy(22*i, 0, 22, 22);
		}
		delete smallmap;
		delete smallmap_coast;
	}
	else {
		Image *smallmap = Image::loadImage("data/mlm_smallmap");
		if( smallmap == NULL )
			return false;
		/*smallmap->scale(scale_width,scale_height);
		smallmap->setScale(2, 2);*/
		/*if( !smallmap->scaleTo(scale_width*default_width_c) )
		return false;*/
		smallmap->scale(scale_factor, scale_factor);
		smallmap->setScale(scale_width, scale_height);

		for(int i=0;i<MAP_N_COLOURS;i++) {
			for(int j=0;j<n_map_sq_c;j++)
				map_sq[i][j] = NULL;
		}

		map_sq[MAP_ORANGE][14] = smallmap->copy(0, 0, 22, 22);
		map_sq[MAP_ORANGE][7] = smallmap->copy(32, 0, 22, 22);
		map_sq[MAP_ORANGE][11] = smallmap->copy(64, 0, 22, 22);
		map_sq[MAP_ORANGE][4] = smallmap->copy(96, 0, 22, 22);
		map_sq[MAP_ORANGE][5] = smallmap->copy(128, 0, 22, 22);
		map_sq[MAP_ORANGE][1] = smallmap->copy(160, 0, 22, 22);
		map_sq[MAP_ORANGE][2] = smallmap->copy(192, 0, 22, 22);
		map_sq[MAP_ORANGE][8] = smallmap->copy(224, 0, 22, 22);

		map_sq[MAP_GREEN][6] = smallmap->copy(256, 0, 22, 22);
		map_sq[MAP_GREEN][12] = smallmap->copy(288, 0, 22, 22);
		map_sq[MAP_GREEN][3] = smallmap->copy(0, 22, 22, 22);
		map_sq[MAP_GREEN][9] = smallmap->copy(32, 22, 22, 22);
		map_sq[MAP_GREEN][5] = smallmap->copy(64, 22, 22, 22);
		map_sq[MAP_GREEN][10] = smallmap->copy(96, 22, 22, 22);

		map_sq[MAP_BROWN][6] = smallmap->copy(128, 22, 22, 22);
		map_sq[MAP_BROWN][4] = smallmap->copy(160, 22, 22, 22);
		map_sq[MAP_BROWN][1] = smallmap->copy(192, 22, 22, 22);
		map_sq[MAP_BROWN][8] = smallmap->copy(224, 22, 22, 22);
		map_sq[MAP_BROWN][0] = smallmap->copy(256, 22, 22, 22);

		map_sq[MAP_DBROWN][6] = smallmap->copy(32, 66, 22, 22);
		map_sq[MAP_DBROWN][12] = smallmap->copy(64, 66, 22, 22);
		map_sq[MAP_DBROWN][7] = smallmap->copy(96, 66, 22, 22);
		map_sq[MAP_DBROWN][13] = smallmap->copy(128, 66, 22, 22);
		map_sq[MAP_DBROWN][3] = smallmap->copy(160, 66, 22, 22);
		map_sq[MAP_DBROWN][11] = smallmap->copy(192, 66, 22, 22);
		map_sq[MAP_DBROWN][9] = smallmap->copy(224, 66, 22, 22);
		map_sq[MAP_DBROWN][4] = smallmap->copy(256, 66, 22, 22);
		map_sq[MAP_DBROWN][5] = smallmap->copy(288, 66, 22, 22);
		map_sq[MAP_DBROWN][1] = smallmap->copy(0, 88, 22, 22);
		map_sq[MAP_DBROWN][2] = smallmap->copy(32, 88, 22, 22);
		map_sq[MAP_DBROWN][10] = smallmap->copy(64, 88, 22, 22);

		map_sq[MAP_WHITE][1] = smallmap->copy(224, 44, 22, 22);
		map_sq[MAP_WHITE][2] = smallmap->copy(256, 44, 22, 22);
		map_sq[MAP_WHITE][3] = smallmap->copy(96, 44, 22, 22);
		map_sq[MAP_WHITE][4] = smallmap->copy(192, 44, 22, 22);
		map_sq[MAP_WHITE][6] = smallmap->copy(288, 22, 22, 22);
		map_sq[MAP_WHITE][8] = smallmap->copy(0, 66, 22, 22);
		map_sq[MAP_WHITE][9] = smallmap->copy(160, 44, 22, 22);
		map_sq[MAP_WHITE][10] = smallmap->copy(288, 44, 22, 22);
		map_sq[MAP_WHITE][11] = smallmap->copy(128, 44, 22, 22);
		map_sq[MAP_WHITE][12] = smallmap->copy(32, 44, 22, 22);
		map_sq[MAP_WHITE][14] = smallmap->copy(0, 44, 22, 22);
		map_sq[MAP_WHITE][15] = smallmap->copy(64, 44, 22, 22);

		map_sq[MAP_DGREEN][6] = smallmap->copy(96, 88, 22, 22);
		map_sq[MAP_DGREEN][14] = smallmap->copy(128, 88, 22, 22);
		map_sq[MAP_DGREEN][12] = smallmap->copy(160, 88, 22, 22);
		map_sq[MAP_DGREEN][7] = smallmap->copy(192, 88, 22, 22);
		map_sq[MAP_DGREEN][15] = smallmap->copy(224, 88, 22, 22);
		map_sq[MAP_DGREEN][13] = smallmap->copy(256, 88, 22, 22);
		map_sq[MAP_DGREEN][3] = smallmap->copy(288, 88, 22, 22);
		map_sq[MAP_DGREEN][11] = smallmap->copy(0, 110, 22, 22);
		map_sq[MAP_DGREEN][9] = smallmap->copy(32, 110, 22, 22);

		map_sq[MAP_GREY][0] = smallmap->copy(0, 132, 22, 22);
		map_sq[MAP_GREY][1] = smallmap->copy(192, 110, 22, 22);
		map_sq[MAP_GREY][2] = smallmap->copy(224, 110, 22, 22);
		map_sq[MAP_GREY][4] = smallmap->copy(128, 110, 22, 22);
		map_sq[MAP_GREY][5] = smallmap->copy(160, 110, 22, 22);
		map_sq[MAP_GREY][8] = smallmap->copy(288, 110, 22, 22);
		map_sq[MAP_GREY][10] = smallmap->copy(256, 110, 22, 22);
		map_sq[MAP_GREY][12] = smallmap->copy(64, 110, 22, 22);
		map_sq[MAP_GREY][15] = smallmap->copy(96, 110, 22, 22);

		delete smallmap;
	}

	bool attackers_shadow = false;
	{
		for(int i=0;i<=n_epochs_c;i++) {
			for(int j=0;j<n_attacker_frames_c;j++) {
				for(int k=0;k<n_players_c;k++)
					attackers_walking[k][i][j] = NULL;
			}
		}
		for(int i=0;i<n_epochs_c;i++)
			for(int j=0;j<N_ATTACKER_AMMO_DIRS;j++)
				attackers_ammo[i][j] = NULL;

		if( using_new_graphics ) {
			Image *gfx_def_image = Image::loadImage("gfx/defenders.png");
			if( gfx_def_image == NULL )
				return false;
			/*if( !gfx_def_image->scaleTo(scale_width*default_width_c) )
			return false;*/
			gfx_def_image->scale(scale_factor, scale_factor);
			gfx_def_image->setScale(scale_width, scale_height);
			for(int k=0;k<n_players_c;k++) {
				for(int i=0;i<n_epochs_c;i++) {
					for(int j=0;j<n_defender_frames_c;j++) {
						defenders[k][i][j] = gfx_def_image->copy(16*i, 0, 16, 16);
					}
				}
			}

			for(int i=0;i<=5;i++) {
				char filename[300] = "";
				sprintf(filename, "gfx/attacker_walking_%d.png", i);
				Image *gfx_image = Image::loadImage(filename);
				if( gfx_image == NULL )
					return false;
				gfx_image->setScale(scale_width, scale_height);
				/*if( !gfx_image->scaleTo(scale_width*default_width_c) )
				return false;*/
				gfx_image->scale(scale_factor, scale_factor);
				for(int j=0;j<n_attacker_frames_c;j++) {
					for(int k=0;k<n_players_c;k++) {
						if( j % 4 > 2 )
							attackers_walking[k][i][j] = gfx_image->copy(0, 0, 16, 16); // todo
						else
							attackers_walking[k][i][j] = gfx_image->copy(16*(j % 4), 0, 16, 16);
					}
				}
				delete gfx_image;
			}

			//Image *gfx_image = Image::loadImage("gfx/attacker_walking_base.png");
			//Image *gfx_image = Image::loadImage("gfx/attacker_walking_1.png");
			Image *gfx_image = Image::loadImage("gfx/attacker_walking_10.png");
			if( gfx_image == NULL )
				return false;
			/*if( !gfx_image->scaleTo(scale_width*default_width_c) )
			return false;*/
			gfx_image->scale(scale_factor, scale_factor);
			gfx_image->setScale(scale_width, scale_height);
			for(int j=0;j<n_attacker_frames_c;j++) {
				for(int k=0;k<n_players_c;k++) {
					if( j % 4 > 2 )
						attackers_walking[k][10][j] = gfx_image->copy(0, 0, 16, 16); // todo
					/*else if( j / 4 == ATTACKER_AMMO_LEFT ) {
					attackers_walking[k][10][j] = attackers_walking[k][10][ATTACKER_AMMO_RIGHT*4+(j%4)]->copy(0, 0, 16, 16);
					attackers_walking[k][10][j]->flipX();
					}*/
					else
						attackers_walking[k][10][j] = gfx_image->copy(16*(j % 4), 0, 16, 16);
				}
			}

			Image *gfx_planes = Image::loadImage("gfx/attacker_flying.png");
			if( gfx_planes == NULL )
				return false;
			/*if( !gfx_planes->scaleTo(scale_width*default_width_c) )
			return false;*/
			gfx_planes->scale(scale_factor, scale_factor);
			gfx_planes->setScale(scale_width, scale_height);
			for(int i=0;i<n_players_c;i++) {
				for(int j=0;j<n_epochs_c;j++)
					planes[i][j] = NULL;
				planes[i][6] = gfx_planes->copy(32*i, 0, 32, 32);
				planes[i][7] = gfx_planes->copy(128+32*i, 0, 32, 32);
			}

			for(int i=0;i<n_players_c;i++) {
				nukes[i][0] = gfx_planes->copy(64*i, 32, 32, 32);
				nukes[i][1] = gfx_planes->copy(64*i+32, 32, 32, 32);
			}
			for(int i=0;i<n_players_c;i++) {
				for(int j=0;j<n_saucer_frames_c;j++) {
					saucers[i][j] = gfx_planes->copy(32*j, 64, 32, 32);
				}
			}

			Image *gfx_ammo = Image::loadImage("gfx/attacker_ammo.png");
			if( gfx_ammo == NULL )
				return false;
			attackers_shadow = false;
			/*if( !gfx_ammo->scaleTo(scale_width*default_width_c) )
			return false;*/
			gfx_ammo->scale(scale_factor, scale_factor);
			gfx_ammo->setScale(scale_width, scale_height);
			for(int i=0;i<6;i++) {
				attackers_ammo[i][ATTACKER_AMMO_RIGHT] = gfx_ammo->copy(0, 16*i, 16, 16);
				attackers_ammo[i][ATTACKER_AMMO_LEFT] = gfx_ammo->copy(16, 16*i, 16, 16);
				attackers_ammo[i][ATTACKER_AMMO_UP] = gfx_ammo->copy(32, 16*i, 16, 16);
				attackers_ammo[i][ATTACKER_AMMO_DOWN] = gfx_ammo->copy(48, 16*i, 16, 16);
			}
			// bombs
			//attackers_ammo[6][ATTACKER_AMMO_BOMB] = armies->copy(304, 208, 16, 16);
			attackers_ammo[6][ATTACKER_AMMO_BOMB] = gfx_ammo->copy(0, 96, 16, 16); // different size

			delete gfx_def_image;
			delete gfx_image;
			delete gfx_planes;
			delete gfx_ammo;
		}
		else {
			Image *armies = Image::loadImage("data/mlm_armies");
			if( armies == NULL ) {
				return false;
			}
			attackers_shadow = armies->isPaletted() && armies->getNColors() == 16;
			/*armies->scale(scale_width,scale_height);
			armies->setScale(2, 2);*/
			/*if( !armies->scaleTo(scale_width*default_width_c) )
			return false;*/
			armies->scale(scale_factor, scale_factor);
			armies->setScale(scale_width, scale_height);

			for(int i=0;i<=5;i++) {
				for(int j=0;j<n_defender_frames_c;j++) {
					for(int k=0;k<n_players_c;k++)
						defenders[k][i][j] = armies->copy(16*j, 16 + 32*i, 16, 16);
				}
			}
			for(int j=0;j<n_defender_frames_c;j++) {
				for(int k=0;k<n_players_c;k++)
					defenders[k][6][j] = armies->copy(128 + 16*j, 192, 16, 16);
			}
			for(int j=0;j<2;j++) {
				for(int k=0;k<n_players_c;k++)
					defenders[k][7][j] = armies->copy(224 + 16*j, 240, 16, 16);
			}
			for(int j=2;j<4;j++) {
				for(int k=0;k<n_players_c;k++)
					defenders[k][7][j] = armies->copy(288 + 16*(j-2), 224, 16, 16);
			}
			for(int j=4;j<6;j++) {
				for(int k=0;k<n_players_c;k++)
					defenders[k][7][j] = armies->copy(256 + 16*(j-4), 240, 16, 16);
			}
			for(int j=6;j<8;j++) {
				for(int k=0;k<n_players_c;k++)
					defenders[k][7][j] = armies->copy(288 + 16*(j-6), 240, 16, 16);
			}
			for(int j=0;j<n_defender_frames_c;j++) {
				defenders[0][8][j] = armies->copy(192, 256, 16, 16);
				defenders[1][8][j] = armies->copy(192, 272, 16, 16);
				defenders[2][8][j] = armies->copy(208, 256, 16, 16);
				defenders[3][8][j] = armies->copy(208, 272, 16, 16);
			}
			for(int k=0;k<n_players_c;k++) {
				int kx = k / 2;
				int ky = k % 2;
				for(int j=0;j<n_defender_frames_c;j++) {
					int j2 = j % 4;
					if( j2 == 0 )
						j2 = 1;
					else if( j2 == 1 )
						j2 = 0;
					defenders[k][9][j] = armies->copy(192 + kx * 64 + j2 * 16, 288 + ky * 13, 16, 13);
				}
			}


			for(int i=0;i<=5;i++) {
				for(int j=0;j<n_attacker_frames_c;j++) {
					for(int k=0;k<n_players_c;k++) {
						attackers_walking[k][i][j] = armies->copy(16*j, 32*i, 16, 16);
					}
				}
			}

			for(int j=0;j<n_attacker_frames_c;j++) {
				for(int k=0;k<n_players_c;k++) {
					attackers_walking[k][10][j] = armies->copy(16*j, 320, 16, 16);
				}
			}

			for(int i=0;i<n_players_c;i++) {
				for(int j=0;j<n_epochs_c;j++)
					planes[i][j] = NULL;
				planes[i][6] = armies->copy(32*i, 192, 32, 32);
				planes[i][7] = armies->copy(32*i, 232, 32, 24);
			}

			for(int i=0;i<n_players_c;i++) {
				nukes[i][0] = armies->copy(48*i, 256, 16, 32);
				nukes[i][1] = armies->copy(48*i+16, 256, 32, 32);
			}
			for(int i=0;i<n_players_c;i++) {
				for(int j=0;j<n_saucer_frames_c;j++) {
					saucers[i][j] = armies->copy(32*j, 288, 32, 21);
				}
			}

			// ammo:
			// rock
			attackers_ammo[0][ATTACKER_AMMO_RIGHT] = armies->copy(272, 24, 16, 8);
			attackers_ammo[0][ATTACKER_AMMO_LEFT] = armies->copy(288, 24, 16, 8);
			attackers_ammo[0][ATTACKER_AMMO_UP] = armies->copy(288, 16, 16, 8);
			attackers_ammo[0][ATTACKER_AMMO_DOWN] = armies->copy(272, 16, 16, 8);
			// catapult/sword
			attackers_ammo[1][ATTACKER_AMMO_RIGHT] = armies->copy(272, 24, 16, 8);
			attackers_ammo[1][ATTACKER_AMMO_LEFT] = armies->copy(288, 24, 16, 8);
			attackers_ammo[1][ATTACKER_AMMO_UP] = armies->copy(288, 16, 16, 8);
			attackers_ammo[1][ATTACKER_AMMO_DOWN] = armies->copy(272, 16, 16, 8);
			/* spear
			attackers_ammo[1][ATTACKER_AMMO_RIGHT] = armies->copy(272, 32, 16, 8);
			attackers_ammo[1][ATTACKER_AMMO_LEFT] = armies->copy(272, 40, 16, 8);
			attackers_ammo[1][ATTACKER_AMMO_UP] = armies->copy(288, 32, 16, 16);
			attackers_ammo[1][ATTACKER_AMMO_DOWN] = armies->copy(304, 32, 16, 16);*/
			// pike
			attackers_ammo[2][ATTACKER_AMMO_RIGHT] = armies->copy(272, 64, 16, 8);
			attackers_ammo[2][ATTACKER_AMMO_LEFT] = armies->copy(272, 72, 16, 8);
			attackers_ammo[2][ATTACKER_AMMO_UP] = armies->copy(288, 64, 16, 16);
			attackers_ammo[2][ATTACKER_AMMO_DOWN] = armies->copy(304, 64, 16, 16);
			/* bow and arrow
			attackers_ammo[2][ATTACKER_AMMO_RIGHT] = armies->copy(272, 80, 16, 8);
			attackers_ammo[2][ATTACKER_AMMO_LEFT] = armies->copy(272, 88, 16, 8);
			attackers_ammo[2][ATTACKER_AMMO_UP] = armies->copy(288, 80, 16, 16);
			attackers_ammo[2][ATTACKER_AMMO_DOWN] = armies->copy(304, 80, 16, 16);*/
			// longbow
			attackers_ammo[3][ATTACKER_AMMO_RIGHT] = armies->copy(272, 96, 16, 8);
			attackers_ammo[3][ATTACKER_AMMO_LEFT] = armies->copy(272, 104, 16, 8);
			attackers_ammo[3][ATTACKER_AMMO_UP] = armies->copy(288, 96, 16, 16);
			attackers_ammo[3][ATTACKER_AMMO_DOWN] = armies->copy(304, 96, 16, 16);
			// trebuchet
			attackers_ammo[4][ATTACKER_AMMO_RIGHT] = armies->copy(256, 144, 16, 8);
			attackers_ammo[4][ATTACKER_AMMO_LEFT] = armies->copy(256, 144, 16, 8);
			attackers_ammo[4][ATTACKER_AMMO_UP] = armies->copy(256, 144, 16, 16);
			attackers_ammo[4][ATTACKER_AMMO_DOWN] = armies->copy(256, 144, 16, 16);
			// cannon
			attackers_ammo[5][ATTACKER_AMMO_RIGHT] = armies->copy(272, 160, 10, 9);
			attackers_ammo[5][ATTACKER_AMMO_LEFT] = armies->copy(272, 160, 10, 9);
			attackers_ammo[5][ATTACKER_AMMO_UP] = armies->copy(272, 160, 10, 9);
			attackers_ammo[5][ATTACKER_AMMO_DOWN] = armies->copy(272, 160, 10, 9);
			// bombs
			//attackers_ammo[6][ATTACKER_AMMO_BOMB] = armies->copy(304, 208, 16, 16);
			attackers_ammo[6][ATTACKER_AMMO_BOMB] = armies->copy(288, 206, 12, 12);

			delete armies;
		}

		/*nuke_defences[0] = armies->copy(192, 256, 16, 16);
		nuke_defences[1] = armies->copy(208, 256, 16, 16);
		nuke_defences[2] = armies->copy(192, 272, 16, 16);
		nuke_defences[3] = armies->copy(208, 272, 16, 16);*/
		for(int k=0;k<n_players_c;k++) {
			nuke_defences[k] = defenders[k][nuclear_epoch_c][0];
		}

		attackers_ammo[7][ATTACKER_AMMO_BOMB] = attackers_ammo[6][ATTACKER_AMMO_BOMB];
		attackers_ammo[9][ATTACKER_AMMO_BOMB] = attackers_ammo[6][ATTACKER_AMMO_BOMB];

	}

	// features
	if( using_new_graphics ) {
		Image *gfx_features = Image::loadImage("gfx/features.png");
		if( gfx_features == NULL )
			return false;
		/*if( !gfx_features->scaleTo(scale_width*default_width_c) )
		return false;*/
		gfx_features->scale(scale_factor, scale_factor);
		gfx_features->setScale(scale_width, scale_height);
		icon_openpitmine = gfx_features->copy(0, 0, 47, 24);

		Image *gfx_trees = Image::loadImage("gfx/trees.png");
		if( gfx_trees == NULL )
			return false;
		/*if( !gfx_trees->scaleTo(scale_width*default_width_c) )
		return false;*/
		gfx_trees->scale(scale_factor, scale_factor);
		gfx_trees->setScale(scale_width, scale_height);
		for(int i=0;i<4;i++) {
			icon_trees[i] = gfx_trees->copy(24*i, 0, 24, 28);
		}

		delete gfx_features;
		delete gfx_trees;
	}
	else {
		Image *features = Image::loadImage("data/mlm_features");
		if( features == NULL )
			return false;
		/*features->scale(scale_width,scale_height);
		features->setScale(2, 2);*/
		/*if( !features->scaleTo(scale_width*default_width_c) )
		return false;*/
		features->scale(scale_factor, scale_factor);
		features->setScale(scale_width, scale_height);

		icon_openpitmine = features->copy(256, 118, 47, 24);

		for(int i=0;i<4;i++) {
			icon_trees[i] = features->copy(96 + 32*i, 114, 24, 28);
		}

		delete features;
	}

	if( using_new_graphics )
		background_islands = Image::loadImage("gfx/sunrise.jpg");
	else
		background_islands = Image::loadImage("data/mlm_sunrise");
	if( background_islands == NULL )
		return false;
	/*background_islands->scale(scale_width,scale_height);
	background_islands->setScale(2, 2);*/
	/*if( !background_islands->scaleTo(scale_width*default_width_c) )
	return false;*/
	background_islands->scale(scale_factor, scale_factor);
	background_islands->setScale(scale_width, scale_height);

	for(int i=0;i<TrackedObject::getNumTags();i++) {
		TrackedObject *to = TrackedObject::getTag(i);
		if( to != NULL && strcmp( to->getClass(), "CLASS_IMAGE" ) == 0 ) {
			Image *image = (Image *)to;
			if( image != player_select && image != background && image != background_islands ) {
				image->setColor(0, 255, 0, 255);
				image->setMaskColor(255, 0, 255);
			}
		}
	}

	// finished loading/extracting images

	for(int i=0;i<n_epochs_c && attackers_shadow;i++) {
		for(int j=0;j<N_ATTACKER_AMMO_DIRS;j++) {
			Image *image = attackers_ammo[i][j];
			if( image != NULL ) {
				createAlphaShadows(image, 0, 0);
			}
		}
	}
	for(int i=0;i<n_epochs_c && buildings_shadow;i++) {
		createAlphaShadows(i, Building::BUILDING_TOWER, fortress[i]);
		createAlphaShadows(i, Building::BUILDING_MINE, mine[i]);
	}
	for(int i=0;i<TrackedObject::getNumTags();i++) {
		TrackedObject *to = TrackedObject::getTag(i);
		if( to != NULL && strcmp( to->getClass(), "CLASS_IMAGE" ) == 0 ) {
			Image *image = (Image *)to;
			image->createAlphaForColor(127, 0, 127, shadow_alpha_c);
			image->convertToHiColor(false);
		}
	}
	//panel_design_dark->brighten(0.4, 0.4, 0.4, true); // must be done after converting to 24 bit
	for(int i=0;i<MAP_N_COLOURS;i++) {
		for(int j=0;j<n_map_sq_c;j++) {
			if( map_sq[i][j] != NULL )
				map_sq[i][j]->smooth(true);
		}
		land[i]->smooth(true);
	}

	for(int i=0;i<=n_epochs_c;i++) {
		if( i == 6 || i == 7 || i == 8 || i == 9 )
			continue;
		for(int j=0;j<n_attacker_frames_c;j++) {
			for(int k=0;k<4;k++) {
				int r = 0, g = 0, b = 0, col = 0;
				PlayerType::getColour(&r, &g, &b, (PlayerType::PlayerTypeID)k);
				attackers_walking[k][i][j]->remap(240, 0, 0, r, g, b);
			}
			/*attackers_walking[1][i][j]->remap(240, 0, 0, 0, 192, 0);
			attackers_walking[2][i][j]->remap(240, 0, 0, 240, 240, 0);
			attackers_walking[3][i][j]->remap(240, 0, 0, 64, 64, 240);*/
		}
	}
	/*for(int i=6;i<=7;i++) {
	for(int k=0;k<4;k++) {
	int r = 0, g = 0, b = 0, col = 0;
	PlayerType::getColour(&r, &g, &b, (PlayerType::PlayerTypeID)k);
	planes[k][i]->remap(240, 0, 0, r, g, b);
	}
	}*/
	for(int i=0;i<n_saucer_frames_c;i++) {
		for(int k=0;k<4;k++) {
			int r = 0, g = 0, b = 0, col = 0;
			PlayerType::getColour(&r, &g, &b, (PlayerType::PlayerTypeID)k);
			saucers[k][i]->remap(240, 0, 0, r, g, b);
		}
	}
	for(int i=0;i<n_epochs_c;i++) {
		if( defenders[0][i][0] == NULL )
			continue;
		for(int j=0;j<n_defender_frames_c;j++) {
			for(int k=0;k<4;k++) {
				int r = 0, g = 0, b = 0, col = 0;
				PlayerType::getColour(&r, &g, &b, (PlayerType::PlayerTypeID)k);
				defenders[k][i][j]->remap(240, 0, 0, r, g, b);
			}
			/*defenders[1][i][j]->remap(240, 0, 0, 0, 192, 0);
			defenders[2][i][j]->remap(240, 0, 0, 240, 240, 0);
			defenders[3][i][j]->remap(240, 0, 0, 64, 64, 240);*/
		}
	}

	return true;
}

bool setupInventions() {
	/*for(int i=0;i<n_shields_c;i++)
	invention_shields[i] = new Invention("SHIELD", Invention::SHIELD, start_epoch + i);*/
	for(int i=0;i<n_epochs_c;i++)
		invention_shields[i] = new Invention("SHIELD", Invention::SHIELD, i);

	invention_defences[0] = new Invention("STICK", Invention::DEFENCE, 0);
	invention_defences[1] = new Invention("SPEAR", Invention::DEFENCE, 1);
	invention_defences[2] = new Invention("SHORTBOW", Invention::DEFENCE, 2);
	invention_defences[3] = new Invention("CAULDRON OF OIL", Invention::DEFENCE, 3);
	invention_defences[4] = new Invention("CROSSBOW", Invention::DEFENCE, 4);
	invention_defences[5] = new Invention("RIFLE", Invention::DEFENCE, 5);
	invention_defences[6] = new Invention("MACHINE GUN", Invention::DEFENCE, 6);
	invention_defences[7] = new Invention("ROCKET", Invention::DEFENCE, 7);
	invention_defences[8] = new Invention("NUCLEAR DEFENCE", Invention::DEFENCE, 8);
	invention_defences[9] = new Invention("SDI LASER", Invention::DEFENCE, 9);

	invention_weapons[0] = new Weapon("ROCK WEAPON", 0, 1);
	//invention_weapons[1] = new Weapon("CATAPULT", 1, 1);
	invention_weapons[1] = new Weapon("SWORD", 1, 1);
	invention_weapons[2] = new Weapon("PIKE", 2, 1);
	invention_weapons[3] = new Weapon("LONGBOW", 3, 1);
	invention_weapons[4] = new Weapon("TREBUCHET", 4, 2);
	invention_weapons[5] = new Weapon("CANNON", 5, 3);
	invention_weapons[6] = new Weapon("BIPLANE", 6, 2);
	invention_weapons[7] = new Weapon("BOMBER", 7, 3);
	invention_weapons[8] = new Weapon("NUCLEAR MISSILE", 8, 0);
	invention_weapons[9] = new Weapon("SPACESHIP", 9, 10);

	return true;
}

bool setupElements() {
	elements[Element::WOOD] = new Element("WOOD", Element::WOOD, Element::GATHERABLE);
	elements[Element::ROCK] = new Element("ROCK", Element::ROCK, Element::GATHERABLE);
	elements[Element::BONE] = new Element("BONE", Element::BONE, Element::GATHERABLE);
	elements[Element::SLATE] = new Element("SLATE", Element::SLATE, Element::GATHERABLE);
	elements[Element::MOONLITE] = new Element("MOONLITE", Element::MOONLITE, Element::OPENPITMINE);
	elements[Element::PLANETARIUM] = new Element("PLANETARIUM", Element::PLANETARIUM, Element::OPENPITMINE);
	elements[Element::BETHLIUM] = new Element("BETHLIUM", Element::BETHLIUM, Element::OPENPITMINE);
	elements[Element::SOLARIUM] = new Element("SOLARIUM", Element::SOLARIUM, Element::OPENPITMINE);
	elements[Element::ARULDITE] = new Element("ARULDITE", Element::ARULDITE, Element::DEEPMINE);
	elements[Element::HERBIRITE] = new Element("HERBIRITE", Element::HERBIRITE, Element::DEEPMINE);
	elements[Element::YERIDIUM] = new Element("YERIDIUM", Element::YERIDIUM, Element::DEEPMINE);
	elements[Element::VALIUM] = new Element("VALIUM", Element::VALIUM, Element::DEEPMINE);
	elements[Element::PARASITE] = new Element("PARASITE", Element::PARASITE, Element::DEEPMINE);
	elements[Element::AQUARIUM] = new Element("AQUARIUM", Element::AQUARIUM, Element::DEEPMINE);
	elements[Element::PALADIUM] = new Element("PALADIUM", Element::PALADIUM, Element::DEEPMINE);
	elements[Element::ONION] = new Element("ONION", Element::ONION, Element::DEEPMINE);
	elements[Element::TEDIUM] = new Element("TEDIUM", Element::TEDIUM, Element::DEEPMINE);
	elements[Element::MORON] = new Element("MORON", Element::MORON, Element::DEEPMINE);
	elements[Element::MARMITE] = new Element("MAAMITE", Element::MARMITE, Element::DEEPMINE);
	elements[Element::ALIEN] = new Element("ALIEN", Element::ALIEN, Element::DEEPMINE);
	return true;
}

void cleanupPlayers() {
	for(int i=0;i<n_players_c;i++) {
		if( players[i] )
			delete players[i];
		players[i] = NULL;
	}

	// need to reset alliance flags!
	Player::resetAllAlliances();
}

void setupPlayers() {
	cleanupPlayers();

	int seed = clock();
	/*static int count = 0;
	count++;
	seed = 331 + count;*/
	//seed = 1000;
	LOG("setupPlayers(): set random seed to %d\n", seed);
	srand( seed );

	/*for(int i=0;i<n_players_c;i++)
	players[i] = NULL;*/
	// Personalities:
	//   0 - Red team
	//   1 - Green team
	//   2 - Yellow team
	//   3 - Blue team

	//human_player = 0;
	//human_player = 2;
	//enemy_player = 3;

	int n_opponents = maps[start_epoch][selected_island]->getNOpponents();
	ASSERT( n_opponents+1 <= maps[start_epoch][selected_island]->getNSquares() );
	int n_free = 4;
	if( human_player != PLAYER_DEMO ) {
		players[human_player] = new Player(human_player);
		n_free--;
	}
	else {
		n_opponents++;
	}
	for(int i=0;i<n_opponents && n_free > 0;i++) {
		int indx = rand() % n_free;
		for(int j=0;j<4;j++) {
			if( players[j] == NULL ) {
				if( indx == 0 ) {
					players[j] = new Player(j);
					n_free--;
					break;
				}
				indx--;
			}
		}
	}

	/*players[0] = new Player(0);
	players[1] = new Player(1);
	players[2] = new Player(2);
	players[3] = new Player(3);*/

}

bool openScreen(bool fullscreen) {
	if( !fullscreen ) {
		int user_width = 0, user_height = 0;
#ifdef _WIN32
//#if 0
		// we do it using system calls instead of SDL, to ignore the start bar (if showing)
		RECT rect;
		SystemParametersInfo(SPI_GETWORKAREA, 0, &rect, 0);
		user_width = rect.right - rect.left;
		user_height = rect.bottom - rect.top;
		LOG("desktop is %d x %d\n", user_width, user_height);
		user_height -= GetSystemMetrics(SM_CYCAPTION); // also ignore the window height
		LOG("available height is %d\n", user_height);
#else
		const SDL_VideoInfo *videoInfo = SDL_GetVideoInfo();
		user_width = videoInfo->current_w;
		user_height = videoInfo->current_h;
		LOG("desktop is %d x %d\n", user_width, user_height);
#endif

		if( user_width >= 4*default_width_c && user_height >= 4*default_height_c ) {
			scale_width = scale_height = 4.0f;
			LOG("scale 4x\n");
		}
		else if( user_width >= 2*default_width_c && user_height >= 2*default_height_c ) {
			scale_width = scale_height = 2.0f;
			LOG("scale 2x\n");
		}
		else if( user_width >= default_width_c && user_height >= default_height_c ) {
			scale_width = scale_height = 1.0f;
			LOG("scale 1x\n");
		}
		else {
			LOG("desktop resolution too small even for 1x scale\n");
			return false;
		}

		/*
#ifdef _WIN32
		//int user_width = GetSystemMetrics(SM_CXMAXIMIZED);
		//int user_height = GetSystemMetrics(SM_CYSCREEN);
		//int user_height = GetSystemMetrics(SM_CYMAXIMIZED);
		//LOG("desktop is %d x %d\n", user_width, user_height);
		//user_height -= GetSystemMetrics(SM_CYCAPTION);
		//LOG("available height is %d\n", user_height);
		RECT rect;
		SystemParametersInfo(SPI_GETWORKAREA, 0, &rect, 0);
		int user_width = rect.right - rect.left;
		int user_height = rect.bottom - rect.top;
		LOG("desktop is %d x %d\n", user_width, user_height);
		user_height -= GetSystemMetrics(SM_CYCAPTION);
		LOG("available height is %d\n", user_height);
		if( user_width >= 4*default_width_c && user_height >= 4*default_height_c ) {
			scale_width = scale_height = 4.0f;
			LOG("scale 4x\n");
		}
		else if( user_width >= 2*default_width_c && user_height >= 2*default_height_c ) {
			scale_width = scale_height = 2.0f;
			LOG("scale 2x\n");
		}
		else if( user_width >= default_width_c && user_height >= default_height_c ) {
			scale_width = scale_height = 1.0f;
			LOG("scale 1x\n");
		}
		else {
			LOG("desktop resolution too small even for 1x scale\n");
			return false;
		}
#else
		//scale_width = scale_height = 2.0f;
		//LOG("scale 2x\n");
		const SDL_VideoInfo *videoInfo = SDL_GetVideoInfo();
		int user_width = videoInfo->current_w;
		int user_height = videoInfo->current_h;
		LOG("desktop is %d x %d\n", user_width, user_height);
#ifdef _WIN32
		user_height -= GetSystemMetrics(SM_CYCAPTION); // ignore start bar if not hidden
#endif
		LOG("available height is %d\n", user_height);
		if( user_width >= 4*default_width_c && user_height >= 4*default_height_c ) {
			scale_width = scale_height = 4.0f;
			LOG("scale 4x\n");
		}
		else if( user_width >= 2*default_width_c && user_height >= 2*default_height_c ) {
			scale_width = scale_height = 2.0f;
			LOG("scale 2x\n");
		}
		else if( user_width >= default_width_c && user_height >= default_height_c ) {
			scale_width = scale_height = 1.0f;
			LOG("scale 1x\n");
		}
		else {
			LOG("desktop resolution too small even for 1x scale\n");
			return false;
		}
#endif
		*/
		int screen_width = (int)(scale_width * default_width_c);
		int screen_height = (int)(scale_height * default_height_c);

		screen = new Screen();
		if( !screen->open(screen_width, screen_height, fullscreen) )
			return false;

	}
	else {
		/*if( Screen::canOpenFullscreen(4*default_width_c, 4*default_height_c) ) {
		scale_width = scale_height = 4.0f;
		LOG("scale 4x\n");
		}
		else if( Screen::canOpenFullscreen(2*default_width_c, 2*default_height_c) ) {
		scale_width = scale_height = 2.0f;
		LOG("scale 2x\n");
		}
		else if( Screen::canOpenFullscreen(4*default_width_c, 4*default_height_c) ) {
		scale_width = scale_height = 1.0f;
		LOG("scale 1x\n");
		}
		else {
		LOG("can't even open screen at 1x scale\n");
		return false;
		}*/
		screen = new Screen();
		if( screen->open(4*default_width_c, 4*default_height_c, fullscreen) ) {
			scale_width = scale_height = 4.0f;
			LOG("scale 4x\n");
		}
		else if( screen->open(2*default_width_c, 2*default_height_c, fullscreen) ) {
			scale_width = scale_height = 2.0f;
			LOG("scale 2x\n");
		}
		else if( screen->open(default_width_c, default_height_c, fullscreen) ) {
			scale_width = scale_height = 1.0f;
			LOG("scale 1x\n");
		}
		else {
			LOG("can't even open screen at 1x scale\n");
			return false;
		}
	}

	char buffer[256] = "";
	sprintf(buffer, "Gigalomania, version %d.%d - Loading...", majorVersion, minorVersion);
	screen->setTitle(buffer);
	return true;
}

char *maps_dirname = "islands";

bool readSectors(Map *map) {
	char fullname[4096] = "";
	sprintf(fullname, "%s/%s", maps_dirname, map->getFilename());
	FILE *file = fopen(fullname, "rb");
	if( file == NULL ) {
		// open in binary mode, so that we parse files in an OS-independent manner
		// (otherwise, Windows will parse "\r\n" as being "\n", but Linux will still read it as "\n")
		LOG("failed to open file: %s\n", fullname);
		perror("perror returns: ");
		return false;
	}

	bool ok = true;
	const int MAX_LINE = 4096;
	char line[MAX_LINE+1] = "";
	int sec_x = -1, sec_y = -1;
	bool done_header = false;
	while( ok ) {
		errno = 0;
		if( fgets(line, MAX_LINE, file) == NULL ) {
			if( errno ) {
				LOG("error reading line\n");
				ok = false;
			}
			break;
		}
		else {
			line[ strlen(line) - 1 ] = '\0'; // trim new line
			line[ strlen(line) - 1 ] = '\0'; // trim carriage return
			if( !done_header ) {
				if( line[0] != '#' ) {
					LOG("expected first character to be '#'\n");
					ok = false;
					break;
				}
				// ignore rest of header
				done_header = true;
			}
			else {
				char *line_ptr = line;
				while( *line_ptr == ' ' || *line_ptr == '\t' ) // avoid initial whitespace
					line_ptr++;
				char *comment = strchr(line_ptr, '#');
				if( comment != NULL ) { // trim comments
					*comment = '\0';
				}
				char *ptr = strtok(line_ptr, " ");
				if( ptr == NULL ) {
					LOG("can't find first word\n");
					ok = false;
					break;
				}
				else if( strcmp(ptr, "SECTOR") == 0 ) {
					ptr = strtok(NULL, " ");
					if( ptr == NULL ) {
						LOG("can't find sec_x\n");
						ok = false;
						break;
					}
					sec_x = atoi(ptr);
					if( sec_x < 0 || sec_x >= map_width_c ) {
						LOG("invalid map x %d\n", sec_x);
						ok = false;
						break;
					}

					ptr = strtok(NULL, " ");
					if( ptr == NULL ) {
						LOG("can't find sec_y\n");
						ok = false;
						break;
					}
					sec_y = atoi(ptr);
					if( sec_y < 0 || sec_y >= map_height_c ) {
						LOG("invalid map y %d\n", sec_y);
						ok = false;
						break;
					}
				}
				else if( strcmp(ptr, "ELEMENT") == 0 ) {
					if( sec_x == -1 || sec_y == -1 ) {
						LOG("sector not defined\n");
						ok = false;
						break;
					}
					ptr = strtok(NULL, " ");
					if( ptr == NULL ) {
						LOG("can't find element name\n");
						ok = false;
						break;
					}
					char elementname[MAX_LINE+1] = "";
					strcpy(elementname, ptr);

					ptr = strtok(NULL, " ");
					if( ptr == NULL ) {
						LOG("can't find n_elements\n");
						ok = false;
						break;
					}
					int n_elements = atoi(ptr);

					Element::Id element = Element::UNDEFINED;
					for(int i=0;i<Element::N_ID;i++) {
						if( strcmp( elements[i]->getName(), elementname ) == 0 ) {
							element = (Element::Id)i;
						}
					}
					if( element == Element::UNDEFINED ) {
						LOG("unknown element: %s\n", elementname);
						ok = false;
						break;
					}

					//map->sectors[sec_x][sec_y]->setElements(element, n_elements);
					map->getSector(sec_x, sec_y)->setElements(element, n_elements);
				}
				else {
					LOG("unknown word: %s\n", ptr);
					ok = false;
					break;
				}
			}
		}
	}
	fclose(file);
	return ok;
}

bool readMap(char *filename) {
	char fullname[4096] = "";
	sprintf(fullname, "%s/%s", maps_dirname, filename);
	FILE *file = fopen(fullname, "rb");
	if( file == NULL ) {
		// open in binary mode, so that we parse files in an OS-independent manner
		// (otherwise, Windows will parse "\r\n" as being "\n", but Linux will still read it as "\n")
		LOG("failed to open file: %s\n", fullname);
		return false;
	}

	bool ok = true;
	const int MAX_LINE = 4096;
	char line[MAX_LINE+1] = "";
	Map *map = NULL;
	int epoch = -1;
	int index = -1;
	while( ok ) {
		errno = 0;
		if( fgets(line, MAX_LINE, file) == NULL ) {
			if( errno ) {
				LOG("error reading line\n");
				ok = false;
			}
			break;
		}
		else {
			line[ strlen(line) - 1 ] = '\0'; // trim new line
			line[ strlen(line) - 1 ] = '\0'; // trim carriage return
			if( map == NULL ) {
				if( line[0] != '#' ) {
					LOG("expected first character to be '#'\n");
					ok = false;
					break;
				}

				char name[MAX_LINE+1] = "";
				int n_opponents = -1;
				char colname[MAX_LINE+1] = "";

				char *ptr = strtok(&line[1], " ");
				if( ptr == NULL ) {
					LOG("can't find map name\n");
					ok = false;
					break;
				}
				strcpy(name, ptr);

				ptr = strtok(NULL, " ");
				if( ptr == NULL ) {
					LOG("can't find epoch\n");
					ok = false;
					break;
				}
				epoch = atoi(ptr);

				ptr = strtok(NULL, " ");
				if( ptr == NULL ) {
					LOG("can't find n_opponents\n");
					ok = false;
					break;
				}
				n_opponents = atoi(ptr);

				ptr = strtok(NULL, " ");
				if( ptr == NULL ) {
					LOG("can't find colour name\n");
					ok = false;
					break;
				}
				strcpy(colname, ptr);

				MapColour map_colour = MAP_UNDEFINED_COL;
				if( strcmp(colname, "ORANGE") == 0 ) {
					map_colour = MAP_ORANGE;
				}
				else if( strcmp(colname, "GREEN") == 0 ) {
					map_colour = MAP_GREEN;
				}
				else if( strcmp(colname, "BROWN") == 0 ) {
					map_colour = MAP_BROWN;
				}
				else if( strcmp(colname, "WHITE") == 0 ) {
					map_colour = MAP_WHITE;
				}
				else if( strcmp(colname, "DBROWN") == 0 ) {
					map_colour = MAP_DBROWN;
				}
				else if( strcmp(colname, "DGREEN") == 0 ) {
					map_colour = MAP_DGREEN;
				}
				else if( strcmp(colname, "GREY") == 0 ) {
					map_colour = MAP_GREY;
				}
				else {
					LOG("unknown map colour: %s\n", colname);
					ok = false;
					break;
				}

				index = 0;
				while( index < max_islands_per_epoch_c && maps[epoch][index] != NULL )
					index++;
				if( index == max_islands_per_epoch_c ) {
					LOG("too many islands for this epoch\n");
					ok = false;
					break;
				}
				map = maps[epoch][index] = new Map(map_colour, n_opponents, name);
				//strcpy(map->filename, filename);
				map->setFilename(filename);
			}
			else {
				char *line_ptr = line;
				while( *line_ptr == ' ' || *line_ptr == '\t' )
					line_ptr++;
				char *ptr = strtok(line_ptr, " ");
				if( ptr == NULL ) {
					LOG("can't find first word\n");
					ok = false;
					break;
				}
				else if( strcmp(ptr, "SECTOR") == 0 ) {
					ptr = strtok(NULL, " ");
					if( ptr == NULL ) {
						LOG("can't find sec_x\n");
						ok = false;
						break;
					}
					int sec_x = atoi(ptr);
					if( sec_x < 0 || sec_x >= map_width_c ) {
						LOG("invalid map x %d\n", sec_x);
						ok = false;
						break;
					}

					ptr = strtok(NULL, " ");
					if( ptr == NULL ) {
						LOG("can't find sec_y\n");
						ok = false;
						break;
					}
					int sec_y = atoi(ptr);
					if( sec_y < 0 || sec_y >= map_height_c ) {
						LOG("invalid map y %d\n", sec_y);
						ok = false;
						break;
					}
					map->newSquareAt(sec_x, sec_y);
				}
				else if( strcmp(ptr, "ELEMENT") == 0 ) {
					// ignore for now
				}
				else {
					LOG("unknown word: %s\n", ptr);
					ok = false;
					break;
				}
			}
		}
	}
	fclose(file);
	if( !ok && map != NULL ) {
		LOG("delete map that was created\n");
		delete map;
		ASSERT(epoch != -1); // should have been set, if map!=NULL
		ASSERT(index != -1); // should have been set, if map!=NULL
		map = maps[epoch][index] = NULL;
	}
	return ok;
}

int sortMapsFunc(const void *a, const void *b) {
	Map *map_a = *(Map **)a;
	Map *map_b = *(Map **)b;
	// handling for 'Ohm'...
	if( map_a->getName()[0] == '0' )
		return 1;
	else if( map_b->getName()[0] == '0' )
		return -1;
	return strcmp(map_a->getName(), map_b->getName());
}

bool createMaps() {
	LOG("createMaps()...\n");
#ifdef _WIN32
	WIN32_FIND_DATA findFileData;
	char maps_dirname_w[256];
	sprintf(maps_dirname_w, "%s\\*", maps_dirname);
	HANDLE handle = FindFirstFile(maps_dirname_w, &findFileData);
	if( handle == INVALID_HANDLE_VALUE ) {
		LOG("Invalid File Handle. GetLastError reports %d\n", GetLastError());
		return false;
	}
	for(;;) {
		LOG("found file: %s\n", findFileData.cFileName);
		if( !readMap(findFileData.cFileName) ) {
			LOG("failed reading map: %s\n", findFileData.cFileName);
			// don't fail altogether, just ignore
		}
		if( FindNextFile(handle, &findFileData) == 0 ) {
			FindClose(handle);
			DWORD error = GetLastError();
			if( error != ERROR_NO_MORE_FILES ) {
				LOG("error reading directory: %d\n", error);
				return false;
			}
			break;
		}
	}
#endif
#ifdef __linux
	DIR *dir = opendir(maps_dirname);
	if( dir == NULL ) {
		LOG("failed to open directory: %s\n", maps_dirname);
		return false;
	}
	for(;;) {
		errno = 0;
		dirent *ent = readdir(dir);
		if( ent == NULL ) {
			closedir(dir);
			if( errno ) {
				LOG("error reading directory: %d\n", errno);
				return false;
			}
			break;
		}
		LOG("found file: %s\n", ent->d_name);
		if( !readMap(ent->d_name) ) {
			LOG("failed reading map: %s\n", ent->d_name);
			// don't fail altogether, just ignore
		}
	}
#endif
	LOG("done reading directory\n");

	for(int i=0;i<n_epochs_c;i++) {
		int n_islands = 0;
		while(n_islands < max_islands_per_epoch_c && maps[i][n_islands] != NULL)
			n_islands++;
		qsort((void *)&maps[i], n_islands, sizeof(maps[i][0]), sortMapsFunc);
	}
	return true;

}

void createSectors(PlayingGameState *gamestate, int x, int y, int n_men) {
	LOG("createSectors(%d, %d, %d)\n", x, y, n_men);

	map->createSectors(gamestate, start_epoch);
	//current_sector = map->sectors[x][y];
	current_sector = map->getSector(x, y);
	if( human_player != PLAYER_DEMO )
		current_sector->createTower(human_player, n_men);
	//current_sector->createTower(human_player, 10);

	//current_sector->getArmy(enemy_player)->soldiers[10] = 0;
	//map->sectors[2][2]->getArmy(enemy_player)->soldiers[0] = 10;

	//Sector *enemy_sector = map->sectors[1][2];

	for(int i=0;i<n_players_c;i++) {
		if( i == human_player || players[i] == NULL )
			continue;
		int ex = 0, ey = 0;
		while( true ) {
			map->findRandomSector(&ex, &ey);
			/*if( ex != x || ey != y )
			break;*/
			//ASSERT( map->sector_at[ex][ey] );
			ASSERT( map->isSectorAt(ex, ey) );
			//if( map->sectors[ex][ey]->getPlayer() == -1 )
			if( map->getSector(ex, ey)->getPlayer() == -1 )
				break;
		}
		//Sector *enemy_sector = map->sectors[ex][ey];
		Sector *enemy_sector = map->getSector(ex, ey);
		//enemy_sector->getArmy(enemy_player)->soldiers[10] = 30;
		//enemy_sector->createTower(enemy_player, 12);
		if( start_epoch == end_epoch_c ) {
			//players[i]->n_men_for_this_island = n_suspended[i];
			players[i]->setNMenForThisIsland(100);
		}
		else {
			players[i]->setNMenForThisIsland(20 + 5*start_epoch);
			// total: 360*3 + 65 = 1145 men
		}
		LOG("Enemy %d created at %d , %d\n", i, ex, ey);
		enemy_sector->createTower(i, players[i]->getNMenForThisIsland());
		//enemy_sector->createTower(enemy_player, 20);
		//enemy_sector->createTower(enemy_player, 200);
	}

	if( !readSectors(map) ) {
		LOG("failed to read map sector info!\n");
		ASSERT(false);
	}
}

GameState *dispose_gamestate = NULL;

void disposeGameState() {
	ASSERT( dispose_gamestate == NULL );
	dispose_gamestate = gamestate;
	gamestate = NULL;
}

void setGameStateID(GameStateID state) {
	gameStateID = state;
	playMusic();

	GameState *old_gamestate = gamestate;
	if( gamestate != NULL ) {
		//delete gamestate; // todo: postpone until later!
		disposeGameState();
	}

	if( gameStateID == GAMESTATEID_CHOOSEGAMETYPE )
		gamestate = new ChooseGameTypeGameState();
	else if( gameStateID == GAMESTATEID_CHOOSEDIFFICULTY )
		gamestate = new ChooseDifficultyGameState();
	else if( gameStateID == GAMESTATEID_CHOOSEPLAYER )
		gamestate = new ChoosePlayerGameState();
	else if( gameStateID == GAMESTATEID_PLACEMEN )
		gamestate = new PlaceMenGameState();
	else if( gameStateID == GAMESTATEID_PLAYING ) {
		gamestate = new PlayingGameState();
		int map_x = static_cast<PlaceMenGameState *>(old_gamestate)->getStartMapX();
		int map_y = static_cast<PlaceMenGameState *>(old_gamestate)->getStartMapY();
		int n_men = human_player == PLAYER_DEMO ? 0 : players[human_player]->getNMenForThisIsland();
		createSectors(static_cast<PlayingGameState *>(gamestate), map_x, map_y, n_men);
	}
	else if( gameStateID == GAMESTATEID_ENDISLAND )
		gamestate = new EndIslandGameState();
	else if( gameStateID == GAMESTATEID_GAMECOMPLETE )
		gamestate = new GameCompleteGameState();
	else {
		ASSERT(false);
	}

	gamestate->reset();
	state_changed = false;
	//gameWon = GAME_PLAYING;
	/*if( state == GAMESTATE_PLACEMEN ) {
	placeMenInfo.init();
	}*/
}

void startIsland() {
	ASSERT(gameStateID == GAMESTATEID_PLACEMEN);
	/*int map_x = static_cast<PlaceMenGameState *>(gamestate)->getStartMapX();
	int map_y = static_cast<PlaceMenGameState *>(gamestate)->getStartMapY();*/

	//setupPlayers();
	time_rate = human_player == PLAYER_DEMO ? 5 : 1;
	LOG("time_rate = %d\n", time_rate);
	setGameStateID(GAMESTATEID_PLAYING);
	/*int n_men = human_player == PLAYER_DEMO ? players[human_player]->getNMenForThisIsland() : 0;
	createSectors(static_cast<PlayingGameState *>(gamestate), map_x, map_y, n_men);*/
	gamestate->fadeScreen(false, 0, NULL);
	gameResult = GAMERESULT_UNDEFINED;
}

void endIsland() {
	ASSERT(gameStateID == GAMESTATEID_PLAYING);
	map->calculateStats();
	map->freeSectors();
	for(int i=0;i<n_players_c;i++) {
		if( players[i] != NULL )
			n_suspended[i] += players[i]->getNSuspended();
	}
	//cleanupPlayers();
	setGameStateID(GAMESTATEID_ENDISLAND);
	gamestate->fadeScreen(false, 0, NULL);

	if( gameResult == GAMERESULT_QUIT ) {
		// pick a random non-human player
		int n_cpu = 0;
		for(int i=0;i<n_players_c;i++) {
			if( players[i] != NULL && !players[i]->isDead() && i != human_player ) {
				n_cpu++;
			}
		}
		if( n_cpu > 0 ) {
			int index = rand() % n_cpu;
			n_cpu = 0;
			for(int i=0;i<n_players_c;i++) {
				if( players[i] != NULL && !players[i]->isDead() && i != human_player ) {
					if( n_cpu == index ) {
						playSample( s_quit[i] );
						break;
					}
					n_cpu++;
				}
			}
		}
	}
}

void returnToChooseIsland() {
	ASSERT(gameStateID == GAMESTATEID_ENDISLAND);
	if( gameResult == GAMERESULT_WON && gameType == GAMETYPE_ALLISLANDS ) {
		//n_men_store -= n_men_for_this_island;
		n_men_store -= players[human_player]->getNMenForThisIsland();
		if( start_epoch == n_epochs_c-1 ) {
			setGameStateID(GAMESTATEID_GAMECOMPLETE);
		}
		else {
			setGameStateID(GAMESTATEID_PLACEMEN);

			ASSERT( !completed_island[selected_island] );
			completed_island[selected_island] = true;
			// check advance to next epoch
			bool completed_epoch = true;
			for(int i=0;i<max_islands_per_epoch_c && maps[start_epoch][i] != NULL && completed_epoch;i++) {
				if( !completed_island[i] )
					completed_epoch = false;
			}
			LOG("completed epoch? %d\n", completed_epoch);
			if( completed_epoch ) {
				n_men_store += getMenPerEpoch();
				nextEpoch();
			}
			else {
				nextIsland();
			}
		}
	}
	else {
		setGameStateID(GAMESTATEID_PLACEMEN);
		// test
		//setGameStateID(GAMESTATEID_GAMECOMPLETE);
	}

	gamestate->fadeScreen(false, 0, NULL);

	//n_men_for_this_island = 0;
	if( human_player != PLAYER_DEMO )
		players[human_player]->setNMenForThisIsland(0);
}

void startNewGame() {
	ASSERT(gameStateID == GAMESTATEID_GAMECOMPLETE);
	setGameStateID(GAMESTATEID_PLACEMEN);
	gamestate->fadeScreen(false, 0, NULL);
	newGame();
}

void placeTower() {
	ASSERT( gameStateID == GAMESTATEID_PLACEMEN );
	play_music = ((PlaceMenGameState *)gamestate)->getChooseMenPanel()->musicOn();
	if( !state_changed ) {
		//createSectors(x, y, n_men); // now done in startIsland()
		state_changed = true;
		gamestate->fadeScreen(true, 0, startIsland);
		//startIsland();
	}
}


void cleanup() {
	LOG("cleanup()\n");
	for(int i=0;i<n_epochs_c;i++) {
		for(int j=0;j<max_islands_per_epoch_c;j++) {
			if( maps[i][j] != NULL )
				delete maps[i][j];
		}
	}
	if( screen != NULL )
		delete screen;
	TrackedObject::cleanup();
	// no longer need to stop music, as it's deleted as a TrackedObject
	//stopMusic();
	freeSound();
	SDL_Quit();
}

bool quit = false;
bool debugwindow = false;

void playGame(int n_args, char *args[]) {
	bool fullscreen = false;
#ifdef _DEBUG
	debugwindow = true;
	//fullscreen = true;
#endif

	for(int i=0;i<n_args;i++) {
		if( strcmp(args[i], "fullscreen") == 0 )
			fullscreen = true;
		else if( strcmp(args[i], "debugwindow") == 0 )
			debugwindow = true;
	}
#ifdef _WIN32
	if( debugwindow ) {
		AllocConsole();
		FILE *dummy = NULL;
		freopen_s(&dummy, "con", "w", stdout);
		SetConsoleTitle("DEBUG:");
	}
	else {
		FreeConsole(); // get rid of Visual Studio startup window
	}
#endif

	initLogFile();

	/*if( access("data", 0)==0 ) {
	use_amigadata = true;
	}
	else if( access("Mega Lo Mania", 0)==0 ) {
	use_amigadata = false;
	}
	else {
	LOG("can't find data folder\n");
	return;
	}*/

	for(int i=0;i<n_epochs_c;i++)
		for(int j=0;j<max_islands_per_epoch_c;j++)
			maps[i][j] = NULL;

	for(int i=0;i<max_islands_per_epoch_c;i++)
		completed_island[i] = false;

	for(int i=0;i<n_players_c;i++)
		n_suspended[i] = 0;

	// init SDL
	if( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO) == -1 ) {
		LOG("SDL_Init failed!!!\n");
		return;
	}
	// init FMOD
	else if( !initSound() ) {
		// don't fail, just warn
		LOG("Failed to initialise sound system!!!\n");
		/*SDL_Quit();
		return;*/
	}

	LOG("successfully opened libraries\n");

	bool ok = true;
	if( !openScreen(fullscreen) ) {
		LOG("failed to open screen\n");
		ok = false;
#ifdef _WIN32
		MessageBoxA(NULL, "Failed to open screen", "Error", MB_OK|MB_ICONEXCLAMATION);
#endif
	}
	else if( !loadImages() ) {
		LOG("failed to load images\n");
		ok = false;
#ifdef _WIN32
		MessageBoxA(NULL, "Failed to load images", "Error", MB_OK|MB_ICONEXCLAMATION);
#endif
	}

	if( !ok ) {
		cleanup();
		return;
	}
	// n.b., still need to load samples even if sound failed to initialise, as we want the Sample objects for the textual display
	if( !loadSamples() ) {
		// don't fail, just warn
		LOG("warning - failed to load samples\n");
		// no longer show message - no longer an error, as the default install won't have any samples!
/*#ifdef _WIN32
		MessageBoxA(NULL, "Failed to load all samples", "Warning", MB_OK|MB_ICONEXCLAMATION);
#endif*/
	}

	setupInventions();
	setupElements();
	Design::setupDesigns();
	if( !createMaps() ) {
		LOG("failed to create maps\n");
		cleanup();
#ifdef _WIN32
		MessageBoxA(NULL, "Failed to create maps", "Error", MB_OK|MB_ICONEXCLAMATION);
#endif
		return;
	}

	map = maps[start_epoch][selected_island];

	for(int i=0;i<TrackedObject::getNumTags();i++) {
		TrackedObject *to = TrackedObject::getTag(i);
		if( to != NULL && strcmp( to->getClass(), "CLASS_IMAGE" ) == 0 ) {
			Image *image = (Image *)to;
			image->convertToDisplayFormat();
		}
	}

	char buffer[256] = "";
	sprintf(buffer, "Gigalomania, version %d.%d", majorVersion, minorVersion);
	screen->setTitle(buffer);

	setGameStateID(GAMESTATEID_CHOOSEGAMETYPE);
	//setGameStateID(GAMESTATEID_CHOOSEPLAYER);

	Uint32 elapsed_time = SDL_GetTicks();

	lastmouseclick_time = 0;
	const int mouseclick_delay = 100;

	SDL_Event event;
	bool paused = false;
	quit = false;
	while(!quit) {
		//LOG("start game loop\n");
		//FSOUND_Update();
		updateSound();
		// draw screen
		if( !paused )
			gamestate->draw();

		// time stuff
		SDL_Delay(timeLeft());
		Uint32 new_time = SDL_GetTicks();
		if( !paused ) {
			updateTime(new_time - elapsed_time);
		}
		else
			updateTime(0);
		elapsed_time = new_time;

		// user input
		if( SDL_PollEvent(&event) == 1 ) {
			bool playing_game_quit = false;
			switch (event.type) {
								case SDL_QUIT:
									if( gameStateID == GAMESTATEID_PLAYING ) {
										if( !state_changed )
											playing_game_quit = true;
									}
									else
										quit = true;
									break;
								case SDL_KEYDOWN:
									{
										SDL_keysym key = event.key.keysym;
										if( key.sym == SDLK_ESCAPE ) {
											if( gameStateID == GAMESTATEID_PLAYING ) {
												if( !state_changed )
													playing_game_quit = true;
											}
											else
												quit = true;
										}
										else if( key.sym == SDLK_p ) {
											paused = !paused;
											if( paused ) {
												playSample(s_on_hold);
												//playSample(s_conquered); // uncomment to test chained samples
												//playSample(s_explosion, SOUND_CHANNEL_FX);
												//playSample(s_explosion);
											}
										}
										else if( gameStateID == GAMESTATEID_PLAYING && key.sym == SDLK_q && !state_changed ) {
											playing_game_quit = true;
										}
										break;
									}
									/*case SDL_MOUSEBUTTONDOWN:
									{
									if( !paused ) {
									SDL_MouseButtonEvent *button = &event.button;
									screen->mouseClick(button->x, button->y, button->button == SDL_BUTTON_LEFT, button->button == SDL_BUTTON_RIGHT);
									}
									break;
									}*/
			}
			if( playing_game_quit ) {
				state_changed = true;
				gameResult = GAMERESULT_QUIT;
				gamestate->fadeScreen(true, 0, endIsland);
			}
		}
		SDL_PumpEvents();
		if( !paused ) {
			int m_x = 0, m_y = 0;
			Uint8 m_b = SDL_GetMouseState(&m_x,&m_y);
			if( m_b != 0 && getRealTime() - lastmouseclick_time >= mouseclick_delay ) {
				//LOG("ping: %d - %d\n", getRealTime(), lastmouseclick_time);
				lastmouseclick_time = getRealTime();
				gamestate->mouseClick(m_x, m_y, m_b);
			}
			if( m_b == 0 ) {
				resetMouseClick();
			}
		}

		// update
		if( !paused ) {
			if( gameStateID == GAMESTATEID_PLAYING ) {
				for(int i=0;i<n_players_c;i++) {
					if( i != human_player && players[i] != NULL )
						players[i]->doAIUpdate(static_cast<PlayingGameState *>(gamestate));
				}
				//players[ enemy_player ]->doAIUpdate();
				gamestate->update();
				for(int y=0;y<map_height_c;y++) {
					for(int x=0;x<map_width_c;x++) {
						/*if( map->sectors[x][y] != NULL )
							map->sectors[x][y]->update();*/
						Sector *sector = map->getSector(x, y);
						if( sector != NULL ) {
							sector->update();
						}
					}
				}
			}
		}

		if( gameStateID == GAMESTATEID_PLAYING && !state_changed ) {
			if( human_player != PLAYER_DEMO && !playerAlive(human_player) ) {
				playSample(s_itis_all_over);
				state_changed = true;
				gameResult = GAMERESULT_LOST;
				gamestate->fadeScreen(true, SHORT_DELAY, endIsland);
			}
			else {
				bool all_dead = true;
				/*for(int i=0;i<n_players_c && all_dead;i++) {
				if( i != human_player && players[i] != NULL && playerAlive(i) )
				all_dead = false;
				}*/
				for(int i=0;i<n_players_c;i++) {
					if( i != human_player && players[i] != NULL && !players[i]->isDead() ) {
						if( playerAlive(i) )
							all_dead = false;
						else {
							players[i]->kill(static_cast<PlayingGameState *>(gamestate));
						}
					}
				}
				//if( !playerAlive(enemy_player) ) {
				if( all_dead ) {
					playSample(s_won);
					state_changed = true;
					gameResult = GAMERESULT_WON;
					gamestate->fadeScreen(true, SHORT_DELAY, endIsland);
				}
			}
		}

		if( dispose_gamestate != NULL ) {
			delete dispose_gamestate;
			dispose_gamestate = NULL;
		}
	}

	cleanup();
	LOG("exiting..\n");
}

void quitGame() {
	quit = true;
}

bool playerAlive(int player) {
	/*int n_player_sectors = 0;
	int n_army = 0;
	for(int x=0;x<map_width_c;x++) {
	for(int y=0;y<map_height_c;y++) {
	Sector *sector = map->sectors[x][y];
	if( sector != NULL ) {
	if( player == sector->getActivePlayer() )
	n_player_sectors++;
	n_army += sector->getArmy(player)->getTotal();
	}
	}
	}
	return ( n_player_sectors > 0 || n_army > 0 );*/
	for(int x=0;x<map_width_c;x++) {
		for(int y=0;y<map_height_c;y++) {
			//Sector *sector = map->sectors[x][y];
			Sector *sector = map->getSector(x, y);
			if( sector != NULL ) {
				if( player == sector->getActivePlayer() )
					return true;
				else if( sector->getArmy(player)->any(true) )
					return true;
			}
		}
	}
	return false;
}
