//---------------------------------------------------------------------------
#ifdef _WIN32
#define _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES 1
#endif

#ifdef __linux
#include <SDL/SDL.h>
#include <stdlib.h>
#else
#include <sdl.h>
#endif

#include <cassert>
#include <cmath>

#include "player.h"
#include "panel.h"
#include "gui.h"
#include "screen.h"
#include "gamestate.h"
#include "image.h"
#include "sound.h"
#include "utils.h"
//---------------------------------------------------------------------------

const int offset_panel_x_c = 0;
const int offset_panel_y_c = 100;
const int offset_attack_x_c = 4;
const int space_attack_x_c = 18;

ChooseGameTypePanel::ChooseGameTypePanel() : MultiPanel(1, 0, 0) {
	this->c_page = 0;
	this->addToPanel(0, new Button(105, 50, "CHOOSE A GAME TYPE", letters_large));
	this->button_singleisland = new Button(120, 80, "SINGLE ISLAND", letters_large);
	this->button_singleisland->setInfoLMB("choose to play on\nany island");
	this->addToPanel(0, button_singleisland);
	this->button_allislands = new Button(125, 100, "ALL ISLANDS", letters_large);
	this->button_allislands->setInfoLMB("start on first epoch and\nplay full game");
	this->addToPanel(0, button_allislands);
}

void ChooseGameTypePanel::input(int m_x,int m_y,Uint8 m_b) {
	MultiPanel::input(m_x, m_y, m_b);
	bool m_left = mouse_left(m_b);
	//bool m_right = mouse_right(m_b);

	if( m_left && this->button_singleisland->mouseOver(m_x, m_y) ) {
		gameType = GAMETYPE_SINGLEISLAND;
		setGameStateID(GAMESTATEID_CHOOSEPLAYER);
		registerClick();
	}
	else if( m_left && this->button_allislands->mouseOver(m_x, m_y) ) {
		gameType = GAMETYPE_ALLISLANDS;
		//setGameStateID(GAMESTATEID_CHOOSEPLAYER);
		setGameStateID(GAMESTATEID_CHOOSEDIFFICULTY);
		registerClick();
	}
}

ChooseDifficultyPanel::ChooseDifficultyPanel() : MultiPanel(1, 0, 0) {
	this->c_page = 0;
	this->addToPanel(0, new Button(105, 50, "CHOOSE DIFFICULTY", letters_large));
	this->button_easy = new Button(120, 80, "EASY", letters_large);
	this->addToPanel(0, button_easy);
	this->button_medium = new Button(120, 100, "MEDIUM", letters_large);
	this->addToPanel(0, button_medium);
	this->button_hard = new Button(120, 120, "HARD", letters_large);
	this->addToPanel(0, button_hard);
}

void ChooseDifficultyPanel::input(int m_x,int m_y,Uint8 m_b) {
	MultiPanel::input(m_x, m_y, m_b);
	bool m_left = mouse_left(m_b);
	//bool m_right = mouse_right(m_b);

	if( m_left && this->button_easy->mouseOver(m_x, m_y) ) {
		difficulty_level = DIFFICULTY_EASY;
		setGameStateID(GAMESTATEID_CHOOSEPLAYER);
		registerClick();
	}
	else if( m_left && this->button_medium->mouseOver(m_x, m_y) ) {
		difficulty_level = DIFFICULTY_MEDIUM;
		setGameStateID(GAMESTATEID_CHOOSEPLAYER);
		registerClick();
	}
	else if( m_left && this->button_hard->mouseOver(m_x, m_y) ) {
		difficulty_level = DIFFICULTY_HARD;
		setGameStateID(GAMESTATEID_CHOOSEPLAYER);
		registerClick();
	}
}

ChooseMenPanel::ChooseMenPanel(PlaceMenGameState *gamestate) : MultiPanel(N_STATES, 0, 0), gamestate(gamestate) {
	this->c_page = (int)STATE_CHOOSEISLAND;
	this->n_men = 0;

	this->button_nextisland = NULL;
	int mx = gamestate->getOffX();
	int fw = letters_large[0]->getScaledWidth();
	if( maps[start_epoch][1] != NULL ) {
		//this->button_nextisland = new Button(14, 146, "NEXT ISLAND", letters_large);
		this->button_nextisland = new Button(mx - 5.5*fw, 146, "NEXT ISLAND", letters_large);
		this->addToPanel(STATE_CHOOSEISLAND, button_nextisland);
	}
	this->button_nextepoch = NULL;
	if( gameType == GAMETYPE_SINGLEISLAND ) {
		//this->button_nextepoch = new Button(18, 164, "NEXT EPOCH", letters_large);
		this->button_nextepoch = new Button(mx - 5*fw, 164, "NEXT EPOCH", letters_large);
		this->addToPanel(STATE_CHOOSEISLAND, button_nextepoch);
	}
	//this->button_options = new Button(28, 182, "OPTIONS", letters_large);
	this->button_options = new Button(mx - 3.5*fw, 182, "OPTIONS", letters_large);
	this->addToPanel(STATE_CHOOSEISLAND, button_options);
	//this->button_play = new Button(16, 200, "PLAY ISLAND", letters_large);
	this->button_play = new Button(mx - 5.5*fw, 200, "PLAY ISLAND", letters_large);
	this->addToPanel(STATE_CHOOSEISLAND, button_play);
	//this->button_quit = new Button(36, 218, "QUIT", letters_large);
	this->button_quit = new Button(mx - 2*fw, 218, "QUIT", letters_large);
	this->addToPanel(STATE_CHOOSEISLAND, button_quit);

	//this->button_nmen = new PanelPage(32, 180, 24, 12);
	this->button_nmen = new PanelPage(mx - 12, 180, 24, 12);
	this->button_nmen->setInfoLMB("decrease the number of men\nto play this island");
	this->button_nmen->setInfoRMB("increase the number of men\nto play this island");
	this->addToPanel(STATE_CHOOSEMEN, button_nmen);

	//this->button_continue = new Button(24, 164, "CONTINUE", letters_large);
	this->button_continue = new Button(mx - 4*fw, 164, "CONTINUE", letters_large);
	this->addToPanel(STATE_OPTIONS, button_continue);
	char *music_texts[] = { "MUSIC ON", "MUSIC OFF" };
	//this->button_music = new CycleButton(24, 182, music_texts, 2, letters_large);
	this->button_music = new CycleButton(mx - 4.5*fw, 182, music_texts, 2, letters_large);
	this->button_music->setActive( play_music ? 0 : 1 );
	this->addToPanel(STATE_OPTIONS, button_music);
	this->button_new = NULL;
	this->button_load = NULL;
	this->button_save = NULL;
	if( gameType == GAMETYPE_ALLISLANDS ) {
		//this->button_new = new Button(24, 200, "NEW GAME", letters_large);
		this->button_new = new Button(mx - 4*fw, 200, "NEW GAME", letters_large);
		this->addToPanel(STATE_OPTIONS, button_new);
		//this->button_load = new Button(36, 218, "LOAD", letters_large);
		this->button_load = new Button(mx - 2*fw, 218, "LOAD", letters_large);
		this->addToPanel(STATE_OPTIONS, button_load);
		//this->button_save = new Button(36, 236, "SAVE", letters_large);
		this->button_save = new Button(mx - 2*fw, 236, "SAVE", letters_large);
		this->addToPanel(STATE_OPTIONS, button_save);
	}

	/*this->button_load_load = new Button(16, 228, "LOAD", letters_small);
	this->addToPanel(STATE_LOADGAME, button_load_load);
	this->button_load_cancel = new Button(48, 228, "CANCEL", letters_small);
	this->addToPanel(STATE_LOADGAME, button_load_cancel);*/
	//this->button_load_cancel = new Button(28, 228, "CANCEL", letters_small);
	this->button_load_cancel = new Button(mx - 3*fw, 228, "CANCEL", letters_small);
	this->addToPanel(STATE_LOADGAME, button_load_cancel);

	//this->button_save_cancel = new Button(28, 228, "CANCEL", letters_small);
	this->button_save_cancel = new Button(mx - 3*fw, 228, "CANCEL", letters_small);
	this->addToPanel(STATE_SAVEGAME, button_save_cancel);

	for(int i=0;i<n_slots_c;i++) {
		this->button_load_load[i] = NULL;
		this->button_save_save[i] = NULL;
	}
	if( gameType == GAMETYPE_ALLISLANDS ) {
		refreshLoadSaveButtons();
	}
}

void ChooseMenPanel::refreshLoadSaveButtons() {
	for(int i=0;i<n_slots_c;i++) {
		if( this->button_load_load[i] != NULL )
			delete this->button_load_load[i];
		this->button_load_load[i] = NULL;
		DifficultyLevel difficulty = DIFFICULTY_EASY;
		int player = 0;
		int n_men = 0;
		int epoch = 0;
		int suspended[n_players_c];
		bool completed[max_islands_per_epoch_c];
		if( loadGameInfo( &difficulty, &player, &n_men, suspended, &epoch, completed, i ) ) {
			char buffer[256] = "";
			char diff_ch = '\0';
			if( difficulty == DIFFICULTY_EASY )
				diff_ch = 'E';
			else if( difficulty == DIFFICULTY_MEDIUM )
				diff_ch = 'M';
			else if( difficulty == DIFFICULTY_HARD )
				diff_ch = 'H';
			else {
				ASSERT(false);
			}
			sprintf(buffer, "%d %c %s Epoch %d  Men %d  Suspended %d", i, diff_ch, PlayerType::getName((PlayerType::PlayerTypeID)player), epoch+1, n_men, suspended[player]);
			//this->button_load_load[i] = new Button(96, 228 + (i-n_slots_c) * 8, buffer, letters_small);
			this->button_load_load[i] = new Button(16, 228 + (i-n_slots_c) * 8, buffer, letters_small);
			this->addToPanel(STATE_LOADGAME, button_load_load[i]);
		}
	}

	for(int i=0;i<n_slots_c;i++) {
		delete this->button_save_save[i];
		DifficultyLevel difficulty = DIFFICULTY_EASY;
		int player = 0;
		int n_men = 0;
		int epoch = 0;
		char buffer[256] = "";
		int suspended[n_players_c];
		bool completed[max_islands_per_epoch_c];
		if( loadGameInfo( &difficulty, &player, &n_men, suspended, &epoch, completed, i ) ) {
			char diff_ch = '\0';
			if( difficulty == DIFFICULTY_EASY )
				diff_ch = 'E';
			else if( difficulty == DIFFICULTY_MEDIUM )
				diff_ch = 'M';
			else if( difficulty == DIFFICULTY_HARD )
				diff_ch = 'H';
			else {
				ASSERT(false);
			}
			//sprintf(buffer, "%d %c %s Epoch %d Men %d Stored %d", i, diff_ch, PlayerType::getName((PlayerType::PlayerTypeID)player), epoch+1, n_men, suspended[player]);
			sprintf(buffer, "%d %c %s Epoch %d  Men %d  Saved %d", i, diff_ch, PlayerType::getName((PlayerType::PlayerTypeID)player), epoch+1, n_men, suspended[player]);
		}
		else {
			sprintf(buffer, "UNUSED Slot %d", i);
		}
		//this->button_save_save[i] = new Button(96, 228 + (i-n_slots_c) * 8, buffer, letters_small);
		this->button_save_save[i] = new Button(16, 228 + (i-n_slots_c) * 8, buffer, letters_small);
		this->addToPanel(STATE_SAVEGAME, button_save_save[i]);
	}
}

void ChooseMenPanel::draw() {
	MultiPanel::draw();

	if( this->getPage() == STATE_CHOOSEMEN ) {
		//unarmed_man->draw( this->button_nmen->getLeft() + 4, this->button_nmen->getBottom() + 4, true);
		men[start_epoch]->draw( this->button_nmen->getLeft() + 4, this->button_nmen->getBottom() + 4, true);
		Image::writeNumbers( this->button_nmen->getXCentre(), this->button_nmen->getTop() + 2, numbers_white, this->n_men,Image::JUSTIFY_CENTRE, true);
		Image::writeNumbers( this->button_nmen->getXCentre(), this->button_nmen->getBottom() + 24, numbers_white, getMenAvailable() - this->n_men,Image::JUSTIFY_CENTRE, true);
	}
}

void ChooseMenPanel::input(int m_x,int m_y,Uint8 m_b) {
	MultiPanel::input(m_x, m_y, m_b);
	bool m_left = mouse_left(m_b);
	bool m_right = mouse_right(m_b);

	if( this->c_page == STATE_CHOOSEISLAND ) {
		if( m_left && this->button_nextisland != NULL && this->button_nextisland->mouseOver(m_x, m_y) ) {
			registerClick();
			nextIsland();
		}
		else if( m_left && this->button_nextepoch != NULL && this->button_nextepoch->mouseOver(m_x, m_y) ) {
			registerClick();
			nextEpoch();
		}
		else if( m_left && this->button_options->mouseOver(m_x, m_y) ) {
			this->c_page = STATE_OPTIONS;
			registerClick();
		}
		else if( m_left && this->button_play->mouseOver(m_x, m_y) ) {
			this->c_page = STATE_CHOOSEMEN;
			registerClick();
			setupPlayers();
		}
		else if( m_left && this->button_quit->mouseOver(m_x, m_y) ) {
			quitGame();
		}
	}
	else if( this->c_page == STATE_CHOOSEMEN ) {
		if( ( m_left || m_right ) && this->button_nmen->mouseOver(m_x, m_y) ) {
			int n = getNClicks();
			if( m_left && this->n_men > 0 ) {
				this->n_men -= n;
				if( this->n_men < 0 )
					this->n_men = 0;
			}
			else if( m_right ) {
				this->n_men += n;
				if( this->n_men > getMenAvailable() )
					this->n_men = getMenAvailable();
			}
			for(int y=0;y<map_height_c;y++) {
				for(int x=0;x<map_width_c;x++) {
					if( map->isSectorAt(x, y) ) {
						/*ASSERT( ((PlaceMenGameState *)gamestate)->map_panels[x][y] != NULL );
						((PlaceMenGameState *)gamestate)->map_panels[x][y]->setEnabled( n_men > 0 );*/
						/*ASSERT( ((PlaceMenGameState *)gamestate)->getMapPanel(x, y) != NULL );
						((PlaceMenGameState *)gamestate)->getMapPanel(x, y)->setEnabled( n_men > 0 );*/
						ASSERT( gamestate->getMapPanel(x, y) != NULL );
						gamestate->getMapPanel(x, y)->setEnabled( n_men > 0 );
					}
				}
			}
		}
	}
	else if( this->c_page == STATE_OPTIONS ) {
		if( m_left && this->button_continue->mouseOver(m_x, m_y) ) {
			this->c_page = STATE_CHOOSEISLAND;
			registerClick();
		}
		else if( m_left && this->button_new != NULL && this->button_new->mouseOver(m_x, m_y) ) {
			registerClick();
			//loadGame();
			newGame();
		}
		else if( m_left && this->button_load != NULL && this->button_load->mouseOver(m_x, m_y) ) {
			this->c_page = STATE_LOADGAME;
			registerClick();
		}
		else if( m_left && this->button_save != NULL && this->button_save->mouseOver(m_x, m_y) ) {
			//this->c_page = STATE_CHOOSEISLAND;
			this->c_page = STATE_SAVEGAME;
			registerClick();
		}
	}
	else if( this->c_page == STATE_LOADGAME ) {
		if( m_left && this->button_load_cancel->mouseOver(m_x, m_y) ) {
			this->c_page = STATE_CHOOSEISLAND;
			registerClick();
		}
		else if( m_left ) {
			for(int i=0;i<n_slots_c;i++) {
				if( this->button_load_load[i] != NULL && this->button_load_load[i]->mouseOver(m_x, m_y) ) {
					loadGame(i);
					this->c_page = STATE_CHOOSEISLAND;
					registerClick();
					return; // hack - need to return, as loading the game destroys this class!
				}
			}
		}
		/*else if( m_left && this->button_load_load->mouseOver(m_x, m_y) ) {
		loadGame(0);
		this->c_page = STATE_CHOOSEISLAND;
		}*/
	}
	else if( this->c_page == STATE_SAVEGAME ) {
		if( m_left && this->button_save_cancel->mouseOver(m_x, m_y) ) {
			this->c_page = STATE_CHOOSEISLAND;
			registerClick();
		}
		else if( m_left ) {
			for(int i=0;i<n_slots_c;i++) {
				if( this->button_save_save[i]->mouseOver(m_x, m_y) ) {
					saveGame(i);
					this->c_page = STATE_CHOOSEISLAND;
					registerClick();
					this->refreshLoadSaveButtons();
				}
			}
		}
	}
}

GamePanel::GamePanel(PlayingGameState *gamestate) : MultiPanel(GamePanel::N_STATES, offset_panel_x_c, offset_panel_y_c), gamestate(gamestate) {
	//this->state = STATE_SECTORCONTROL;
	this->c_page = (int)STATE_SECTORCONTROL;
	this->mousestate = MOUSESTATE_NORMAL;

	this->deploy_shield = -1;
	this->deploy_defence = -1;
	this->deploy_weapon = -1;
	this->designinfo = NULL;

	//this->setup();
}

GamePanel::~GamePanel() {
}

void GamePanel::setup() {
	char buffer[256] = "";
	this->free(false);

	// SECTORCONTROL

	const int start_x = 8;
	const int step_x = 20;
	const int step_y = 30;
	int xpos = start_x;
	//const int ypos = 224 - offset_panel_y_c;
	int ypos = 18;
	xpos += step_x;
	//ypos += step_y;

	//
	//this->button_shield = new ImageButton(28, 18, panel_shield, "check on building\ndamage status\nand repair buildings");
	this->button_shield = new ImageButton(xpos, ypos, panel_shield, "check on building\ndamage status\nand repair buildings");
	this->addToPanel(STATE_SECTORCONTROL, button_shield);
	xpos += step_x;

	//
	this->button_defence = new ImageButton(xpos, ypos, panel_defence, "view building weapon\nstocks and deploy\nbuilding weapons");
	this->addToPanel(STATE_SECTORCONTROL, button_defence);
	xpos += step_x;

	//
	this->button_attack = new ImageButton(xpos, ypos, panel_attack, "view army weapon stocks\nand assemble army");
	this->addToPanel(STATE_SECTORCONTROL, button_attack);
	//xpos += step_x;
	xpos = start_x;
	ypos += step_y;

	//this->button_knowndesigns = new ImageButton(8, 18, panel_knowndesigns, "view blueprints\nof completed designs");
	this->button_knowndesigns = new ImageButton(xpos, ypos, panel_knowndesigns, "view blueprints\nof completed designs");
	this->addToPanel(STATE_SECTORCONTROL, button_knowndesigns);
	xpos += step_x;

	//
	//this->button_design = new ImageButton(8, 40, panel_design, "view and alter\ncurrent design");
	this->button_design = new ImageButton(xpos, ypos, panel_design, "view and alter\ncurrent design");
	this->addToPanel(STATE_SECTORCONTROL, button_design);
	this->button_ndesigners = new PanelPage(xpos, ypos+16, 16, 16);
	this->button_ndesigners->setInfoLMB("decrease the number of designers");
	this->button_ndesigners->setInfoRMB("increase the number of designers");
	this->addToPanel(STATE_SECTORCONTROL, button_ndesigners);
	xpos += step_x;

	//
	//this->button_elementstocks = new ImageButton(40, 70, mine_gatherable_small, "view element stocks");
	/*this->button_elementstocks = new ImageButton(xpos, ypos, mine_gatherable_small, "view element stocks");
	this->addToPanel(STATE_SECTORCONTROL, button_elementstocks);
	xpos += step_x;*/

	//
	//this->button_factory = new ImageButton(68, 40, panel_factory, "view and alter current\nitem being manufactured");
	this->button_factory = new ImageButton(xpos, ypos, panel_factory, "view and alter current\nitem being manufactured");
	this->addToPanel(STATE_SECTORCONTROL, button_factory);
	this->button_nworkers = new PanelPage(xpos, ypos+16, 16, 16);
	this->button_nworkers->setInfoLMB("decrease the number of workers");
	this->button_nworkers->setInfoRMB("increase the number of workers");
	this->addToPanel(STATE_SECTORCONTROL, button_nworkers);
	xpos += step_x;

	xpos = start_x;
	ypos += step_y;
	//
	{
		for(int i=0;i<4;i++) {
			this->button_elements[i] = NULL;
			this->element_index[i] = -1;
		}
		int indx = 0;
		for(int i=0;i<Element::N_ID;i++) {
			if( indx == 4 )
				break;
			int n_elements = 0, fraction = 0;
			current_sector->getTotalElements(&n_elements, &fraction, i);
			if( n_elements > 0 || fraction > 0 ) {
				/*this->button_elements[indx] = new Button(8 + 24*indx, 90, icon_elements[i]);
				this->add(STATE_SECTORCONTROL, button_elements[indx]);*/
				this->element_index[indx] = i;
				/*this->button_nminers[indx] = new Button(8 + 24*indx, 98, 16, 24);
				this->add(STATE_SECTORCONTROL, button_nminers[indx]);*/
				indx++;
			}
		}
	}
	for(int i=0;i<4;i++) {
		if( this->element_index[i] == -1 ) {
			this->button_elements[i] = new PanelPage(0, 0, 0, 0);
			this->addToPanel(STATE_SECTORCONTROL, button_elements[i]);
			this->button_nminers[i] = new PanelPage(0, 0, 0, 0);
			this->addToPanel(STATE_SECTORCONTROL, button_nminers[i]);
			xpos += step_x;
			continue;
		}
		ASSERT_ELEMENT_ID( this->element_index[i] );
		//this->button_elements[i] = new ImageButton(8 + 24*i, 90, icon_elements[ this->element_index[i] ]);
		this->button_elements[i] = new ImageButton(xpos, ypos, icon_elements[ this->element_index[i] ], "view element stocks");
		//this->button_elements[i]->setInfoLMB("view element stocks");
		this->addToPanel(STATE_SECTORCONTROL, button_elements[i]);
		this->button_nminers[i] = new PanelPage(xpos, ypos+16, 16, 12);
		this->button_nminers[i]->setInfoLMB("decrease the number of miners");
		this->button_nminers[i]->setInfoRMB("increase the number of miners");
		this->addToPanel(STATE_SECTORCONTROL, button_nminers[i]);
		xpos += step_x;
	}
	xpos = start_x;
	ypos += step_y;

	//
	for(int i=0;i<Building::N_BUILDINGS;i++) {
		this->button_build[i] = NULL;
		this->button_nbuilders[i] = NULL;
	}

	//this->button_build[Building::BUILDING_MINE] = new ImageButton(8, 118, panel_build[Building::BUILDING_MINE]);
	this->button_build[Building::BUILDING_MINE] = new ImageButton(xpos, ypos, panel_build[Building::BUILDING_MINE]);
	this->button_build[Building::BUILDING_MINE]->setInfoLMB("change the number of builders");
	this->addToPanel(STATE_SECTORCONTROL, button_build[Building::BUILDING_MINE]);
	//this->button_nbuilders[Building::BUILDING_MINE] = new PanelPage(8, 134, 16, 12);
	this->button_nbuilders[Building::BUILDING_MINE] = new PanelPage(xpos, ypos+16, 16, 12);
	this->button_nbuilders[Building::BUILDING_MINE]->setInfoLMB("decrease the number of builders\nbuilding a mine");
	this->button_nbuilders[Building::BUILDING_MINE]->setInfoRMB("increase the number of builders\nbuilding a mine");
	this->addToPanel(STATE_SECTORCONTROL, button_nbuilders[Building::BUILDING_MINE]);
	xpos += step_x;

	//this->button_build[Building::BUILDING_FACTORY] = new ImageButton(28, 118, panel_build[Building::BUILDING_FACTORY]);
	this->button_build[Building::BUILDING_FACTORY] = new ImageButton(xpos, ypos, panel_build[Building::BUILDING_FACTORY]);
	this->button_build[Building::BUILDING_FACTORY]->setInfoLMB("change the number of builders");
	this->addToPanel(STATE_SECTORCONTROL, button_build[Building::BUILDING_FACTORY]);
	//this->button_nbuilders[Building::BUILDING_FACTORY] = new PanelPage(28, 134, 16, 12);
	this->button_nbuilders[Building::BUILDING_FACTORY] = new PanelPage(xpos, ypos+16, 16, 12);
	this->button_nbuilders[Building::BUILDING_FACTORY]->setInfoLMB("decrease the number of builders\nbuilding a factory");
	this->button_nbuilders[Building::BUILDING_FACTORY]->setInfoRMB("increase the number of builders\nbuilding a factory");
	this->addToPanel(STATE_SECTORCONTROL, button_nbuilders[Building::BUILDING_FACTORY]);
	xpos += step_x;

	//this->button_build[Building::BUILDING_LAB] = new ImageButton(48, 118, panel_build[Building::BUILDING_LAB]);
	this->button_build[Building::BUILDING_LAB] = new ImageButton(xpos, ypos, panel_build[Building::BUILDING_LAB]);
	this->button_build[Building::BUILDING_LAB]->setInfoLMB("change the number of builders");
	this->addToPanel(STATE_SECTORCONTROL, button_build[Building::BUILDING_LAB]);
	//this->button_nbuilders[Building::BUILDING_LAB] = new PanelPage(48, 134, 16, 12);
	this->button_nbuilders[Building::BUILDING_LAB] = new PanelPage(xpos, ypos+16, 16, 12);
	this->button_nbuilders[Building::BUILDING_LAB]->setInfoLMB("decrease the number of builders\nbuilding a lab");
	this->button_nbuilders[Building::BUILDING_LAB]->setInfoRMB("increase the number of builders\nbuilding a lab");
	this->addToPanel(STATE_SECTORCONTROL, button_nbuilders[Building::BUILDING_LAB]);
	xpos += step_x;

	// SHIELD
	this->button_bigshield = new ImageButton(33, 0, panel_bigshield, "return to main screen");
	this->addToPanel(STATE_SHIELD, button_bigshield);
	for(int i=0;i<n_sub_epochs;i++) {
		this->button_deploy_shields[i] = new ImageButton(28 + 22*i, 24, icon_shields[i]);
		this->button_deploy_shields[i]->setInfoLMB("select a shield to use");
		this->addToPanel(STATE_SHIELD, button_deploy_shields[i]);
	}
	this->button_shutdown = new ImageButton(8, 40, men[n_epochs_c-1], "get ready to\nshutdown the sector");
	this->addToPanel(STATE_SHIELD, button_shutdown);

	// DEFENCE
	this->button_bigdefence = new ImageButton(33, 0, panel_bigdefence, "return to main screen");
	this->addToPanel(STATE_DEFENCE, button_bigdefence);
	for(int i=0;i<n_sub_epochs;i++) {
		this->button_deploy_defences[i] = new ImageButton(28 + 22*i, 24, numbered_defences[start_epoch + i]);
		sprintf(buffer, "deploy a %s", Invention::getInvention(Invention::DEFENCE, start_epoch + i)->getName());
		this->button_deploy_defences[i]->setInfoLMB(buffer);
		this->addToPanel(STATE_DEFENCE, button_deploy_defences[i]);
	}

	// ATTACK
	this->button_bigattack = new ImageButton(33, 0, panel_bigattack, "return to main screen");
	this->addToPanel(STATE_ATTACK, button_bigattack);
	this->button_deploy_unarmedmen = new ImageButton(offset_attack_x_c, 24, 16, 28, unarmed_man);
	//this->button_deploy_unarmedmen = new ImageButton(offset_attack_x_c, 24, 16, 28, men[current_sector->getBuildingEpoch()]);
	this->button_deploy_unarmedmen->setInfoLMB("add an unarmed man to the army");
	this->addToPanel(STATE_ATTACK, button_deploy_unarmedmen);
	for(int i=0;i<n_sub_epochs;i++) {
		this->button_deploy_attackers[i] = new ImageButton(offset_attack_x_c + space_attack_x_c*i, 56, 16, 28, numbered_weapons[start_epoch + i]);
		sprintf(buffer, "add a %s to the army", Invention::getInvention(Invention::WEAPON, start_epoch + i)->getName());
		this->button_deploy_attackers[i]->setInfoLMB(buffer);
		this->addToPanel(STATE_ATTACK, button_deploy_attackers[i]);
	}
	this->button_return_attackers = new ImageButton(80, 96, 16, 28, panel_twoattack);
	this->button_return_attackers->setInfoLMB("return the assembled army");
	this->addToPanel(STATE_ATTACK, button_return_attackers);

	// DESIGN
	this->button_bigdesign = new ImageButton(33, 0, panel_bigdesign, "return to main screen");
	this->addToPanel(STATE_DESIGN, button_bigdesign);
	this->button_designers = new ImageButton(40, 16, 16, 28, men[current_sector->getBuildingEpoch()]);
	this->button_designers->setInfoLMB("decrease the number of designers");
	this->button_designers->setInfoRMB("increase the number of designers");
	this->addToPanel(STATE_DESIGN, button_designers);
	for(int i=0;i<n_sub_epochs;i++) {
		this->button_shields[i] = new ImageButton(8, 60 + i*16, icon_shields[i]);
		this->button_shields[i]->setInfoLMB("design a shield");
		this->addToPanel(STATE_DESIGN, button_shields[i]);
		this->button_defences[i] = new ImageButton(40, 60 + i*16, icon_defences[start_epoch + i]);
		sprintf(buffer, "design a %s", Invention::getInvention(Invention::DEFENCE, start_epoch + i)->getName());
		this->button_defences[i]->setInfoLMB(buffer);
		this->addToPanel(STATE_DESIGN, button_defences[i]);
		this->button_weapons[i] = new ImageButton(72, 60 + i*16, icon_weapons[start_epoch + i]);
		sprintf(buffer, "design a %s", Invention::getInvention(Invention::WEAPON, start_epoch + i)->getName());
		this->button_weapons[i]->setInfoLMB(buffer);
		this->addToPanel(STATE_DESIGN, button_weapons[i]);
	}

	// ELEMENTSTOCKS
	this->button_bigelementstocks = new ImageButton(33, 0, mine_gatherable_large, "return to main screen");
	this->addToPanel(STATE_ELEMENTSTOCKS, button_bigelementstocks);
	for(int i=0;i<4;i++) {
		if( this->element_index[i] == -1 ) {
			this->button_elements2[i] = new PanelPage(0, 0, 0, 0);
			this->addToPanel(STATE_SECTORCONTROL, button_elements2[i]);
			this->button_nminers2[i] = new PanelPage(0, 0, 0, 0);
			this->addToPanel(STATE_SECTORCONTROL, button_nminers2[i]);
			continue;
		}
		ASSERT_ELEMENT_ID( this->element_index[i] );
		this->button_elements2[i] = new ImageButton(64, 20 + 28*i, 16, 24, icon_elements[ this->element_index[i] ]);
		this->addToPanel(STATE_ELEMENTSTOCKS, button_elements2[i]);
		this->button_nminers2[i] = new PanelPage(40, 20 + 28*i, 16, 16);
		this->button_nminers2[i]->setInfoLMB("decrease the number of miners");
		this->button_nminers2[i]->setInfoRMB("increase the number of miners");
		this->addToPanel(STATE_ELEMENTSTOCKS, button_nminers2[i]);
	}

	// BUILD
	this->button_bigbuild = new ImageButton(33, 0, panel_bigbuild, "return to main screen");
	this->addToPanel(STATE_BUILD, button_bigbuild);
	for(int i=0;i<Building::N_BUILDINGS;i++)
		this->button_nbuilders2[i] = NULL;
	this->button_nbuilders2[Building::BUILDING_MINE] = new ImageButton(40, 20, 19, 28, panel_build[Building::BUILDING_MINE]);
	this->button_nbuilders2[Building::BUILDING_MINE]->setInfoLMB("decrease the number of builders\nbuilding a mine");
	this->button_nbuilders2[Building::BUILDING_MINE]->setInfoRMB("increase the number of builders\nbuilding a mine");
	this->addToPanel(STATE_BUILD, button_nbuilders2[Building::BUILDING_MINE]);
	this->button_nbuilders2[Building::BUILDING_FACTORY] = new ImageButton(40, 52, 19, 28, panel_build[Building::BUILDING_FACTORY]);
	this->button_nbuilders2[Building::BUILDING_FACTORY]->setInfoLMB("decrease the number of builders\nbuilding a factory");
	this->button_nbuilders2[Building::BUILDING_FACTORY]->setInfoRMB("increase the number of builders\nbuilding a factory");
	this->addToPanel(STATE_BUILD, button_nbuilders2[Building::BUILDING_FACTORY]);
	this->button_nbuilders2[Building::BUILDING_LAB] = new ImageButton(40, 84, 19, 28, panel_build[Building::BUILDING_LAB]);
	this->button_nbuilders2[Building::BUILDING_LAB]->setInfoLMB("decrease the number of builders\nbuilding a lab");
	this->button_nbuilders2[Building::BUILDING_LAB]->setInfoRMB("increase the number of builders\nbuilding a lab");
	this->addToPanel(STATE_BUILD, button_nbuilders2[Building::BUILDING_LAB]);

	// KNOWNDESIGNS
	this->button_bigknowndesigns = new ImageButton(33, 0, panel_bigknowndesigns, "return to main screen");
	this->addToPanel(STATE_KNOWNDESIGNS, button_bigknowndesigns);
	for(int i=0;i<n_sub_epochs;i++) {
		this->button_knownshields[i] = new ImageButton(8, 40 + i*16, icon_shields[i]);
		this->button_knownshields[i]->setInfoLMB("view blueprint for a shield");
		this->addToPanel(STATE_KNOWNDESIGNS, button_knownshields[i]);
		this->button_knowndefences[i] = new ImageButton(40, 40 + i*16, icon_defences[start_epoch + i]);
		sprintf(buffer, "view blueprint for a %s", Invention::getInvention(Invention::DEFENCE, start_epoch + i)->getName());
		this->button_knowndefences[i]->setInfoLMB(buffer);
		this->addToPanel(STATE_KNOWNDESIGNS, button_knowndefences[i]);
		this->button_knownweapons[i] = new ImageButton(72, 40 + i*16, icon_weapons[start_epoch + i]);
		sprintf(buffer, "view blueprint for a %s", Invention::getInvention(Invention::WEAPON, start_epoch + i)->getName());
		this->button_knownweapons[i]->setInfoLMB(buffer);
		this->addToPanel(STATE_KNOWNDESIGNS, button_knownweapons[i]);
	}

	// DESIGNINFO
	this->button_bigdesigninfo = new ImageButton(33, 0, panel_bigknowndesigns, "view blueprints\nof completed designs");
	this->addToPanel(STATE_DESIGNINFO, button_bigdesigninfo);
	this->button_trashdesign = new ImageButton(64, 100, icon_trash);
	button_trashdesign->setInfoBMB("trash this design");
	this->addToPanel(STATE_DESIGNINFO, button_trashdesign);

	// FACTORY
	this->button_bigfactory = new ImageButton(33, 0, panel_bigfactory, "return to main screen");
	this->addToPanel(STATE_FACTORY, button_bigfactory);
	this->button_workers = new ImageButton(40, 16, 16, 28, men[current_sector->getBuildingEpoch()]);
	this->button_workers->setInfoLMB("decrease the number of workers");
	this->button_workers->setInfoRMB("increase the number of workers");
	this->addToPanel(STATE_FACTORY, button_workers);
	this->button_famount = new ImageButton(40, 44, 16, 28, men[current_sector->getBuildingEpoch()]);
	this->button_famount->setInfoLMB("decrease the number to produce");
	this->button_famount->setInfoRMB("increase the number to produce");
	this->addToPanel(STATE_FACTORY, button_famount);
	for(int i=0;i<n_sub_epochs;i++) {
		this->button_fshields[i] = new ImageButton(8, 88 + i*16, icon_shields[i]);
		this->button_fshields[i]->setInfoLMB("manufacture a shield");
		this->addToPanel(STATE_FACTORY, button_fshields[i]);
		this->button_fdefences[i] = new ImageButton(40, 88 + i*16, icon_defences[start_epoch + i]);
		sprintf(buffer, "manufacture a %s", Invention::getInvention(Invention::DEFENCE, start_epoch + i)->getName());
		this->button_fdefences[i]->setInfoLMB(buffer);
		this->addToPanel(STATE_FACTORY, button_fdefences[i]);
		this->button_fweapons[i] = new ImageButton(72, 88 + i*16, icon_weapons[start_epoch + i]);
		sprintf(buffer, "manufacture a %s", Invention::getInvention(Invention::WEAPON, start_epoch + i)->getName());
		this->button_fweapons[i]->setInfoLMB(buffer);
		this->addToPanel(STATE_FACTORY, button_fweapons[i]);
	}

	//printf(">>> %d\n", this->pages[STATE_SECTORCONTROL]->nChildren());
	this->refresh();
	//printf(">>> %d\n", this->pages[STATE_SECTORCONTROL]->nChildren());
}

void GamePanel::refreshCanDesign() {
	if( current_sector->getActivePlayer() != human_player ) {
		return;
	}
	if( start_epoch == end_epoch_c ) {
		this->button_design->setEnabled(false);
		return;
	}
	bool any_design = false;
	if( current_sector->getCurrentDesign() != NULL ) {
		any_design = true; // even though no new designs available anymore, we can keep designing the selected design
	}
	for(int i=0;i<n_epochs_c && !any_design;i++) {
		if( current_sector->canResearch(Invention::SHIELD, i) )
			any_design = true;
		else if( current_sector->canResearch(Invention::DEFENCE, i) )
			any_design = true;
		else if( current_sector->canResearch(Invention::WEAPON, i) )
			any_design = true;
	}
	this->button_design->setEnabled(any_design);
	if( current_sector->getBuilding(Building::BUILDING_LAB) != NULL ) {
		this->button_design->setImage(panel_lab);
		this->button_bigdesign->setImage(panel_biglab);
	}
	else {
		//this->button_design->image = any_design ? panel_design : panel_design_dark;
		this->button_design->setImage(panel_design);
		this->button_bigdesign->setImage(panel_bigdesign);
	}
	this->button_ndesigners->setEnabled( current_sector->getCurrentDesign() != NULL );
}

void GamePanel::refreshDesignInventions() {
	if( current_sector->getActivePlayer() != human_player ) {
		return;
	}
	for(int i=0;i<n_sub_epochs;i++) {
		this->button_shields[i]->setEnabled(true);
		this->button_defences[i]->setEnabled(true);
		this->button_weapons[i]->setEnabled(true);

		if( !current_sector->canResearch(Invention::SHIELD, start_epoch + i) )
			this->button_shields[i]->setEnabled(false);
		if( !current_sector->canResearch(Invention::DEFENCE, start_epoch + i) )
			this->button_defences[i]->setEnabled(false);
		if( !current_sector->canResearch(Invention::WEAPON, start_epoch + i) )
			this->button_weapons[i]->setEnabled(false);
	}
}

void GamePanel::refreshManufactureInventions() {
	if( current_sector->getActivePlayer() != human_player ) {
		return;
	}
	for(int i=0;i<n_sub_epochs;i++) {
		this->button_fshields[i]->setEnabled(true);
		this->button_fdefences[i]->setEnabled(true);
		this->button_fweapons[i]->setEnabled(true);

		if( start_epoch + i < factory_epoch_c || !current_sector->canBuildDesign(Invention::SHIELD, start_epoch + i) )
			this->button_fshields[i]->setEnabled(false);
		if( start_epoch + i < factory_epoch_c || !current_sector->canBuildDesign(Invention::DEFENCE, start_epoch + i) )
			this->button_fdefences[i]->setEnabled(false);
		if( start_epoch + i < factory_epoch_c || !current_sector->canBuildDesign(Invention::WEAPON, start_epoch + i) )
			this->button_fweapons[i]->setEnabled(false);
	}
}

void GamePanel::refreshDeployInventions() {
	if( current_sector->getActivePlayer() != human_player ) {
		return;
	}
	for(int i=0;i<n_sub_epochs;i++) {
		this->button_deploy_shields[i]->setEnabled(false);
		if( current_sector->inventionKnown(Invention::SHIELD, start_epoch + i) )
			this->button_deploy_shields[i]->setEnabled(true);
	}

	for(int i=0;i<n_sub_epochs;i++) {
		this->button_deploy_defences[i]->setEnabled(false);
		if( current_sector->inventionKnown(Invention::DEFENCE, start_epoch + i) )
			this->button_deploy_defences[i]->setEnabled(true);
	}

	this->button_deploy_unarmedmen->setEnabled(true);
	for(int i=0;i<n_sub_epochs;i++) {
		this->button_deploy_attackers[i]->setEnabled(false);
		if( current_sector->inventionKnown(Invention::WEAPON, start_epoch + i) ||
			current_sector->getStoredArmy()->getSoldiers(start_epoch + i) > 0 )
			this->button_deploy_attackers[i]->setEnabled(true);
	}
}

void GamePanel::refreshShutdown() {
	if( current_sector->getActivePlayer() != human_player ) {
		return;
	}
	this->button_shutdown->setEnabled( current_sector->canShutdown() );
}

void GamePanel::refresh() {
	//LOG("GamePanel::refresh()\n");
	// set all enabled to false
	/*for(i=0;i<nChildren();i++) {
	for(int j=0;j<get(i)->nChildren();j++) {
	PanelPage *panel = get(i)->get(j);
	panel->setEnabled(false);
	}
	}*/
	if( current_sector->getActivePlayer() != human_player ) {
		this->setEnabled(false);
		return;
	}

	this->setEnabled(true);

	this->refreshCanDesign();

	bool any_defences = false;
	for(int i=0;i<n_epochs_c && !any_defences;i++)
		any_defences = ( current_sector->getStoredDefenders(i) > 0 || current_sector->inventionKnown(Invention::DEFENCE, i) );
	bool any_designs = ( current_sector->getNDesigns() > 0 );

	this->button_shield->setEnabled(true);
	this->button_defence->setEnabled(any_defences);
	this->button_attack->setEnabled(true);
	//this->button_elementstocks->setEnabled(start_epoch != end_epoch_c);
	for(int i=0;i<4;i++) {
		this->button_elements[i]->setEnabled(false);
		this->button_elements2[i]->setEnabled(false);
		this->button_nminers[i]->setEnabled(false);
		this->button_nminers2[i]->setEnabled(false);

		//if( this->element_index[i] != -1 && current_sector->canMine( (Element::Id)this->element_index[i] ) ) {
		if( this->element_index[i] != -1 ) {
			ASSERT_ELEMENT_ID( this->element_index[i] );
			bool can_mine = current_sector->canMine( (Element::Id)this->element_index[i] );
			int n_elements = 0, fraction = 0;
			//current_sector->getTotalElements(&n_elements, &fraction, this->element_index[i]);
			current_sector->getElementStocks(&n_elements, &fraction, (Element::Id)this->element_index[i]);
			if( n_elements > 0 || fraction > 0 || can_mine ) {
				this->button_elements[i]->setEnabled(true);
				this->button_elements2[i]->setEnabled(true);
			}
			/*current_sector->getElements(&n_elements, &fraction, this->element_index[i]);
			if( ( n_elements > 0 || fraction > 0 ) && elements[ this->element_index[i] ]->type != Element::GATHERABLE ) {*/
			if( can_mine && elements[ this->element_index[i] ]->getType() != Element::GATHERABLE ) {
				this->button_nminers[i]->setEnabled(true);
				this->button_nminers2[i]->setEnabled(true);
			}
		}
	}
	if( !current_sector->canBuild( Building::BUILDING_MINE ) ) {
		this->button_build[Building::BUILDING_MINE]->setEnabled(false);
		this->button_nbuilders[Building::BUILDING_MINE]->setEnabled(false);
		this->button_nbuilders2[Building::BUILDING_MINE]->setEnabled(false);
	}
	if( !current_sector->canBuild( Building::BUILDING_FACTORY ) ) {
		this->button_build[Building::BUILDING_FACTORY]->setEnabled(false);
		this->button_nbuilders[Building::BUILDING_FACTORY]->setEnabled(false);
		this->button_nbuilders2[Building::BUILDING_FACTORY]->setEnabled(false);
	}
	if( !current_sector->canBuild( Building::BUILDING_LAB ) ) {
		this->button_build[Building::BUILDING_LAB]->setEnabled(false);
		this->button_nbuilders[Building::BUILDING_LAB]->setEnabled(false);
		this->button_nbuilders2[Building::BUILDING_LAB]->setEnabled(false);
	}
	this->button_bigbuild->setEnabled(true);
	this->button_knowndesigns->setEnabled(any_designs);
	if( current_sector->getBuilding(Building::BUILDING_FACTORY) == NULL ) {
		this->button_factory->setEnabled(false);
	}
	this->button_nworkers->setEnabled( current_sector->getCurrentManufacture() != NULL );

	this->button_bigdesign->setEnabled(true);
	this->button_designers->setEnabled( current_sector->getCurrentDesign() != NULL );
	this->button_designers->enableHelpText( current_sector->getCurrentDesign() != NULL );
	this->refreshDesignInventions();

	this->button_bigshield->setEnabled(true);
	this->refreshShutdown();

	this->button_bigdefence->setEnabled(true);

	this->button_bigattack->setEnabled(true);
	this->button_return_attackers->setEnabled(true);

	this->refreshDeployInventions();

	this->button_bigelementstocks->setEnabled(true);

	this->button_bigknowndesigns->setEnabled(true);
	for(int i=0;i<n_sub_epochs;i++) {
		if( !current_sector->knownDesign(Invention::SHIELD, start_epoch + i) )
			this->button_knownshields[i]->setEnabled(false);
		if( !current_sector->knownDesign(Invention::DEFENCE, start_epoch + i) )
			this->button_knowndefences[i]->setEnabled(false);
		if( !current_sector->knownDesign(Invention::WEAPON, start_epoch + i) )
			this->button_knownweapons[i]->setEnabled(false);
	}

	this->button_bigdesigninfo->setEnabled(true);
	this->button_trashdesign->setEnabled(true);

	this->button_bigfactory->setEnabled(true);
	this->button_workers->setEnabled( current_sector->getCurrentManufacture() != NULL );
	this->button_workers->enableHelpText( current_sector->getCurrentManufacture() != NULL );
	this->button_famount->setEnabled( current_sector->getCurrentManufacture() != NULL );
	this->button_famount->enableHelpText( current_sector->getCurrentManufacture() != NULL );
	this->refreshManufactureInventions();

	//((PlayingGameState *)gamestate)->refreshButtons();
	gamestate->refreshButtons();
}

void GamePanel::setPage(int page) {
	this->c_page = page;
	this->setMouseState(MOUSESTATE_NORMAL);
}

GamePanel::MouseState GamePanel::getMouseState() const {
	return this->mousestate;
}

void GamePanel::setMouseState(MouseState mousestate) {
	this->mousestate = mousestate;
	//((PlayingGameState *)gamestate)->refreshButtons();
	gamestate->refreshButtons();
}

void GamePanel::draw() {
	if( this->c_page == STATE_SECTORCONTROL ) {
		//Image::write(offset_panel_x_c + 8, offset_panel_y_c + 6, letters_small, map->getName(), Image::JUSTIFY_LEFT, true, scale_width, scale_height);
		Image::writeMixedCase(offset_panel_x_c + 8, offset_panel_y_c - 1, letters_large, letters_small, NULL, map->getName(), Image::JUSTIFY_LEFT, true);
		if( start_epoch != end_epoch_c ) {
			int year = epoch_dates[current_sector->getEpoch()];
			//bool shiny = current_sector->getEpoch() == end_epoch_c;
			bool shiny = current_sector->getEpoch() == n_epochs_c-1;
			Image::writeNumbers(offset_panel_x_c + 80, offset_panel_y_c, shiny ? numbers_largeshiny : numbers_largegrey, abs(year),Image::JUSTIFY_RIGHT,true);
			Image *era = ( year < 0 ) ? icon_bc :
				shiny ? icon_ad_shiny : icon_ad;
			era->draw(offset_panel_x_c + 81, offset_panel_y_c + 7, true);
		}

		if( current_sector->getActivePlayer() == human_player ) {
			const int xpos = 8, ypos = 18;
			//const int xpos = 40, ypos = 40;
			men[current_sector->getBuildingEpoch()]->draw(offset_panel_x_c + xpos, offset_panel_y_c + ypos, true);
			Image::writeNumbers(offset_panel_x_c + xpos + 8, offset_panel_y_c + ypos + 18, numbers_grey, current_sector->getSparePopulation(),Image::JUSTIFY_CENTRE, true);
			/*{
				bool can_research = false;
				for(int i=0;i<n_epochs_c && !can_research;i++) {
					if( current_sector->canResearch(Invention::WEAPON, i) )
						can_research = true;
					else if( current_sector->canResearch(Invention::DEFENCE, i) )
						can_research = true;
					else if( current_sector->canResearch(Invention::SHIELD, i) )
						can_research = true;
				}
				if( can_research ) {
					SDL_Rect rect;
					rect.x = this->button_design->getLeft();
					rect.y = this->button_design->getTop();
					rect.w = this->button_design->getWidth();
					rect.h = this->button_design->getHeight();
					rect.x *= scale_width;
					rect.y *= scale_height;
					rect.w *= scale_width;
					rect.h *= scale_height;
					Uint32 color = SDL_MapRGB(screen->getSurface()->format, 255, 255, 255);
					SDL_FillRect(screen->getSurface(), &rect, color);
				}
			}*/
			if( current_sector->getCurrentDesign() != NULL ) {
				//Image::writeNumbers(offset_panel_x_c + 16, offset_panel_y_c + 58, numbers_white, current_sector->getDesigners(),Image::JUSTIFY_CENTRE,true,scale_width,scale_height);
				Image::writeNumbers( this->button_ndesigners->getXCentre(), this->button_ndesigners->getTop() + 2, numbers_white, current_sector->getDesigners(),Image::JUSTIFY_CENTRE,true);
			}
			for(int i=0;i<4;i++) {
				Element::Id element = (Element::Id)this->element_index[i];
				if( this->button_nminers[i]->isEnabled() ) {
					ASSERT_ELEMENT_ID( this->element_index[i] );
					int n_miners = current_sector->getMiners( element );
					Image::writeNumbers( this->button_nminers[i]->getXCentre(), this->button_nminers[i]->getTop() + 2, numbers_white, n_miners,Image::JUSTIFY_CENTRE,true);
				}
			}
			if( current_sector->getCurrentManufacture() != NULL ) {
				Image::writeNumbers( this->button_nworkers->getXCentre(), this->button_nworkers->getTop() + 2, numbers_white, current_sector->getWorkers(),Image::JUSTIFY_CENTRE,true);
			}
			for(int i=0;i<Building::N_BUILDINGS;i++) {
				if( this->button_nbuilders[i] != NULL && this->button_nbuilders[i]->isEnabled() ) {
					Image::writeNumbers( this->button_nbuilders[i]->getXCentre(), this->button_nbuilders[i]->getBottom() - 8, numbers_white, current_sector->getBuilders((Building::Type)i),Image::JUSTIFY_CENTRE,true);
				}
			}
		}
		else if( current_sector->getPlayer() == human_player ) {
			ASSERT( current_sector->isShutdown() );
			men[current_sector->getBuildingEpoch()]->draw(offset_panel_x_c + 40, offset_panel_y_c + 40, true);
			Image::writeNumbers(offset_panel_x_c + 48, offset_panel_y_c + 58, numbers_grey, current_sector->getPopulation(),Image::JUSTIFY_CENTRE, true);
		}
		else if( current_sector->getPlayer() == -1 ) {
			int n_players_in_sector = 0;
			int player_in_sector = -1;
			for(int i=0;i<n_players_c;i++) {
				if( current_sector->getArmy(i)->getTotal() > 0 ) {
					player_in_sector = i;
					n_players_in_sector++;
				}
			}
			if( n_players_in_sector == 1 && player_in_sector == human_player ) {
				int halfdays = 0, hours = 0;
				current_sector->buildingTowerTimeLeft(player_in_sector, &halfdays, &hours);
				int clock_index = hours + 1;
				icon_clocks[ clock_index ]->draw(offset_panel_x_c + 72, offset_panel_y_c + 16, true);
				Image::writeNumbers(offset_panel_x_c + 80, offset_panel_y_c + 34, numbers_white, halfdays,Image::JUSTIFY_CENTRE,true);
			}
		}
	}
	else if( this->c_page == STATE_DESIGN ) {
		this->button_designers->setImage( men[current_sector->getBuildingEpoch()] );
		if( current_sector->getCurrentDesign() == NULL ) {
			dash_grey->draw(offset_panel_x_c + 45, offset_panel_y_c + 36, true);
		}
		else {
			Invention *current_invention = current_sector->getCurrentDesign()->getInvention();
			current_invention->getImage()->draw(offset_panel_x_c + 8, offset_panel_y_c + 16, true);
			Image::writeNumbers(offset_panel_x_c + 48, offset_panel_y_c + 34, numbers_white, current_sector->getDesigners(),Image::JUSTIFY_CENTRE,true);
			if( current_sector->getDesigners() > 0 ) {
				int halfdays = 0, hours = 0;
				current_sector->inventionTimeLeft(&halfdays, &hours);
				int clock_index = hours + 1;
				icon_clocks[ clock_index ]->draw(offset_panel_x_c + 72, offset_panel_y_c + 16, true);
				Image::writeNumbers(offset_panel_x_c + 80, offset_panel_y_c + 34, numbers_white, halfdays,Image::JUSTIFY_CENTRE,true);
			}
			else {
				icon_clocks[ 0 ]->draw(offset_panel_x_c + 72, offset_panel_y_c + 16, true);
				icon_infinity->draw(offset_panel_x_c + 74, offset_panel_y_c + 32, true);
			}
		}
		icon_shield->draw(offset_panel_x_c + 8, offset_panel_y_c + 44, true);
		icon_defence->draw(offset_panel_x_c + 40, offset_panel_y_c + 44, true);
		icon_weapon->draw(offset_panel_x_c + 72, offset_panel_y_c + 44, true);
	}
	else if( this->c_page == STATE_SHIELD ) {
		//int col = SDL_MapRGB(screen->getSurface()->format, 0, 255, 0);
		int col = SDL_MapRGB(screen->getSurface()->format, 0, 0, 0);
		for(int i=0;i<Building::N_BUILDINGS;i++) {
			if( current_sector->getBuilding((Building::Type)i) != NULL ) {
				int width = building_health->getScaledWidth();
				int health = current_sector->getBuilding((Building::Type)i)->getHealth();
				int offx = offset_panel_x_c + 24;
				SDL_Rect rect;
				/*rect.x = (short)(scale_width * ( offset_panel_x_c + 24 ));
				rect.y = (short)(scale_height * ( offset_panel_y_c + 64 + 16 * i ));
				rect.w = (short)(( scale_width * health ) * 0.6);
				rect.h = (short)(scale_height * 4);*/
				rect.x = (short)(scale_width * offx + ( scale_width * health * width ) / 100.0);
				rect.y = (short)(scale_height * ( offset_panel_y_c + 64 + 16 * i ));
				rect.w = (short)ceil(( scale_width * (100 - health) * width ) / 100.0);
				rect.h = (short)(scale_height * 5);
				building_health->draw(offx, (int)(rect.y/scale_height), true);
				SDL_FillRect(screen->getSurface(), &rect, col);
				panel_building[i]->draw(offset_panel_x_c + 8, offset_panel_y_c + 64 - 5 + 16 * i, true);
			}
		}

		for(int i=0;i<n_sub_epochs;i++) {
			if( this->button_deploy_shields[i]->isEnabled() ) {
				int n_store = current_sector->getStoredShields(i);
				if( current_sector->canBuildDesign(Invention::SHIELD, start_epoch+i) || n_store > 0 ) {
					if( n_store == 0 && start_epoch + i < factory_epoch_c )
						Image::write(offset_panel_x_c + 36 + 22*i, offset_panel_y_c + 42, letters_small, "OK", Image::JUSTIFY_CENTRE, true);
					else
						Image::writeNumbers(offset_panel_x_c + 36 + 22*i, offset_panel_y_c + 42, numbers_yellow, n_store, Image::JUSTIFY_CENTRE, true);
				}
				else {
					dash_grey->draw(offset_panel_x_c + 32 + 22*i, offset_panel_y_c + 46, true);
				}
			}
		}
	}
	else if( this->c_page == STATE_DEFENCE ) {
		//unarmed_man->draw(offset_panel_x_c + 8, offset_panel_y_c + 24, true);
		men[current_sector->getBuildingEpoch()]->draw(offset_panel_x_c + 8, offset_panel_y_c + 24, true);
		Image::writeNumbers(offset_panel_x_c + 16, offset_panel_y_c + 42, numbers_grey, current_sector->getSparePopulation(), Image::JUSTIFY_CENTRE, true);
		for(int i=0;i<n_sub_epochs;i++) {
			if( this->button_deploy_defences[i]->isEnabled() ) {
				int n_store = current_sector->getStoredDefenders(start_epoch+i);
				if( current_sector->canBuildDesign(Invention::DEFENCE, start_epoch+i) || n_store > 0 ) {
					if( n_store == 0 && start_epoch + i < factory_epoch_c )
						Image::write(offset_panel_x_c + 36 + 22*i, offset_panel_y_c + 42, letters_small, "OK", Image::JUSTIFY_CENTRE, true);
					else
						Image::writeNumbers(offset_panel_x_c + 36 + 22*i, offset_panel_y_c + 42, numbers_yellow, n_store, Image::JUSTIFY_CENTRE, true);
				}
				else {
					dash_grey->draw(offset_panel_x_c + 32 + 22*i, offset_panel_y_c + 46, true);
				}
			}
		}
	}
	else if( this->c_page == STATE_ATTACK ) {
		Image::writeNumbers(offset_panel_x_c + offset_attack_x_c + 8, offset_panel_y_c + 42, numbers_yellow, current_sector->getSparePopulation(), Image::JUSTIFY_CENTRE, true);
		for(int i=0;i<n_sub_epochs;i++) {
			if( this->button_deploy_attackers[i]->isEnabled() ) {
				int n_store = current_sector->getStoredArmy()->getSoldiers(start_epoch+i);
				if( current_sector->canBuildDesign(Invention::WEAPON, start_epoch+i) || n_store > 0 ) {
					if( n_store == 0 && start_epoch + i < factory_epoch_c )
						Image::write(offset_panel_x_c + offset_attack_x_c + 8 + space_attack_x_c*i, offset_panel_y_c + 74, letters_small, "OK", Image::JUSTIFY_CENTRE, true);
					else
						Image::writeNumbers(offset_panel_x_c + offset_attack_x_c + 8 + space_attack_x_c*i, offset_panel_y_c + 74, numbers_yellow, n_store, Image::JUSTIFY_CENTRE, true);
				}
				else {
					dash_grey->draw(offset_panel_x_c + offset_attack_x_c + 4 + space_attack_x_c*i, offset_panel_y_c + 78, true);
				}
			}
		}

		int n_army = current_sector->getAssembledArmy()->getTotal();
		if( n_army == 0 )
			dash_grey->draw(offset_panel_x_c + 84, offset_panel_y_c + 116, true);
		else
			Image::writeNumbers(offset_panel_x_c + 88, offset_panel_y_c + 116, numbers_orange, n_army, Image::JUSTIFY_CENTRE, true);
	}
	else if( this->c_page == STATE_ELEMENTSTOCKS ) {
		for(int i=0;i<4;i++) {
			Element::Id element = (Element::Id)this->element_index[i];
			if( element == -1 )
				continue;
			ASSERT_ELEMENT_ID( this->element_index[i] );
			int n_elements = 0, fraction = 0;
			current_sector->getElementStocks(&n_elements, &fraction, element);
			int off = 0;
			if( n_elements > 0 ) {
				Image::writeNumbers(offset_panel_x_c + 72, offset_panel_y_c + 38 + 28 * i, numbers_blue, n_elements, Image::JUSTIFY_LEFT, true);
				off += 8 * n_digits(n_elements);
			}
			if( fraction == 1 ) {
				numbers_half->draw(offset_panel_x_c + 72 + off, offset_panel_y_c + 38 + 28 * i, true);
			}
			/*if( n_elements > 0 || fraction > 0 ) {
			Image::writeNumbers(offset_panel_x_c + 72, offset_panel_y_c + 38 + 28*i, numbers_grey, n_elements, Image::JUSTIFY_CENTRE, true);
			}*/
			/*current_sector->getElements(&n_elements, &fraction, element);
			if( ( n_elements > 0 || fraction > 0 ) && elements[element]->type != Element::GATHERABLE ) {*/
			if( this->button_nminers2[i]->isEnabled() ) {
				int n_miners = current_sector->getMiners( element );
				Image::writeNumbers( this->button_nminers2[i]->getXCentre(), this->button_nminers2[i]->getTop() + 2, numbers_white, n_miners,Image::JUSTIFY_CENTRE,true);
			}
		}
	}
	else if( this->c_page == STATE_BUILD ) {
		for(int i=0;i<Building::N_BUILDINGS;i++)
		{
			if( this->button_nbuilders2[i] != NULL && this->button_nbuilders2[i]->isEnabled() ) {
				Image::writeNumbers( this->button_nbuilders2[i]->getXCentre(), this->button_nbuilders2[i]->getBottom() - 8, numbers_white, current_sector->getBuilders((Building::Type)i),Image::JUSTIFY_CENTRE,true);
				if( current_sector->getBuilders((Building::Type)i) > 0 ) {
					int halfdays = 0, hours = 0;
					current_sector->buildingTimeLeft((Building::Type)i, &halfdays, &hours);
					int clock_index = hours + 1;
					icon_clocks[ clock_index ]->draw(offset_panel_x_c + 72, offset_panel_y_c + 16 + 32*(i-1), true);
					Image::writeNumbers(offset_panel_x_c + 80, offset_panel_y_c + 34 + 32*(i-1), numbers_white, halfdays,Image::JUSTIFY_CENTRE,true);
				}
				else {
					icon_clocks[ 0 ]->draw(offset_panel_x_c + 72, offset_panel_y_c + 16 + 32*(i-1), true);
					icon_infinity->draw(offset_panel_x_c + 74, offset_panel_y_c + 32 + 32*(i-1), true);
				}
			}
		}
		/*if( this->button_build2[Building::BUILDING_MINE]->isEnabled() ) {
		Image::writeNumbers( this->button_build2[Building::BUILDING_MINE]->getXCentre(), this->button_build2[Building::BUILDING_MINE]->getBottom() - 8, numbers_white, current_sector->getBuilders(Building::BUILDING_MINE),Image::JUSTIFY_CENTRE,true);
		if( current_sector->getBuilders(Building::BUILDING_MINE) > 0 ) {
		int halfdays = 0, hours = 0;
		current_sector->buildingTimeLeft(Building::BUILDING_MINE, &halfdays, &hours);
		int clock_index = hours + 1;
		icon_clocks[ clock_index ]->draw(offset_panel_x_c + 72, offset_panel_y_c + 16, true);
		Image::writeNumbers(offset_panel_x_c + 80, offset_panel_y_c + 34, numbers_white, halfdays,Image::JUSTIFY_CENTRE,true);
		}
		else {
		icon_clocks[ 0 ]->draw(offset_panel_x_c + 72, offset_panel_y_c + 16, true);
		icon_infinity->draw(offset_panel_x_c + 74, offset_panel_y_c + 32, true);
		}
		}*/
	}
	else if( this->c_page == STATE_KNOWNDESIGNS ) {
		icon_shield->draw(offset_panel_x_c + 8, offset_panel_y_c + 24, true);
		icon_defence->draw(offset_panel_x_c + 40, offset_panel_y_c + 24, true);
		icon_weapon->draw(offset_panel_x_c + 72, offset_panel_y_c + 24, true);
	}
	else if( this->c_page == STATE_DESIGNINFO ) {
		ASSERT(this->designinfo != NULL);
		Design *design = current_sector->knownDesign( this->designinfo->getType(), this->designinfo->getEpoch() );
		ASSERT(design != NULL);
		int cnt = 0;
		for(int i=0;i<Element::N_ID;i++) {
			int cost = design->getCost((Element::Id)i);
			if( cost > 0 ) {
				int whole = cost / element_multiplier_c;
				int frac = cost % element_multiplier_c;
				icon_elements[i]->draw(offset_panel_x_c + 16, offset_panel_y_c + 32 + 18 * cnt, true);
				int off = 0;
				if( whole > 0 ) {
					Image::writeNumbers(offset_panel_x_c + 36, offset_panel_y_c + 34 + 18 * cnt, numbers_blue, whole, Image::JUSTIFY_LEFT, true);
					off += 8 * n_digits(whole);
				}
				if( frac == 1 ) {
					numbers_half->draw(offset_panel_x_c + 36 + off, offset_panel_y_c + 34 + 18 * cnt, true);
				}
				cnt++;
			}
		}
		if( design->isErgonomicallyTerrific() ) {
			icon_ergo->draw(offset_panel_x_c + 64, offset_panel_y_c + 32, true);
		}
	}
	else if( this->c_page == STATE_FACTORY ) {
		this->button_workers->setImage( men[current_sector->getBuildingEpoch()] );
		//this->button_famount->image = men[current_sector->getBuildingEpoch()];
		if( current_sector->getCurrentManufacture() == NULL ) {
			dash_grey->draw(offset_panel_x_c + 45, offset_panel_y_c + 36, true);
			this->button_famount->setImage(NULL);
		}
		else {
			Invention *current_manufacture = current_sector->getCurrentManufacture()->getInvention();
			this->button_famount->setImage( current_manufacture->getImage() );
			//current_manufacture->getImage()->draw(offset_panel_x_c + 8, offset_panel_y_c + 16, true);
			Image::writeNumbers(offset_panel_x_c + 48, offset_panel_y_c + 34, numbers_white, current_sector->getWorkers(),Image::JUSTIFY_CENTRE,true);
			int famount = current_sector->getFAmount();
			int famount_x = offset_panel_x_c + 48;
			int famount_y = offset_panel_y_c + 62;
			if( famount == infinity_c )
				icon_infinity->draw(famount_x - icon_infinity->getScaledWidth()/2, famount_y - 2, true);
			else
				Image::writeNumbers(famount_x, famount_y, numbers_white, famount,Image::JUSTIFY_CENTRE,true);

			if( current_sector->getWorkers() > 0 ) {
				int halfdays = 0, hours = 0;
				int thalfdays = 0, thours = 0;
				current_sector->manufactureTimeLeft(&halfdays, &hours);
				current_sector->manufactureTotalTime(&thalfdays, &thours);
				icon_clocks[ hours+1 ]->draw(offset_panel_x_c + 72, offset_panel_y_c + 16, true);
				Image::writeNumbers(offset_panel_x_c + 80, offset_panel_y_c + 34, numbers_white, halfdays,Image::JUSTIFY_CENTRE,true);
				int amount = current_sector->getFAmount();
				if( amount == infinity_c ) {
					icon_clocks[ 0 ]->draw(offset_panel_x_c + 72, offset_panel_y_c + 44, true);
					icon_infinity->draw(offset_panel_x_c + 74, offset_panel_y_c + 60, true);
				}
				else {
					thours *= (amount-1);
					thalfdays *= (amount-1);
					thours += hours;
					thalfdays += halfdays;
					thalfdays += thours/12;
					thours = thours % 12;
					icon_clocks[ thours+1 ]->draw(offset_panel_x_c + 72, offset_panel_y_c + 44, true);
					Image::writeNumbers(offset_panel_x_c + 80, offset_panel_y_c + 62, numbers_white, thalfdays, Image::JUSTIFY_CENTRE, true);
				}
			}
			else {
				icon_clocks[ 0 ]->draw(offset_panel_x_c + 72, offset_panel_y_c + 16, true);
				icon_infinity->draw(offset_panel_x_c + 74, offset_panel_y_c + 32, true);
				icon_clocks[ 0 ]->draw(offset_panel_x_c + 72, offset_panel_y_c + 44, true);
				icon_infinity->draw(offset_panel_x_c + 74, offset_panel_y_c + 60, true);
			}
		}
		icon_shield->draw(offset_panel_x_c + 8, offset_panel_y_c + 72, true);
		icon_defence->draw(offset_panel_x_c + 40, offset_panel_y_c + 72, true);
		icon_weapon->draw(offset_panel_x_c + 72, offset_panel_y_c + 72, true);
	}

	MultiPanel::draw();
}

void GamePanel::changeMiners(Element::Id element,bool decrease,int n) {
	ASSERT(element >= 0 && element < Element::N_ID);
	ASSERT( ::elements[element]->getType() != Element::GATHERABLE );
	int n_miners = current_sector->getMiners( element );
	int n_spare = current_sector->getAvailablePopulation();
	if( decrease && n_miners > 0 ) {
		n_miners -= n;
		if( n_miners < 0 )
			n_miners = 0;
		current_sector->setMiners( element, n_miners );
	}
	else if( !decrease && n_spare > 0 ) {
		if( n > n_spare )
			n = n_spare;
		n_miners += n;
		current_sector->setMiners( element, n_miners );
	}
}

void GamePanel::input(int m_x,int m_y,Uint8 m_b) {
	MultiPanel::input(m_x, m_y, m_b);
	bool m_left = mouse_left(m_b);
	bool m_right = mouse_right(m_b);

	if( this->c_page == STATE_SECTORCONTROL ) {
		if( m_left && this->button_design->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_DESIGN);
			registerClick();
		}
		else if( m_left && this->button_shield->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_SHIELD);
			registerClick();
		}
		else if( m_left && this->button_defence->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_DEFENCE);
			registerClick();
		}
		else if( m_left && this->button_attack->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_ATTACK);
			registerClick();
		}
		/*else if( m_left && this->button_elementstocks->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_ELEMENTSTOCKS);
			registerClick();
		}*/
		else if( ( m_left || m_right ) && this->button_ndesigners->mouseOver(m_x,m_y) && current_sector->getCurrentDesign() != NULL ) {
			int n_designers = current_sector->getDesigners();
			int n_spare = current_sector->getAvailablePopulation();
			int n = getNClicks();
			if( m_left ) {
				n_designers -= n;
				if( n_designers < 0 )
					n_designers = 0;
				current_sector->setDesigners( n_designers );
			}
			else if( m_right ) {
				if( n > n_spare )
					n = n_spare;
				n_designers += n;
				current_sector->setDesigners( n_designers );
			}
		}
		else if( ( m_left || m_right ) && this->button_nworkers->mouseOver(m_x,m_y) && current_sector->getCurrentManufacture() != NULL ) {
			int n_workers = current_sector->getWorkers();
			int n_spare = current_sector->getAvailablePopulation();
			int n = getNClicks();
			if( m_left ) {
				n_workers -= n;
				if( n_workers < 0 )
					n_workers = 0;
				current_sector->setWorkers( n_workers );
			}
			else if( m_right ) {
				if( n > n_spare )
					n = n_spare;
				n_workers += n;
				current_sector->setWorkers( n_workers );
			}
		}
		else if( m_left && this->button_build[Building::BUILDING_MINE]->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_BUILD);
			registerClick();
		}
		else if( m_left && this->button_build[Building::BUILDING_FACTORY]->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_BUILD);
			registerClick();
		}
		else if( m_left && this->button_build[Building::BUILDING_LAB]->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_BUILD);
			registerClick();
		}
		else if( m_left && this->button_knowndesigns->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_KNOWNDESIGNS);
			registerClick();
		}
		else if( m_left && this->button_factory->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_FACTORY);
			registerClick();
		}

		for(int i=0;i<4 && ( m_left || m_right );i++) {
			Element::Id element = (Element::Id)this->element_index[i];
			if( m_left && this->button_elements[i]->mouseOver(m_x, m_y) ) {
				this->setPage(STATE_ELEMENTSTOCKS);
				registerClick();
			}
			else if( this->button_nminers[i]->mouseOver(m_x,m_y) ) {
				ASSERT_ELEMENT_ID( this->element_index[i] );
				int n = getNClicks();
				changeMiners(element, m_left, n);
			}
		}

		for(int i=0;i<Building::N_BUILDINGS && ( m_left || m_right );i++) {
			if( button_nbuilders[i] != NULL && this->button_nbuilders[i]->mouseOver(m_x,m_y) ) {
				int n_builders = current_sector->getBuilders((Building::Type)i);
				int n_spare = current_sector->getAvailablePopulation();
				int n = getNClicks();
				if( m_left && n_builders > 0 ) {
					n_builders -= n;
					if( n_builders < 0 )
						n_builders = 0;
					current_sector->setBuilders((Building::Type)i, n_builders);
				}
				else if( m_right && n_spare > 0 ) {
					if( n > n_spare )
						n = n_spare;
					n_builders += n;
					current_sector->setBuilders((Building::Type)i, n_builders);
				}
			}
		}
	}
	else if( this->c_page == STATE_DESIGN ) {
		if( m_left && this->button_bigdesign->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_SECTORCONTROL);
			registerClick();
		}
		for(int i=0;i<n_sub_epochs;i++) {
			if( m_left && this->button_shields[i]->mouseOver(m_x,m_y)
				&& ( current_sector->getCurrentDesign() == NULL
				|| current_sector->getCurrentDesign()->getInvention() != invention_shields[start_epoch + i] )
				) {
					Design *design = current_sector->bestDesign(Invention::SHIELD, start_epoch + i);
					current_sector->setCurrentDesign( design );
			}
			else if( m_left && this->button_defences[i]->mouseOver(m_x,m_y)
				&& ( current_sector->getCurrentDesign() == NULL
				|| current_sector->getCurrentDesign()->getInvention() != invention_defences[start_epoch + i] )
				) {
					Design *design = current_sector->bestDesign(Invention::DEFENCE, start_epoch + i);
					current_sector->setCurrentDesign( design );
			}
			else if( m_left && this->button_weapons[i]->mouseOver(m_x,m_y)
				&& ( current_sector->getCurrentDesign() == NULL
				|| current_sector->getCurrentDesign()->getInvention() != invention_weapons[start_epoch + i] )
				) {
					Design *design = current_sector->bestDesign(Invention::WEAPON, start_epoch + i);
					current_sector->setCurrentDesign( design );
			}
		}
		if( ( m_left || m_right ) && this->button_designers->mouseOver(m_x,m_y) ) {
			int n_designers = current_sector->getDesigners();
			int n_spare = current_sector->getAvailablePopulation();
			int n = getNClicks();
			if( m_left ) {
				n_designers -= n;
				if( n_designers < 0 )
					n_designers = 0;
				current_sector->setDesigners( n_designers );
			}
			else if( m_right ) {
				if( n > n_spare )
					n = n_spare;
				n_designers += n;
				current_sector->setDesigners( n_designers );
			}
		}
	}
	else if( this->c_page == STATE_SHIELD ) {
		if( m_left && this->button_bigshield->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_SECTORCONTROL);
			registerClick();
		}
		for(int i=0;i<n_sub_epochs;i++) {
			if( m_left && this->button_deploy_shields[i]->mouseOver(m_x,m_y) ) {
				if( mousestate == MOUSESTATE_DEPLOY_SHIELD && deploy_shield == i )
					setMouseState(MOUSESTATE_NORMAL);
				else {
					int n_store = current_sector->getStoredShields(i);
					if( n_store > 0 || ( start_epoch + i < factory_epoch_c && current_sector->canBuildDesign(Invention::SHIELD, start_epoch+i) ) ) {
						setMouseState(MOUSESTATE_DEPLOY_SHIELD);
						deploy_shield = i;
						ASSERT_EPOCH(deploy_shield);
					}
				}
			}
		}
		if( m_left && this->button_shutdown->mouseOver(m_x,m_y) ) {
			if( mousestate == MOUSESTATE_SHUTDOWN )
				setMouseState(MOUSESTATE_NORMAL);
			else
				setMouseState(MOUSESTATE_SHUTDOWN);
		}
	}
	else if( this->c_page == STATE_DEFENCE ) {
		if( m_left && this->button_bigdefence->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_SECTORCONTROL);
			registerClick();
		}
		for(int i=0;i<n_sub_epochs;i++) {
			if( m_left && this->button_deploy_defences[i]->mouseOver(m_x,m_y) ) {
				if( mousestate == MOUSESTATE_DEPLOY_DEFENCE && deploy_defence == start_epoch + i )
					setMouseState(MOUSESTATE_NORMAL);
				else {
					int n_store = current_sector->getStoredDefenders(start_epoch + i);
					if( n_store > 0 || ( start_epoch + i < factory_epoch_c && current_sector->canBuildDesign(Invention::DEFENCE, start_epoch+i) ) ) {
						setMouseState(MOUSESTATE_DEPLOY_DEFENCE);
						deploy_defence = start_epoch + i;
						ASSERT_EPOCH(deploy_defence);
					}
				}
			}
		}
	}
	else if( this->c_page == STATE_ATTACK ) {
		int n_nukes = 0;
		if( m_left ) {
			n_nukes = current_sector->getAssembledArmy()->getSoldiers(nuclear_epoch_c);
		}
		if( m_left && this->button_bigattack->mouseOver(m_x,m_y) ) {
			// return assembled army
			this->setPage(STATE_SECTORCONTROL);
			registerClick();
			current_sector->returnAssembledArmy();
			setMouseState(MOUSESTATE_NORMAL);
		}
		else if( m_left && this->button_return_attackers->mouseOver(m_x,m_y) ) {
			// return assembled army
			current_sector->returnAssembledArmy();
			setMouseState(MOUSESTATE_NORMAL);
		}
		else if( m_left && this->button_deploy_unarmedmen->mouseOver(m_x,m_y) && n_nukes == 0 ) {
			// add unarmed man to assembled army
			int n_population = current_sector->getPopulation();
			int n_spare = current_sector->getAvailablePopulation();
			int n = getNClicks();
			ASSERT(n >= 1);
			if( n_spare < n )
				n = 1;
			if( n_spare >= n ) {
				current_sector->getAssembledArmy()->add(n_epochs_c, n);
				current_sector->setPopulation( n_population - n );
				setMouseState(MOUSESTATE_DEPLOY_WEAPON);
			}
		}
		// add weapons to assembled army?
		for(int i=0;i<n_sub_epochs && m_left;i++) {
			bool ok = true;
			if( start_epoch + i == nuclear_epoch_c && n_nukes == 0 ) {
				int n_selected = current_sector->getAssembledArmy()->getTotal();
				if( n_selected > 0 )
					ok = false;
			}
			else if( n_nukes > 0 ) {
				ok = false;
			}
			if( ok && this->button_deploy_attackers[i]->mouseOver(m_x,m_y) ) {
				int n = getNClicks();
				if( current_sector->assembleArmy(start_epoch + i, n) ) {
					setMouseState(MOUSESTATE_DEPLOY_WEAPON);
				}
			}
		}
	}
	else if( this->c_page == STATE_ELEMENTSTOCKS ) {
		if( m_left && this->button_bigelementstocks->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_SECTORCONTROL);
			registerClick();
		}
		for(int i=0;i<4;i++) {
			Element::Id element = (Element::Id)this->element_index[i];
			/*int n_elements = 0, fraction = 0;
			current_sector->getElements(&n_elements, &fraction, element);
			if( ( n_elements > 0 || fraction > 0 ) && elements[element]->type != Element::GATHERABLE && ( m_left || m_right ) && this->button_nminers2[i]->mouseOver(m_x,m_y) ) {*/
			if( ( m_left || m_right ) && this->button_nminers2[i]->mouseOver(m_x,m_y) ) {
				ASSERT_ELEMENT_ID( this->element_index[i] );
				int n = getNClicks();
				changeMiners(element, m_left, n);
			}
		}
	}
	else if( this->c_page == STATE_BUILD ) {
		if( m_left && this->button_bigbuild->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_SECTORCONTROL);
			registerClick();
		}
		else if( m_left || m_right ) {
			for(int i=0;i<Building::N_BUILDINGS;i++) {
				if( button_nbuilders2[i] != NULL && this->button_nbuilders2[i]->mouseOver(m_x,m_y) ) {
					int n_builders = current_sector->getBuilders((Building::Type)i);
					int n_spare = current_sector->getAvailablePopulation();
					int n = getNClicks();
					if( m_left && n_builders > 0 ) {
						n_builders -= n;
						if( n_builders < 0 )
							n_builders = 0;
						current_sector->setBuilders((Building::Type)i, n_builders);
					}
					else if( m_right && n_spare > 0 ) {
						if( n > n_spare )
							n = n_spare;
						n_builders += n;
						current_sector->setBuilders((Building::Type)i, n_builders);
					}
				}
			}
		}
	}
	else if( this->c_page == STATE_KNOWNDESIGNS ) {
		if( m_left && this->button_bigknowndesigns->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_SECTORCONTROL);
			registerClick();
		}
		for(int i=0;i<n_sub_epochs;i++) {
			if( m_left && this->button_knownshields[i]->mouseOver(m_x,m_y) ) {
				this->setPage(STATE_DESIGNINFO);
				registerClick();
				this->designinfo = Invention::getInvention(Invention::SHIELD,start_epoch+i);
			}
			else if( m_left && this->button_knowndefences[i]->mouseOver(m_x,m_y) ) {
				this->setPage(STATE_DESIGNINFO);
				registerClick();
				this->designinfo = Invention::getInvention(Invention::DEFENCE,start_epoch+i);
			}
			else if( m_left && this->button_knownweapons[i]->mouseOver(m_x,m_y) ) {
				this->setPage(STATE_DESIGNINFO);
				registerClick();
				this->designinfo = Invention::getInvention(Invention::WEAPON,start_epoch+i);
			}
		}
	}
	else if( this->c_page == STATE_DESIGNINFO ) {
		if( m_left && this->button_bigdesigninfo->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_KNOWNDESIGNS);
			registerClick();
		}
		if( m_left && m_right && this->button_trashdesign->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_KNOWNDESIGNS);
			registerClick();
			ASSERT(this->designinfo != NULL);
			current_sector->trashDesign(this->designinfo);
		}
	}
	else if( this->c_page == STATE_FACTORY ) {
		if( m_left && this->button_bigfactory->mouseOver(m_x,m_y) ) {
			this->setPage(STATE_SECTORCONTROL);
			registerClick();
		}
		for(int i=0;i<n_sub_epochs;i++) {
			if( m_left && this->button_fshields[i]->mouseOver(m_x,m_y)
				&& ( current_sector->getCurrentManufacture() == NULL
				|| current_sector->getCurrentManufacture()->getInvention() != invention_shields[start_epoch + i] )
				) {
					Design *design = current_sector->knownDesign(Invention::SHIELD, start_epoch + i);
					current_sector->setCurrentManufacture( design );
			}
			else if( m_left && this->button_fdefences[i]->mouseOver(m_x,m_y)
				&& ( current_sector->getCurrentManufacture() == NULL
				|| current_sector->getCurrentManufacture()->getInvention() != invention_defences[start_epoch + i] )
				) {
					Design *design = current_sector->knownDesign(Invention::DEFENCE, start_epoch + i);
					current_sector->setCurrentManufacture( design );
			}
			else if( m_left && this->button_fweapons[i]->mouseOver(m_x,m_y)
				&& ( current_sector->getCurrentManufacture() == NULL
				|| current_sector->getCurrentManufacture()->getInvention() != invention_weapons[start_epoch + i] )
				) {
					Design *design = current_sector->knownDesign(Invention::WEAPON, start_epoch + i);
					current_sector->setCurrentManufacture( design );
			}
		}
		if( ( m_left || m_right ) && this->button_workers->mouseOver(m_x,m_y) ) {
			int n_workers = current_sector->getWorkers();
			int n_spare = current_sector->getAvailablePopulation();
			int n = getNClicks();
			if( m_left ) {
				n_workers -= n;
				if( n_workers < 0 )
					n_workers = 0;
				current_sector->setWorkers( n_workers );
			}
			else if( m_right ) {
				if( n > n_spare )
					n = n_spare;
				n_workers += n;
				current_sector->setWorkers( n_workers );
			}
		}
		else if( ( m_left || m_right ) && this->button_famount->mouseOver(m_x,m_y) ) {
			int famount = current_sector->getFAmount();
			if( m_left ) {
				if( famount > 1 )
					current_sector->setFAmount( famount - 1 );
			}
			else if( m_right && famount != infinity_c ) {
				current_sector->setFAmount( famount + 1 );
			}
		}
	}
}
