//---------------------------------------------------------------------------
#ifdef _WIN32
#define _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES 1
#endif

#ifdef __linux
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#else
#include <sdl.h>
#include <sdl_image.h>
#endif
//#include <FreeImage.h>
#include <cstring>
#include <cstdlib>
#include <cctype>
#include <cassert>

#include "utils.h"
#include "image.h"
//---------------------------------------------------------------------------

extern const bool DEBUG;
extern const int DEBUGLEVEL;

SDL_Surface *Image::dest_surf = NULL;

Image::Image() {
	//this->bitmap = NULL;
	this->data = NULL;
	this->need_to_free_data = NULL;
	this->surface = NULL;
	//this->userData = NULL;
	this->scale_x = 1;
	this->scale_y = 1;
	/*this->scale_x = 2;
	this->scale_y = 2;*/
	this->maskcol.r = 0;
	this->maskcol.g = 0;
	this->maskcol.b = 0;
}

Image::~Image() {
	free();
}

void Image::free() {
	if( this->surface != NULL ) {
		SDL_FreeSurface(this->surface);
		this->surface = NULL;
	}
	/*if( this->bitmap != NULL ) {
	FreeImage_Free(this->bitmap);
	this->bitmap = NULL;
	}*/
	if( need_to_free_data && this->data != NULL ) {
		delete [] this->data;
		this->data = NULL;
	}
}

void Image::draw(int x, int y,bool mask) const {
	//drawAlpha(x, y, 0.0);
	//return;
	/*x *= scale_x;
	y *= scale_y;*/
	x = (int)(x * scale_x);
	y = (int)(y * scale_y);
	if( mask ) {
		int index = SDL_MapRGB(this->surface->format, maskcol.r, maskcol.g, maskcol.b);
		SDL_SetColorKey(surface, SDL_SRCCOLORKEY, index);
	}
	SDL_Rect srcrect;
	srcrect.x = 0;
	srcrect.y = 0;
	srcrect.w = (short)this->getWidth();
	srcrect.h = (short)this->getHeight();
	SDL_Rect dstrect;
	dstrect.x = (short)x;
	dstrect.y = (short)y;
	dstrect.w = 0;
	dstrect.h = 0;
	SDL_BlitSurface(surface, &srcrect, dest_surf, &dstrect);
	if( mask )
		SDL_SetColorKey(surface, 0, 0);
}

/*void Image::drawAlpha(int x, int y,float alpha) {
x *= scale_x;
y *= scale_y;
//scale_w = scale_h = 1;
int w = this->getWidth();
int h = this->getHeight();
int d_w = dest_surf->w;
int d_h = dest_surf->h;
SDL_LockSurface(surface);
SDL_LockSurface(dest_surf);
int s_bpp = surface->format->BitsPerPixel;
int d_bpp = dest_surf->format->BitsPerPixel;
int s_bytespp = surface->format->BytesPerPixel;
int d_bytespp = dest_surf->format->BytesPerPixel;
for(int cy=0;cy<h;cy++) {
for(int cx=0;cx<w;cx++) {
int d_x = ( x + cx );
int d_y = ( y + cy );
if( d_x >= 0 && d_x < d_w && d_y >= 0 && d_y < d_h ) {
unsigned char *ptr = &((unsigned char *)surface->pixels)[ surface->pitch * cy + s_bytespp * cx ];
unsigned char *d_ptr = &((unsigned char *)dest_surf->pixels)[ dest_surf->pitch * d_y + d_bytespp * d_x ];
for(int i=0;i<s_bytespp;i++) {
int s_col = ptr[i];
int d_col = d_ptr[i];
//int n_col = alpha * s_col + ( 1.0 - alpha ) * d_col;
int n_col = d_col;
//d_ptr[i] = n_col;
}
}
}
}
SDL_UnlockSurface(surface);
SDL_UnlockSurface(dest_surf);
}*/

/*void Image::draw(int x, int y,bool mask,int scale_w, int scale_h) {
//if( false && scale_w == 1 && scale_h == 1 ) {
if( true ) {
this->draw(x,y,mask);
}
else {
x *= scale_x;
y *= scale_y;
scale_w = scale_h = 1;
int w = this->getWidth();
int h = this->getHeight();
int d_w = dest_surf->w;
int d_h = dest_surf->h;
SDL_LockSurface(surface);
SDL_LockSurface(dest_surf);
for(int cy=0;cy<h;cy++) {
for(int cx=0;cx<w;cx++) {
unsigned char c = ((unsigned char *)surface->pixels)[ surface->pitch * cy + cx ];
if( mask && c == 0 )
continue;
for(int j=0;j<scale_h;j++) {
for(int i=0;i<scale_w;i++) {
int d_x = ( x + cx ) * scale_w + i;
int d_y = ( y + cy ) * scale_h + j;
//int d_x = x * scale_w + cx + i;
//int d_y = y * scale_h + cy + j;
if( d_x >= 0 && d_x < d_w && d_y >= 0 && d_y < d_h )
((unsigned char *)dest_surf->pixels)[ dest_surf->pitch * d_y + d_x ] = c;
}
}
}
}
SDL_UnlockSurface(surface);
SDL_UnlockSurface(dest_surf);
}
}*/

/*void Image::getColor(int *r,int *g,int *b,int index) {
*r = this->palette[index].rgbRed;
*g = this->palette[index].rgbGreen;
*b = this->palette[index].rgbBlue;
}

void Image::getSDLColor(SDL_Color *color,int index) {
color->r = this->palette[index].rgbRed;
color->g = this->palette[index].rgbGreen;
color->b = this->palette[index].rgbBlue;
}*/

int Image::getWidth() const {
	return this->surface->w;
}

int Image::getHeight() const {
	return this->surface->h;
}

void Image::setScale(float scale_x,float scale_y) {
	this->scale_x = scale_x;
	this->scale_y = scale_y;
}

const SDL_Surface *Image::getSDLSurface() const {
	return this->surface;
}

SDL_Surface *Image::getSDLSurface() {
	return this->surface;
}

unsigned char Image::getPixelIndex(int x,int y) const {
	if( !isPaletted() )
		return 0;

	SDL_LockSurface(this->surface);
	unsigned char c = ((unsigned char *)this->surface->pixels)[ y * this->surface->pitch + x ];
	SDL_UnlockSurface(this->surface);
	return c;
}

bool Image::setPixelIndex(int x,int y,unsigned char c) {
	if( !isPaletted() )
		return false;


	SDL_LockSurface(this->surface);
	((unsigned char *)this->surface->pixels)[ y * this->surface->pitch + x ] = c;
	SDL_UnlockSurface(this->surface);
	return true;
}

bool Image::setColor(int index,unsigned char r,unsigned char g,unsigned char b) {
	if( !isPaletted() )
		return false;

	SDL_LockSurface(this->surface);
	SDL_Color *col = &this->surface->format->palette->colors[index];
	col->r = r;
	col->g = g;
	col->b = b;
	SDL_UnlockSurface(this->surface);
	return true;
}

bool Image::setMaskColor(unsigned char r,unsigned char g,unsigned char b) {
	this->maskcol.r = r;
	this->maskcol.g = g;
	this->maskcol.b = b;
	return true;
}

/*
* Return the pixel value at (x, y)
* NOTE: The surface must be locked before calling this!
*/
Uint32 getpixel(SDL_Surface *surface, int x, int y) {
	int bpp = surface->format->BytesPerPixel;
	/* Here p is the address to the pixel we want to retrieve */
	Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

	switch(bpp) {
	case 1:
		return *p;

	case 2:
		return *(Uint16 *)p;

	case 3:
		if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
			return p[0] << 16 | p[1] << 8 | p[2];
		else
			return p[0] | p[1] << 8 | p[2] << 16;

	case 4:
		return *(Uint32 *)p;

	default:
		return 0;       /* shouldn't happen, but avoids warnings */
	}
}

/*
* Set the pixel at (x, y) to the given value
* NOTE: The surface must be locked before calling this!
*/
void putpixel(SDL_Surface *surface, int x, int y, Uint32 pixel) {
	int bpp = surface->format->BytesPerPixel;
	/* Here p is the address to the pixel we want to set */
	Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

	switch(bpp) {
	case 1:
		*p = pixel;
		break;

	case 2:
		*(Uint16 *)p = pixel;
		break;

	case 3:
		if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
			p[0] = (pixel >> 16) & 0xff;
			p[1] = (pixel >> 8) & 0xff;
			p[2] = pixel & 0xff;
		} else {
			p[0] = pixel & 0xff;
			p[1] = (pixel >> 8) & 0xff;
			p[2] = (pixel >> 16) & 0xff;
		}
		break;

	case 4:
		*(Uint32 *)p = pixel;
		break;
	}
}

bool Image::createAlphaForColor(unsigned char ar, unsigned char ag, unsigned char ab, unsigned char alpha) {
	int cx, cy;
	int w = this->getWidth();
	int h = this->getHeight();
	//int bpp = this->surface->format->BitsPerPixel;
	bool set = false;
	SDL_LockSurface(this->surface);
	for(cx=0;cx<w && !set;cx++) {
		for(cy=0;cy<h && !set;cy++) {
			Uint32 pixel = getpixel(this->surface, cx, cy);
			Uint8 r, g, b;
			SDL_GetRGB(pixel, this->surface->format, &r, &g, &b);
			if( r == ar && g == ag && b == ab ) {
				set = true;
			}
		}
	}
	SDL_UnlockSurface(this->surface);
	if( !set ) {
		return false;
	}

	this->convertToHiColor(true);

	SDL_LockSurface(this->surface);
	for(cx=0;cx<w;cx++) {
		for(cy=0;cy<h;cy++) {
			Uint32 pixel = getpixel(this->surface, cx, cy);
			Uint8 r, g, b, a;
			SDL_GetRGB(pixel, this->surface->format, &r, &g, &b);
			a = 255;
			if( r == ar && g == ag && b == ab ) {
				r = 0;
				g = 0;
				b = 0;
				a = alpha;
			}
			else if( r == this->maskcol.r && g == this->maskcol.g && b == maskcol.b ) {
				r = 0;
				g = 0;
				b = 0;
				a = 0;
			}
			pixel = SDL_MapRGBA(this->surface->format, r, g, b, a);
			putpixel(this->surface, cx, cy, pixel);
		}
	}

	SDL_UnlockSurface(this->surface);
	return true;
}

bool Image::convertToHiColor(bool alpha) {
	// todo: repeated conversions don't seem to work? seems to be due to repeated blitting not working
	int bpp = this->surface->format->BitsPerPixel;
	if( bpp == 32 )
		return false;
	/*SDL_PixelFormat new_format = *this->surface->format;
	new_format.palette = NULL;
	new_format.BitsPerPixel = alpha ? 24 : 16;
	new_format.BytesPerPixel = alpha ? 4 : 3;*/
	Uint32 rmask, gmask, bmask, amask;
#if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
	rmask = 0xff000000;
	gmask = 0x00ff0000;
	bmask = 0x0000ff00;
	amask = 0x000000ff;
#else
	rmask = 0x000000ff;
	gmask = 0x0000ff00;
	bmask = 0x00ff0000;
	amask = 0xff000000;
#endif

	/*unsigned char bytes[4];
	int n_bytes = alpha ? 4 : 3;
	SDL_Surface *dummy = SDL_CreateRGBSurfaceFrom(bytes, 1, 1, n_bytes * 8, 1 * n_bytes, rmask, gmask, bmask, amask);

	SDL_Surface *new_surf = SDL_ConvertSurface(this->surface, dummy->format, NULL);
	free();
	this->surface = new_surf;
	this->need_to_free_data = false;*/
	int depth = alpha ? 32 : 24;
	SDL_Surface *new_surf = SDL_CreateRGBSurface(NULL, this->getWidth(), this->getHeight(), depth, rmask, gmask, bmask, amask);
	SDL_BlitSurface(this->surface, NULL, new_surf, NULL);
	free();
	this->surface = new_surf;
	this->need_to_free_data = false;

	return true;
}

/*void Image::convertToFormat(Image *image) {
	SDL_Surface *new_surf = SDL_ConvertSurface(this->surface, image->surface->format, NULL);
	free();
	this->surface = new_surf;
}*/

/*bool Image::convertToHiColor(bool alpha) {
int w = this->getWidth();
int h = this->getHeight();
int bpp = this->surface->format->BitsPerPixel;
if( bpp == 32 )
return false;
int bytesperpixel = this->surface->format->BytesPerPixel;
if( bytesperpixel == 4 )
return false;
SDL_LockSurface(this->surface);
int n_bytes = alpha ? 4 : 3;
unsigned char *new_data = new unsigned char[w * h * n_bytes];
for(int cx=0;cx<w;cx++) {
for(int cy=0;cy<h;cy++) {
Uint32 pixel = getpixel(this->surface, cx, cy);
Uint8 r, g, b;
SDL_GetRGB(pixel, this->surface->format, &r, &g, &b);
if( pixel != 0 ) {
printf("");
}
unsigned char *d_ptr = &new_data[ cy * w * n_bytes + cx * n_bytes ];
d_ptr[0] = r;
d_ptr[1] = g;
d_ptr[2] = b;
if( alpha )
d_ptr[3] = 255;
}
}
SDL_UnlockSurface(this->surface);
free();

Uint32 rmask, gmask, bmask, amask;
#if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
rmask = 0xff000000;
gmask = 0x00ff0000;
bmask = 0x0000ff00;
amask = 0x000000ff;
#else
rmask = 0x000000ff;
gmask = 0x0000ff00;
bmask = 0x00ff0000;
amask = 0xff000000;
#endif

this->surface = SDL_CreateRGBSurfaceFrom(new_data, w, h, n_bytes * 8, w * n_bytes, rmask, gmask, bmask, amask);
this->data = new_data;
//this->need_to_free_data = true;

return true;
}*/

void Image::convertToDisplayFormat() {
	SDL_Surface *new_surf = NULL;
	int bpp = this->surface->format->BitsPerPixel;
	if( bpp == 32 && this->surface->format->Amask != 0 )
		new_surf = SDL_DisplayFormatAlpha(this->surface);
	else
		new_surf = SDL_DisplayFormat(this->surface);
	SDL_FreeSurface(this->surface);
	this->surface = new_surf;
}

//Image *Image::createFromBitmap(FIBITMAP *bitmap, Uint32 rmask, Uint32 gmask, Uint32 bmask, Uint32 amask) {
//Image *image = new Image();
////image->bitmap = bitmap;
//int w = FreeImage_GetWidth(bitmap);
//int h = FreeImage_GetHeight(bitmap);
//int bpp = FreeImage_GetBPP(bitmap);
//unsigned char *bm_data = (unsigned char *)FreeImage_GetBits(bitmap);
////image->n_colours = FreeImage_GetColorsUsed(image->bitmap);
//int pitch = FreeImage_GetPitch(bitmap);
//image->data = new unsigned char[pitch * h];
//image->need_to_free_data = true;
//for(int i=0;i<h * pitch;i++)
//image->data[i] = bm_data[i];
//
///*Uint32 rmask, gmask, bmask, amask;
//#if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
//rmask = 0xff000000;
//gmask = 0x00ff0000;
//bmask = 0x0000ff00;
//amask = 0x000000ff;
//#else
//rmask = 0x000000ff;
//gmask = 0x0000ff00;
//bmask = 0x00ff0000;
//amask = 0xff000000;
//#endif*/
//image->surface = SDL_CreateRGBSurfaceFrom(image->data, w, h, bpp, pitch, rmask, gmask, bmask, amask);
///*RGBQUAD *palette = FreeImage_GetPalette(image->bitmap);
//for(int i=0;i<256;i++) {
//SDL_Color color = { palette[i].rgbRed, palette[i].rgbGreen, palette[i].rgbBlue };
//SDL_SetColors(image->surface, &color, i, 1);
//}*/
////FreeImage_Free(bitmap);
//return image;
//}

bool Image::copyPalette(const Image *image) {
	if( this->surface->format->palette == NULL || image->surface->format->palette == NULL )
		return false;

	if( this->surface->format->palette->ncolors != image->surface->format->palette->ncolors )
		return false;

	SDL_SetColors(this->surface, image->surface->format->palette->colors, 0, this->surface->format->palette->ncolors);
	return true;
}

// side-effect: also converts images with < 256 colours to have 256 colours
void Image::scale(float sx,float sy) {
	//this->scale_x *= sx;
	//this->scale_y *= sy;
	int w = this->getWidth();
	int h = this->getHeight();
	SDL_LockSurface(this->surface);
	unsigned char *src_data = (unsigned char *)this->surface->pixels;
	/*FIBITMAP *new_bitmap = FreeImage_Allocate(sx * w, sy*h, this->surface->format->BitsPerPixel);
	unsigned char *data = (unsigned char *)FreeImage_GetBits(new_bitmap);*/
	int bytesperpixel = this->surface->format->BytesPerPixel;
	unsigned char *new_data = new unsigned char[(int)(sx * w * sy * h * bytesperpixel)];
	for(int cx=0;cx<w;cx++) {
		for(int cy=0;cy<h;cy++) {
			for(int i=0;i<bytesperpixel;i++) {
				int src_indx = cy * this->surface->pitch + cx * bytesperpixel + i;
				//ASSERT(this->surface->pitch == w);
				//ASSERT(src_indx >= 0 && src_indx < w*h*bytesperpixel );
				ASSERT(src_indx >= 0 && src_indx < this->surface->pitch*h );
				unsigned char pt = src_data[ src_indx ];
				for(int x=0;x<sx;x++) {
					for(int y=0;y<sy;y++) {
						int dx = (int)(cx * sx + x);
						int dy = (int)(cy * sy + y);
						int dst_indx = (int)(dy * w * sx * bytesperpixel + dx * bytesperpixel + i);
						ASSERT(dst_indx >= 0 && dst_indx < sx*w*sy*h*bytesperpixel );
						//data[ dy * FreeImage_GetPitch(new_bitmap) + dx * bytesperpixel + i ] = pt;
						//new_data[ dy * w * sx /* * bytesperpixel + dx * bytesperpixel + i*/ ] = 0;
						new_data[ dst_indx ] = pt;
					}
				}
			}
		}
	}
	SDL_UnlockSurface(this->surface);

	{
		/*w *= sx;
		h *= sy;*/
		w = (int)(w * sx);
		h = (int)(h * sy);
		//int w = FreeImage_GetWidth(new_bitmap);
		//int h = FreeImage_GetHeight(new_bitmap);
		//int bpp = FreeImage_GetBPP(new_bitmap);
		int bpp = this->surface->format->BitsPerPixel;
		//unsigned char *data = (unsigned char *)FreeImage_GetBits(new_bitmap);
		//int pitch = FreeImage_GetPitch(new_bitmap);
		int pitch = this->surface->format->BytesPerPixel * w;

		/*Uint32 rmask, gmask, bmask;
		#if SDL_BYTEORDER == SDL_BIG_ENDIAN
		rmask = 0x00ff0000;
		gmask = 0x0000ff00;
		bmask = 0x000000ff;
		#else
		rmask = 0x000000ff;
		gmask = 0x0000ff00;
		bmask = 0x00ff0000;
		#endif*/
		Uint32 rmask = this->surface->format->Rmask;
		Uint32 gmask = this->surface->format->Gmask;
		Uint32 bmask = this->surface->format->Bmask;
		Uint32 amask = this->surface->format->Amask;
		SDL_Surface *new_surf = SDL_CreateRGBSurfaceFrom(new_data, w, h, bpp, pitch, rmask, gmask, bmask, amask);
		if( this->surface->format->palette != NULL ) {
			SDL_SetColors(new_surf, this->surface->format->palette->colors, 0, this->surface->format->palette->ncolors);

			/*SDL_Surface *new_surf2 = SDL_ConvertSurface(new_surf, this->surface->format, NULL);
			SDL_FreeSurface(new_surf);
			new_surf = new_surf2;*/
		}
		free();
		this->surface = new_surf;
		//this->bitmap = new_bitmap;
		this->data = new_data;
		this->need_to_free_data = true;
	}
}

bool Image::scaleTo(int n_w) {
	/*if( n_w == this->getWidth() ) {
		this->scale(1, 1); // hacky way of forcing to 8bit
		return true;
	}
	else if( n_w == this->getWidth()*2 ) {
		this->scale(2, 2);
		//this->setScale(2, 2);
		return true;
	}
	else if( n_w == this->getWidth()*4 ) {
		this->scale(4, 4);
		//this->setScale(4, 4);
		return true;
	}
	LOG("unexpected scale size: from %d , %d to width %d\n", this->getWidth(), this->getHeight(), n_w);
	return false;*/
	float scale_factor = ((float)n_w) / (float)this->getWidth();
	this->scale(scale_factor, scale_factor); // nb, still scale if scale_factor==1, as this is a way of converting to 8bit
	return true;
}

/*bool Image::scaleTo(int n_w,int n_h) {
	if( n_w == this->getWidth() && n_h == this->getHeight() ) {
		return true;
	}
	else if( n_w == this->getWidth()*2 && n_h == this->getHeight()*2 ) {
		this->scale(2, 2);
		this->setScale(2, 2);
		return true;
	}
	else if( n_w == this->getWidth()*4 && n_h == this->getHeight()*4 ) {
		this->scale(4, 4);
		this->setScale(4, 4);
		return true;
	}
	LOG("unexpected scale size: from %d , %d to %d , %d\n", this->getWidth(), this->getHeight(), n_w, n_h);
	return false;
}*/

void Image::smooth(bool mask) {
	if( this->surface->format->BitsPerPixel != 24 ) {
		return;
	}
	int x, y, i;
	int w = getWidth();
	int h = getHeight();
	unsigned char *src_data = (unsigned char *)this->surface->pixels;
	int bytesperpixel = this->surface->format->BytesPerPixel;
	int pitch = this->surface->pitch;
	unsigned char *new_data = new unsigned char[w * h * bytesperpixel];
	bool *maskbits = new bool[w*h];
	if( mask ) {
		i = 0;
		for(x=0;x<w;x++) {
			for(y=0;y<h;y++) {
				int indx = y * pitch + x * bytesperpixel;
				maskbits[i++] = ( src_data[indx] == maskcol.r && src_data[indx+1] == maskcol.g && src_data[indx+2] == maskcol.b );
			}
		}
	}

	SDL_LockSurface(this->surface);
	for(x=0;x<w;x++) {
		for(y=0;y<h;y++) {
			for(i=0;i<bytesperpixel;i++) {
				Uint32 col = 0;
				if( ( !mask || !maskbits[ x * h + y ] ) &&
					x > 0 && x < w-1 && y > 0 && y < h-1 ) {
						Uint32 sq[9];
						int indx = 0;
						for(int sx=x-1;sx<=x+1;sx++) {
							for(int sy=y-1;sy<=y+1;sy++) {
								if( mask && maskbits[ sx * h + sy ] )
									sq[indx++] = 0;
								else
									sq[indx++] = src_data[ sy * pitch + sx * bytesperpixel + i ];
							}
						}
						col = ( sq[0] + 2 * sq[1] + sq[2] + 2 * sq[3] + 4 * sq[4] + 2 * sq[5] + sq[6] + 2 * sq[7] + sq[8] ) / 16;
						//col = ( sq[1] + sq[3] + sq[5] + sq[7] + 12 * sq[4] ) / 16;
				}
				else
					col = src_data[ y * pitch + x * bytesperpixel + i ];
				new_data[ y * w * bytesperpixel + x * bytesperpixel + i] = (unsigned char)col;
			}
		}
	}
	for(x=0;x<w;x++) {
		for(y=0;y<h;y++) {
			for(i=0;i<bytesperpixel;i++) {
				src_data[ y * pitch + x * bytesperpixel + i ] = new_data[ y * w * bytesperpixel + x * bytesperpixel + i];
			}
		}
	}
	delete [] new_data;
	delete [] maskbits;
	SDL_UnlockSurface(this->surface);
}

void Image::remap(unsigned char sr,unsigned char sg,unsigned char sb,unsigned char rr,unsigned char rg,unsigned char rb) {
	if( this->surface->format->BitsPerPixel != 24 && this->surface->format->BitsPerPixel != 32 ) {
		return;
	}
	SDL_LockSurface(this->surface);
	int w = getWidth();
	int h = getHeight();
	int bytesperpixel = this->surface->format->BytesPerPixel;
	int pitch = this->surface->pitch;
	unsigned char *src_data = (unsigned char *)this->surface->pixels;
	for(int x=0;x<w;x++) {
		for(int y=0;y<h;y++) {
			unsigned char *ptr = &src_data[ y * pitch + x * bytesperpixel ];
			if( ptr[0] == sr && ptr[1] == sg && ptr[2] == sb ) {
				ptr[0] = rr;
				ptr[1] = rg;
				ptr[2] = rb;
			}
		}
	}

	SDL_UnlockSurface(this->surface);
}

//Image *Image::copy(int x,int y,int w,int h) {
//LOG("Image::copy(%d,%d,%d,%d)\n",x,y,w,h);
//x *= scale_x;
//y *= scale_y;
//w *= scale_x;
//h *= scale_y;
//FIBITMAP *new_bitmap = FreeImage_Allocate(w, h, this->surface->format->BitsPerPixel);
//SDL_LockSurface(this->surface);
//unsigned char *src_data = (unsigned char *)this->surface->pixels;
//unsigned char *data = (unsigned char *)FreeImage_GetBits(new_bitmap);
//int bytesperpixel = this->surface->format->BytesPerPixel;
//for(int cx=0;cx<w;cx++) {
//for(int cy=0;cy<h;cy++) {
//for(int i=0;i<bytesperpixel;i++) {
//data[ cy * FreeImage_GetPitch(new_bitmap) + cx * bytesperpixel + i ] = src_data[ ( y + cy ) * this->surface->pitch + ( x + cx ) * bytesperpixel + i ];
//}
//}
//}
//SDL_UnlockSurface(this->surface);
//Image *copy_image = createFromBitmap(new_bitmap, this->surface->format->Rmask, this->surface->format->Gmask, this->surface->format->Bmask, this->surface->format->Amask);
//copy_image->copyPalette(this);
//copy_image->scale_x = scale_x;
//copy_image->scale_y = scale_y;
//return copy_image;
//}

void Image::reshadeRGB(int from, bool to_r, bool to_g, bool to_b) {
	ASSERT(from >= 0 && from < 3);
	if( this->surface->format->BitsPerPixel != 24 && this->surface->format->BitsPerPixel != 32 ) {
		return;
	}
	bool to[3] = {to_r, to_g, to_b};
	SDL_LockSurface(this->surface);
	int w = getWidth();
	int h = getHeight();
	int bytesperpixel = this->surface->format->BytesPerPixel;
	int pitch = this->surface->pitch;
	unsigned char *src_data = (unsigned char *)this->surface->pixels;
	for(int x=0;x<w;x++) {
		for(int y=0;y<h;y++) {
			unsigned char *ptr = &src_data[ y * pitch + x * bytesperpixel ];
			int val = ptr[from];
			int t_diff = 0;
			int n = 0;
			for(int j=0;j<3;j++) {
				if( to[j] && j != from ) {
					int val2 = ptr[j];
					int diff = val2 - val;
					t_diff += diff;
					n++;
					ptr[j] = val;
				}
			}
			if( n > 0 && !to[from] ) {
				t_diff /= n;
				val += t_diff;
				ASSERT(val >=0 && val < 256);
				ptr[from] = val;
			}
		}
	}

	SDL_UnlockSurface(this->surface);
}

void Image::brighten(float sr, float sg, float sb,bool mask) {
	if( this->surface->format->BitsPerPixel != 24 && this->surface->format->BitsPerPixel != 32 ) {
		return;
	}
	float scale[3] = {sr, sg, sb};
	SDL_LockSurface(this->surface);
	int w = getWidth();
	int h = getHeight();
	int bytesperpixel = this->surface->format->BytesPerPixel;
	int pitch = this->surface->pitch;
	unsigned char *src_data = (unsigned char *)this->surface->pixels;
	for(int x=0;x<w;x++) {
		for(int y=0;y<h;y++) {
			unsigned char *ptr = &src_data[ y * pitch + x * bytesperpixel ];
			if( mask && ptr[0] == maskcol.r && ptr[1] == maskcol.g && ptr[2] == maskcol.b ) {
				continue;
			}
			for(int j=0;j<3;j++) {
				float col = (float)ptr[j];
				col *= scale[j];
				if( col < 0 )
					col = 0;
				else if( col > 255 )
					col = 255;
				ptr[j] = (unsigned char)col;
			}
		}
	}
	SDL_UnlockSurface(this->surface);
}

void Image::print(const char *inStr) const {
	printf("\n--------------------\nImage %s\n",inStr);
	printf("scale_x %d, scale_y %d\n",this->scale_x, this->scale_y);
	printf("maskcol.r %d, maskcol.g %d, maskcol.b %d\n",this->maskcol.r,this->maskcol.g, this->maskcol.b);
	printf("format %d\n",this->surface->format);
	int w = this->surface->w;
	int h = this->surface->h;
	SDL_LockSurface(this->surface);
	unsigned char *src_data = (unsigned char *)this->surface->pixels;
	int bytesperpixel = this->surface->format->BytesPerPixel;
	for(int cx=0;cx<w;cx++) {
		for(int cy=0;cy<h;cy++) {
			for(int i=0;i<bytesperpixel;i++) {
				printf("%02x ", src_data[ ( cy ) * this->surface->pitch + ( cx ) * bytesperpixel + i ]);
			}
		}
		printf("\n");
	}
	SDL_UnlockSurface(this->surface);
}

void Image::flipX() {
	LOG("Image::flipX()\n");

	int w = this->getWidth();
	int h = this->getHeight();
	unsigned char *data = (unsigned char *)this->surface->pixels;
	SDL_LockSurface(this->surface);
	int bytesperpixel = this->surface->format->BytesPerPixel;
	for(int cx=0;cx<w/2;cx++) {
		int f_cx = w - 1 - cx;
		for(int cy=0;cy<h;cy++) {
			for(int i=0;i<bytesperpixel;i++) {
				unsigned char dummy = data[ cy * this->surface->pitch + cx * bytesperpixel + i ];
				data[ cy * this->surface->pitch + cx * bytesperpixel + i ] = data[ cy * this->surface->pitch + f_cx * bytesperpixel + i ];
				data[ cy * this->surface->pitch + f_cx * bytesperpixel + i ] = dummy;
				//dst_data[ cy * copy_image->surface->pitch + cx * bytesperpixel + i ] = src_data[ ( y + cy ) * this->surface->pitch + ( x + cx ) * bytesperpixel + i ];
			}
		}
	}
	SDL_UnlockSurface(this->surface);
}

Image *Image::copy(int x, int y, int w, int h) const {
	//LOG("Image::copy(%d,%d,%d,%d), (SDL version)\n",x,y,w,h);

	/*x *= scale_x;
	y *= scale_y;
	w *= scale_x;
	h *= scale_y;*/
	x = (int)(x * scale_x);
	y = (int)(y * scale_y);
	w = (int)(w * scale_x);
	h = (int)(h * scale_y);

	Image *copy_image = createBlankImage(w,h,this->surface->format->BitsPerPixel);
	copy_image->surface->format->Amask = this->surface->format->Amask;

	unsigned char *src_data = (unsigned char *)this->surface->pixels;
	unsigned char *dst_data = (unsigned char *)copy_image->surface->pixels;
	SDL_LockSurface(this->surface);
	int bytesperpixel = this->surface->format->BytesPerPixel;
	for(int cx=0;cx<w;cx++) {
		for(int cy=0;cy<h;cy++) {
			for(int i=0;i<bytesperpixel;i++) {
				int src_indx = ( y + cy ) * this->surface->pitch + ( x + cx ) * bytesperpixel + i;
				int dst_indx = cy * copy_image->surface->pitch + cx * bytesperpixel + i;
				ASSERT( src_indx >= 0 && src_indx < this->surface->pitch * this->surface->h * bytesperpixel );
				ASSERT( dst_indx >= 0 && dst_indx < copy_image->surface->pitch * copy_image->surface->h * copy_image->surface->format->BytesPerPixel );
				dst_data[ dst_indx ] = src_data[ src_indx ];
			}
		}
	}
	SDL_UnlockSurface(this->surface);

	copy_image->copyPalette(this);
	copy_image->scale_x = scale_x;
	copy_image->scale_y = scale_y;

	return copy_image;
}

Image *Image::loadImage(const char *filename) {
	LOG("Image::loadImage(\"%s\") (SDL version)\n",filename);
	//image->surface=IMG_LoadLBM_RW(SDL_RWFromFile(filename, "rb"));
	SDL_RWops *src = SDL_RWFromFile(filename, "rb");
	if( src == NULL ) {
		LOG("SDL_RWFromFile failed: %s\n", SDL_GetError());
		return NULL;
	}
	Image *image = new Image();
	image->surface = IMG_Load_RW(src, 1);
	if( image->surface == NULL ) {
		LOG("IMG_Load_RW failed: %s\n", IMG_GetError());
		delete image;
		image = NULL;
	}
	return image;
}


//Image *Image::loadImage(char *filename) {
//LOG("Image::loadImage(\"%s\")\n",filename);
//Image *image = NULL;
////FREE_IMAGE_FORMAT type = FreeImage_GetFileType(filename, 0);
////FREE_IMAGE_FORMAT type = FreeImage_GetFileTypeFromExt("iff",0);
////LOG("Got fileformat %d, should be %d.\n", type, FIF_IFF);
//FREE_IMAGE_FORMAT type = FIF_GIF;
//FIBITMAP *bitmap = FreeImage_Load(type, filename,0);
//LOG("Got result %d.\n", bitmap);
//RGBQUAD *palette = FreeImage_GetPalette(bitmap);
///*palette[0].rgbRed = 255;
//palette[0].rgbGreen = 0;
//palette[0].rgbBlue = 255;*/
///*if( bitmap != NULL && FreeImage_GetBPP(bitmap) != 24 ) {
//bitmap = FreeImage_ConvertTo24Bits(bitmap);
//}*/
//
///*if( image->bitmap != NULL ) {
//image->bitmap = FreeImage_ConvertTo8Bits(image->bitmap);
//}*/
//
//if( bitmap != NULL ) {
//LOG("In if.\n");
//image = new Image();
////image->bitmap = bitmap;
//int w = FreeImage_GetWidth(bitmap);
//int h = FreeImage_GetHeight(bitmap);
//int bpp = FreeImage_GetBPP(bitmap);
//palette = FreeImage_GetPalette(bitmap);
//unsigned char *fi_data = (unsigned char *)FreeImage_GetBits(bitmap);
///*unsigned char *data = (unsigned char *)FreeImage_GetBits(image->bitmap);
//image->n_colours = FreeImage_GetColorsUsed(image->bitmap);*/
//int pitch = FreeImage_GetPitch(bitmap);
//int bytesperpixel = bpp / 8;
//image->data = new unsigned char[h * pitch];
//for(int i=0;i<h * pitch;i++)
//image->data[i] = fi_data[i];
//
//// flip
//for(int x=0;x<w;x++) {
//for(int y=0;y<h/2;y++) {
//int y2 = h - 1 - y;
//for(int i=0;i<bytesperpixel;i++) {
//unsigned char dummy = image->data[ y * pitch + x * bytesperpixel + i ];
//image->data[ y * pitch + x * bytesperpixel + i ] = image->data[ y2 * pitch + x * bytesperpixel + i ];
//image->data[ y2 * pitch + x * bytesperpixel + i ] = dummy;
//}
//}
//}
//
//Uint32 rmask, gmask, bmask, amask;
//#if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
//rmask = 0xff000000;
//gmask = 0x00ff0000;
//bmask = 0x0000ff00;
//amask = 0x000000ff;
//#else
//rmask = 0x000000ff;
//gmask = 0x0000ff00;
//bmask = 0x00ff0000;
//amask = 0xff000000;
//#endif
//if( palette == NULL ) {
//rmask = FreeImage_GetRedMask(bitmap);
//gmask = FreeImage_GetGreenMask(bitmap);
//bmask = FreeImage_GetBlueMask(bitmap);
//}
//amask = 0;
//image->surface = SDL_CreateRGBSurfaceFrom(image->data, w, h, bpp, pitch, rmask, gmask, bmask, amask);
//image->need_to_free_data = true;
//if( palette != NULL ) {
//for(int i=0;i<256;i++) {
//SDL_Color color = { palette[i].rgbRed, palette[i].rgbGreen, palette[i].rgbBlue };
//SDL_SetColors(image->surface, &color, i, 1);
//}
//}
//}
//else {
//LOG("Nope.\n");
//delete image;
//image = NULL;
//}
//return image;
//}

Image *Image::createBlankImage(int width,int height, int bpp) {
	Uint32 rmask, gmask, bmask, amask;

#if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
	rmask = 0xff000000;
	gmask = 0x00ff0000;
	bmask = 0x0000ff00;
	amask = 0x000000ff;
#else
	rmask = 0x000000ff;
	gmask = 0x0000ff00;
	bmask = 0x00ff0000;
	amask = 0xff000000;
#endif

	SDL_Surface *surface = SDL_CreateRGBSurface(SDL_SWSURFACE, width, height, bpp, rmask, gmask, bmask, amask);

	Image *image = new Image();
	image->surface = surface;
	image->data = (unsigned char *)image->surface->pixels;
	image->need_to_free_data = false;

	return image;
}

void Image::setGraphicsOutput(SDL_Surface *dest_surf) {
	Image::dest_surf = dest_surf;
}

void Image::writeNumbers(int x,int y,Image *images[10],int number,Justify justify,bool mask) {
	char buffer[16] = "";
	sprintf(buffer,"%d",number);
	int len = strlen(buffer);
	int w = images[0]->getScaledWidth();
	int sx = 0;
	if( justify == JUSTIFY_LEFT )
		sx = x;
	else if( justify == JUSTIFY_CENTRE )
		sx = x - ( w * len ) / 2;
	else if( justify == JUSTIFY_RIGHT )
		sx = x - w * len;

	for(int i=0;i<len;i++) {
		images[ buffer[i] - '0' ]->draw(sx, y, mask);
		sx += w;
	}
}

void Image::write(int x,int y,Image *images[26],const char *text,Justify justify,bool mask) {
	writeMixedCase(x, y, images, images, NULL, text, justify, mask);
}

void Image::writeMixedCase(int x,int y,Image *large[26],Image *little[26],Image *numbers[10],const char *text,Justify justify,bool mask) {
	int len = strlen(text);
	int n_lines = 0;
	int max_wid = 0;
	textLines(&n_lines, &max_wid, text);
	ASSERT(n_lines == 1 || justify == JUSTIFY_LEFT);
	int w = little[0]->getScaledWidth();
	int n_h = 0;
	if( numbers != NULL )
		n_h = numbers[0]->getScaledHeight();
	int s_h = little[0]->getScaledHeight();
	int l_h = large[0]->getScaledHeight();
	int sx = 0;
	if( justify == JUSTIFY_LEFT )
		sx = x;
	else if( justify == JUSTIFY_CENTRE )
		sx = x - ( w * max_wid ) / 2;
	else if( justify == JUSTIFY_RIGHT )
		sx = x - w * max_wid;
	int cx = sx;

	for(int i=0;i<len;i++) {
		char ch = text[i];
		if( numbers == NULL && ch == '0' ) {
			ch = 'O'; // hack for 0hm (we don't spell it Ohm, due to alphabetical ordering)
		}
		if( ch == '\n' ) {
			// newline
			cx = sx;
			y += l_h + 2;
			continue; // don't increase sx
		}
		else if( isspace( ch ) )
			; // do nothing
		else if( ch >= '0' && ch <= '9' ) {
			ASSERT( numbers != NULL );
			int indx = ch - '0';
			numbers[indx]->draw(cx, y + l_h - n_h, mask);
		}
		else if( isupper( ch ) ) {
			int indx = ch - 'A';
			large[indx]->draw(cx, y, mask);
		}
		else {
			little[ ch - 'a' ]->draw(cx, y + l_h - s_h, mask);
		}
		cx += w;
	}
}

