//---------------------------------------------------------------------------
#ifndef imageH
#define imageH

/** Class for loading and handling images.
*/

#include "resource.h"

//struct FIBITMAP;

class Image : public TrackedObject {
	//int n_colours;
	//FIBITMAP *bitmap;
	unsigned char *data;
	bool need_to_free_data;
	SDL_Surface *surface;
	static SDL_Surface *dest_surf;
	//Uint32 userData;
	float scale_x, scale_y;
	SDL_Color maskcol;

	Image();

	void free();

	//       static Image *createFromBitmap(FIBITMAP *bitmap, Uint32 rmask, Uint32 gmask, Uint32 bmask, Uint32 amask);
public:
	virtual ~Image();
	virtual const char *getClass() const { return "CLASS_IMAGE"; }

	void draw(int x, int y,bool mask) const;
	//void drawAlpha(int x, int y,float alpha);
	//void draw(int x, int y,bool mask,int scale_w, int scale_h);
	int getWidth() const;
	int getHeight() const;
	int getScaledWidth() const {
		return (int)(this->getWidth() / scale_x);
	}
	int getScaledHeight() const {
		return (int)(this->getHeight() / scale_y);
	}
	const SDL_Surface *getSDLSurface() const;
	SDL_Surface *getSDLSurface();
	void convertToDisplayFormat();
	bool copyPalette(const Image *image);
	float getScaleX() const {
		return scale_x;
	}
	float getScaleY() const {
		return scale_y;
	}
	void setScale(float scale_x,float scale_y);
	bool scaleTo(int n_w);
	//bool scaleTo(int n_w,int n_h);
	void scale(float sx,float sy);
	void smooth(bool mask);
	void remap(unsigned char sr,unsigned char sg,unsigned char sb,unsigned char rr,unsigned char rg,unsigned char rb);
	void reshadeRGB(int from, bool to_r, bool to_g, bool to_b);
	void brighten(float sr, float sg, float sb,bool mask);
	void flipX();
	Image *copy(int x,int y,int w,int h) const;
	Image *copy() const {
		// need to used scaled width/height, to counteract scaling in copy()
		return this->copy(0, 0, getScaledWidth(), getScaledHeight());
	}
	void print(const char *inStr) const;
	bool isPaletted() const {
		return this->surface->format->palette != NULL;
	}
	int getNColors() const {
		return this->surface->format->palette->ncolors;
		//return 0;
	}
	unsigned char getPixelIndex(int x,int y) const;
	bool setPixelIndex(int x,int y,unsigned char c);
	bool setColor(int index,unsigned char r,unsigned char g,unsigned char b);
	bool setMaskColor(unsigned char r,unsigned char g,unsigned char b);
	SDL_Color getMaskColor() const {
		return maskcol;
	}
	bool createAlphaForColor(unsigned char ar, unsigned char ag, unsigned char ab, unsigned char alpha);
	bool convertToHiColor(bool alpha);
	//void convertToFormat(Image *image);
	/*void setUserInt(int data) {
	this->userData = (Uint32)data;
	}
	void setUserData(void *data) {
	this->userData = (Uint32)data;
	}
	int getUserInt() {
	return (int)this->userData;
	}
	void *getUserData() {
	return (void *)this->userData;
	}*/

	static Image * loadImage(const char *filename);
	static Image * createBlankImage(int width,int height, int bpp);
	static void setGraphicsOutput(SDL_Surface *dest_surf);

	enum Justify {
		JUSTIFY_LEFT = 0,
		JUSTIFY_CENTRE = 1,
		JUSTIFY_RIGHT = 2,
	};
	static void writeNumbers(int x,int y,Image *images[10],int number,Justify justify,bool mask);
	static void write(int x,int y,Image *images[26],const char *text,Justify justify,bool mask);
	//static void writeMixedCase(int x,int y,Image *large[26],Image *small[26],Image *numbers[10],char *text,Justify justify,bool mask);
	static void writeMixedCase(int x,int y,Image *large[26],Image *little[26],Image *numbers[10],const char *text,Justify justify,bool mask);
};

//---------------------------------------------------------------------------
#endif
