//---------------------------------------------------------------------------
#ifndef panelH
#define panelH

/** Set of classes for handling a general GUI - ideally there shouldn't be
*   anything specific to a particular game in these classes.
*/

#include "resource.h"

class Image;

inline bool mouse_left(Uint32 m_b) {
	return ( m_b & SDL_BUTTON(1) ) != 0;
}

inline bool mouse_right(Uint32 m_b) {
	return ( m_b & SDL_BUTTON(3) ) != 0;
}

void registerClick();

#include <vector>
using std::vector;
#include <string>
using std::string;

class PanelPage : public TrackedObject {
private:
	void init_panelpage();
protected:
	bool enabled;
	//Vector *children;
	vector <PanelPage *> *children;
	int popup_x;
	int popup_y;
	bool popup_item;

	bool helpTextOn;
	/*char infoLMB[GUI_MAX_STRING+1];
	char infoRMB[GUI_MAX_STRING+1];
	char infoBMB[GUI_MAX_STRING+1];*/
	string infoLMB;
	string infoRMB;
	string infoBMB;

	int offset_x, offset_y;
	int w, h;

	PanelPage *owner;
	bool survive_owner;

	virtual void drawPopups();
public:

	PanelPage(int offset_x,int offset_y);
	PanelPage(int offset_x,int offset_y,const char *infoLMB);
	PanelPage(int offset_x,int offset_y,int w,int h);
	virtual ~PanelPage();

	virtual const char *getClass() const { return "CLASS_PANELPAGE"; }
	virtual void add(PanelPage *panel);
	virtual void remove(PanelPage *panel);
	virtual PanelPage *get(int index) {
		//return (PanelPage *)this->children->elementAt(index);
		return this->children->at(index);
	}
	virtual int nChildren() const {
		return children->size();
	}
	virtual void setEnabled(bool enabled);
	virtual bool isEnabled() const {
		return this->enabled;
	}
	virtual void enableHelpText(bool helpTextOn) {
		this->helpTextOn = helpTextOn;
	}
	virtual bool isHelpTextOn() const {
		return this->helpTextOn;
	}
	virtual void setInfoLMB(const char *text);
	virtual void setInfoRMB(const char *text);
	virtual void setInfoBMB(const char *text);
	virtual const char *getInfoLMB() const;
	virtual const char *getInfoRMB() const;
	virtual const char *getInfoBMB() const;
	void setSurviveOwner(bool survive_owner) {
		this->survive_owner = survive_owner;
	}

	virtual int getLeft() const;
	virtual int getTop() const;
	virtual int getRight() const;
	virtual int getXCentre() const;
	virtual int getBottom() const;
	int getWidth() const {
		return this->w;
	}
	int getHeight() const {
		return this->h;
	}

	virtual void free(bool free_this);
	virtual void draw();
	virtual bool mouseOver(int m_x,int m_y);
	virtual void input(int m_x,int m_y,Uint8 m_b);
};

class Button : public PanelPage {
	//void init_button();
	Image **font;
	string text;
public:

	/*Button(int x,int y,Image *image);
	Button(int x,int y,Image *image,char *infoLMB);
	Button(int x,int y,int w,int h,Image *image);*/
	Button(int x,int y,const char *text,Image *font[]);
	virtual ~Button();

	virtual const char *getClass() const { return "CLASS_BUTTON"; }
	virtual void draw();
};

class ImageButton : public PanelPage {
	const Image *image;
public:

	ImageButton(int x,int y,const Image *image);
	ImageButton(int x,int y,const Image *image,const char *infoLMB);
	ImageButton(int x,int y,int w,int h,const Image *image);
	virtual ~ImageButton();

	void setImage(const Image *image) {
		this->image = image;
	}
	virtual const char *getClass() const { return "CLASS_IMAGEBUTTON"; }
	virtual void draw();
};

class CycleButton : public PanelPage {
	int active;
	Image **font;
	char **texts;
	int n_texts;

public:
	CycleButton(int x,int y,char *texts[],int n_texts,Image *font[]);
	virtual ~CycleButton();

	virtual const char *getClass() const { return "CLASS_CYCLEBUTTON"; }
	virtual void draw();
	virtual int getActive() const {
		return this->active;
	}
	virtual void setActive(int active);
	virtual void input(int m_x,int m_y,Uint8 m_b);
};

class MultiPanel : public PanelPage {
protected:
	int c_page;
public:
	MultiPanel(int n_pages,int x,int y);
	virtual ~MultiPanel();

	//virtual void free();
	virtual void addToPanel(int page,PanelPage *panel);

	//virtual void drawPopups();
	virtual void draw();
	virtual void input(int m_x,int m_y,Uint8 m_b);
	virtual void setPage(int page) {
		this->c_page = page;
	}
	virtual int getPage() const {
		return this->c_page;
	}
};

//---------------------------------------------------------------------------
#endif
