//---------------------------------------------------------------------------
#include <cstdio>

#ifdef __linux
#include <fmodex/fmod_errors.h>
#else
#include <fmod_errors.h>
#endif

#include "sound.h"
#include "utils.h"
#include "game.h"
#include "gamestate.h"
//---------------------------------------------------------------------------

//const int header_c = 0x60;
const int header_c = 0x64;
//const int header_c = 0;

FMOD::System *fmod_system = NULL;
bool error_occurred = false;

bool errorSound() {
	bool rtn = error_occurred;
	//error_occurred = false;
	return rtn;
}

Sample::~Sample() {
	/*if( this->fsound_sample != NULL ) {
		FSOUND_Sample_Free(this->fsound_sample);
	}*/
	if( this->sound != NULL ) {
		this->sound->release();
	}
}

bool initSound() {
#ifdef OLD_FMOD
#ifdef __linux
	FSOUND_SetOutput(FSOUND_OUTPUT_ALSA);
#else
	FSOUND_SetOutput(FSOUND_OUTPUT_DSOUND);
#endif
	/*
	// no need to set FSOUND_SetMemorySystem or FSOUND_SetBufferSize - doing both of them causes streaming to crash, if no samples are loaded
	if (!FSOUND_SetMemorySystem(malloc(4*1024*1024), 4*1024*1024, NULL, NULL, NULL))
	{
		LOG("FSOUND_SetMemorySystem failed!!!\n");
		LOG("%s\n", FMOD_ErrorString(FSOUND_GetError()));
		error_occurred = true;
		return false;
	}
	FSOUND_SetBufferSize(100);*/
	FSOUND_SetDriver(0);
	if (!FSOUND_Init(44100, 32, FSOUND_INIT_USEDEFAULTMIDISYNTH))
	//if (!FSOUND_Init(44100, 32, 0))
	{
		LOG("FSOUND_Init failed!!!\n");
		LOG("%s\n", FMOD_ErrorString(FSOUND_GetError()));
		error_occurred = true;
		return false;
	}

	return true;
#endif

	FMOD_RESULT result = FMOD::System_Create(&fmod_system);
	if( result != FMOD_OK ) {
		LOG("FMOD error creating system! (%d) %s\n", result, FMOD_ErrorString(result));
		return false;
	}
	else if( fmod_system == NULL ) {
		LOG("FMOD system is NULL\n");
		return false;
	}

/*#ifdef __linux
	fmod_system->setOutput(FMOD_OUTPUTTYPE_ALSA);
#else
	fmod_system->setOutput(FMOD_OUTPUTTYPE_DSOUND);
#endif*/

	result = fmod_system->init(32, FMOD_INIT_NORMAL, 0);
	if( result != FMOD_OK ) {
		LOG("FMOD error initialising system! (%d) %s\n", result, FMOD_ErrorString(result));
		fmod_system->release();
		fmod_system = NULL;
		return false;
	}

	return true;
}

void updateSound() {
	if( fmod_system != NULL ) {
		fmod_system->update();
	}
}

void freeSound() {
	//FSOUND_Close();
	if( fmod_system != NULL ) {
		fmod_system->release();
		fmod_system = NULL;
	}
}

Sample *loadSample(const char *filename, bool iff) {
	LOG("loadSample %s\n", filename);
	//FSOUND_SAMPLE *sample = FSOUND_Sample_Load(FSOUND_FREE, filename, FSOUND_LOADRAW, 0);
	/*FSOUND_SAMPLE *fsound_sample = FSOUND_Sample_Load(FSOUND_FREE, filename, FSOUND_LOADRAW, 0, 0);
	if( !fsound_sample ) {
		LOG("failed to load %s\n", filename);
		error_occurred = true;
	}
	// we still create the structure even if no sample is loaded, as we also use for displaying the associated text
	Sample *sample = new Sample(fsound_sample);*/
	FMOD::Sound *sound = NULL;
	if( fmod_system != NULL ) {
		FMOD_RESULT result = FMOD_ERR_MUSIC_UNINITIALIZED;
		if( iff ) {
			FMOD_CREATESOUNDEXINFO info;
			memset(&info, NULL, sizeof(info));
			info.cbsize = sizeof(info);
			info.defaultfrequency = 14500;
			info.numchannels = 1;
			info.format = FMOD_SOUND_FORMAT_PCM8;
			info.fileoffset = header_c;
			result = fmod_system->createSound(filename, FMOD_OPENRAW, &info, &sound);
		}
		else {
			result = fmod_system->createSound(filename, FMOD_DEFAULT, 0, &sound);
		}
		if( result != FMOD_OK ) {
			LOG("failed to load %s\n", filename);
			LOG("FMOD error %d: %s\n", result, FMOD_ErrorString(result));
			error_occurred = true;
			sound = NULL; // just to be sure
		}
		else if( sound == NULL ) {
			LOG("null sound when loading %s\n", filename);
			error_occurred = true;
		}
	}
	// we still create the structure even if no sample is loaded, as we also use for displaying the associated text
	Sample *sample = new Sample(sound);
	return sample;
}

Sample *loadSamplesChained(const char *filename1,const char *filename2,const char *filename3) {
	int n_buffers = 1;
	if( filename2 != NULL )
		n_buffers++;
	if( filename3 != NULL )
		n_buffers++;
	/*if( n_buffers == 1 )
	return loadSample(filename1);*/

	//FSOUND_SAMPLE *fsound_sample = NULL;
	FMOD::Sound *sound = NULL;
	if( fmod_system != NULL ) {
		FILE *file[3];
		file[0] = fopen(filename1, "rb");
		file[1] = filename2 == NULL ? NULL : fopen(filename2, "rb");
		file[2] = filename3 == NULL ? NULL : fopen(filename3, "rb");
		bool ok = true;
		for(int i=0;i<n_buffers && ok;i++) {
			ok = file[i] != NULL;
		}

		if( ok ) {
			int size = 0;
			int lengths[3];
			for(int i=0;i<n_buffers && ok;i++) {
				long curpos = ftell(file[i]);
				fseek(file[i], 0L, SEEK_END);
				lengths[i] = ftell(file[i]);
				//lengths[i] *= 2;
				//lengths[i] += 0x60;
				/*if( i > 0 )
					lengths[i] -= header_c;*/
				lengths[i] -= header_c;
				fseek(file[i], curpos, SEEK_SET);
				size += lengths[i];
			}
			char *buffer = new char[size];
			for(int i=0;i<size;i++) {
				buffer[i] = '\0';
			}
			//char *ptr = buffer;
			int offset = 0;
			for(int i=0;i<n_buffers && ok;i++) {
				/*if( i > 0 )
					fseek(file[i], header_c, SEEK_SET);*/
				fseek(file[i], header_c, SEEK_SET);
				fread(&buffer[offset], sizeof(char), lengths[i], file[i]);
				offset += lengths[i];
			}
			/*for(int i=0;i<128;i++) {
				char ch = buffer[i];
				LOG("%d : %d : %c\n", i, ch, ch);
			}*/
			//fsound_sample = FSOUND_Sample_Load(FSOUND_FREE, buffer, FSOUND_LOADMEMORY | FSOUND_LOADRAW, size);
			/*fsound_sample = FSOUND_Sample_Load(FSOUND_FREE, buffer, FSOUND_LOADMEMORY | FSOUND_LOADRAW, 0, size);
			if( !fsound_sample ) {
				LOG("failed to load chained samples\n");
				error_occurred = true;
			}*/
			FMOD_CREATESOUNDEXINFO info;
			memset(&info, NULL, sizeof(info));
			info.cbsize = sizeof(info);
			info.defaultfrequency = 14500;
			info.numchannels = 1;
			info.format = FMOD_SOUND_FORMAT_PCM8;
			info.length = size;
			FMOD_RESULT result = fmod_system->createSound(buffer, FMOD_OPENMEMORY | FMOD_OPENRAW, &info, &sound);
			if( result != FMOD_OK ) {
				LOG("failed to load chained samples\n");
				LOG("FMOD error %d: %s\n", result, FMOD_ErrorString(result));
				error_occurred = true;
			}
			else if( sound == NULL ) {
				LOG("null sound when loading chained samples\n");
				error_occurred = true;
			}
		}
		else {
			LOG("failed to open chained samples\n");
			error_occurred = true;
		}

		for(int i=0;i<n_buffers;i++) {
			if( file[i] != NULL )
				fclose(file[i]);
		}
	}
	//Sample *sample = new Sample(fsound_sample); // still create sample struct even if failed to load sample - see note above in loadSample()
	Sample *sample = new Sample(sound); // still create sample struct even if failed to load sample - see note above in loadSample()

	return sample;
}

Sample *loadMusic(const char *filename) {
	LOG("loadMusic %s\n", filename);
	FMOD::Sound *sound = NULL;
	if( fmod_system != NULL ) {
		/*FMOD_CREATESOUNDEXINFO info;
		memset(&info, NULL, sizeof(info));
		info.cbsize = sizeof(info);
		info.defaultfrequency = 44100;
		info.numchannels = 1;
		info.format = FMOD_SOUND_FORMAT_PCM8;*/
		FMOD_RESULT result = fmod_system->createStream(filename, FMOD_DEFAULT | FMOD_LOOP_NORMAL, 0, &sound);
		if( result != FMOD_OK ) {
			LOG("failed to load %s\n", filename);
			LOG("FMOD error %d: %s\n", result, FMOD_ErrorString(result));
			error_occurred = true;
		}
		else if( sound == NULL ) {
			LOG("null sound when loading %s\n", filename);
			error_occurred = true;
		}
	}
	// we still create the structure even if no sample is loaded, as we also use for displaying the associated text
	Sample *sample = new Sample(sound);
	return sample;
}

void Sample::play(int ch) {
	//int ch = FSOUND_PlaySound(block ? 0 : FSOUND_FREE, sample);
	/*if( this->fsound_sample != NULL )  {
		int ch = FSOUND_PlaySound(0, this->fsound_sample);
		//FSOUND_SetFrequency(ch, 15500);
		FSOUND_SetFrequency(ch, 14500);
		FSOUND_SetCurrentPosition(ch, header_c);
	}*/
	if( fmod_system != NULL && this->sound != NULL ) {
		FMOD::Channel *channel = NULL;
		// prefer to reuse channels, so previous sample is cut short to make way for this one
		FMOD_RESULT result = fmod_system->getChannel(ch, &channel);
		if( result != FMOD_OK ) {
			LOG("FMOD error getting channel %d: %s\n", result, FMOD_ErrorString(result));
			channel = NULL;
		}
		else {
			result = fmod_system->playSound(FMOD_CHANNEL_REUSE, sound, false, &channel);
			//FMOD_RESULT result = fmod_system->playSound(FMOD_CHANNEL_FREE, sound, false, &channel);
			if( result != FMOD_OK ) {
				LOG("FMOD error playing sound %d: %s\n", result, FMOD_ErrorString(result));
			}
			else {
				//channel->setFrequency(14500);
				//FMOD_RESULT result = channel->setPosition(header_c, FMOD_TIMEUNIT_PCMBYTES);
				/*if( result != FMOD_OK ) {
					LOG("FMOD error %d: %s\n", result, FMOD_ErrorString(result));
				}*/
			}
		}
	}
	if( this->text.length() > 0 ) {
		int ypos = original_background ? 220 : 240; // need to move text higher up, to avoid clashing with "Mega Lo Mania" logo
		TextEffect *effect = new TextEffect(this->text, 160, ypos, 2000);
		//gamestate->addEffect(effect);
		gamestate->addTextEffect(effect);
	}
}

bool isPlaying(int ch) {
	bool is_playing = false;
	if( fmod_system != NULL ) {
		FMOD::Channel *channel = NULL;
		FMOD_RESULT result = fmod_system->getChannel(ch, &channel);
		if( result != FMOD_OK ) {
			LOG("FMOD error getting channel %d: %s\n", result, FMOD_ErrorString(result));
			channel = NULL;
		}
		else {
			result = channel->isPlaying(&is_playing);
			if( result != FMOD_OK ) {
				LOG("FMOD error getting channel is playing %d: %s\n", result, FMOD_ErrorString(result));
			}
		}
	}
	return is_playing;
}

/*void playStream(FSOUND_STREAM *stream) {
	FSOUND_Stream_Play(FSOUND_FREE, stream);
	//FSOUND_Stream_Play(0, stream);
	//int ch = FSOUND_Stream_Play(FSOUND_FREE, stream);
	//FSOUND_SetFrequency(ch, 14500);
	//FSOUND_SetCurrentPosition(ch, 0x60);
}*/

/*void F_CALLBACKAPI ordercallback(FMUSIC_MODULE *mod, unsigned char param)
{
printf("\nOrder Callback : param %d\n", param);
}

void F_CALLBACKAPI instcallback(FMUSIC_MODULE *mod, unsigned char param)
{
printf("\nInst Callback : param %d\n", param);
}*/

/*void playModule(FMUSIC_MODULE *mod) {
	//FMUSIC_SetOrderCallback(mod, ordercallback, 1);
	//FMUSIC_SetInstCallback(mod, instcallback, 5);
	FMUSIC_SetMasterVolume(mod, 192);
	FMUSIC_SetLooping(mod, true);
	FMUSIC_SetPanSeperation(mod, 0);
	FMUSIC_PlaySong(mod);

}

void stopModule(FMUSIC_MODULE *mod) {
	FMUSIC_StopSong(mod);
	//FMUSIC_FreeSong(mod);
}*/

