//---------------------------------------------------------------------------
#ifdef _WIN32
#define _CRT_SECURE_CPP_OVERLOAD_STANDARD_NAMES 1
#endif

#ifdef __linux
#include <SDL/SDL.h>
#else
#include <sdl.h>
#endif

#ifdef _WIN32
#include <shlobj.h>
#include <Shlwapi.h>
#pragma comment(lib,"shlwapi.lib")

#include <io.h> // for access
#include <direct.h> // for mkdir
#define access _access
#endif

#include <cstdarg>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <cassert>

#include "game.h"
#include "utils.h"
//---------------------------------------------------------------------------

//const bool DEBUG = true;
//const int DEBUGLEVEL = 4;

const int MAX_TIME = 1000;
const int TICK_INTERVAL = 25; // 40 fps max
const int ticks_per_frame_c = 100;
const float time_ratio_c = 0.15f; // game time ticks per time ticks [correct]
//const float time_ratio_c = 0.5f; // game time ticks per time ticks [fast]
int time_rate = 1; // time factor

static int next_time = 0;
int frame_counter = 0;

int timeLeft() {
	int now = SDL_GetTicks();
	int res = 0;
	if(next_time > now)
		res = next_time - now;
	next_time = now + TICK_INTERVAL;
	return res;
}

static int real_time = 0;
static int game_time = 0;
static int loop_time = 0;
//static int current_time = 0;
//static int new_time = 0;

int getRealTime() {
	return real_time;
}

int getGameTime() {
	return game_time;
}

int getLoopTime() {
	//return new_time - current_time;
	return loop_time;
}

/*void initTime(int time) {
current_time = time;
}*/

void updateTime(int time) {
	if( time > MAX_TIME )
		time = MAX_TIME;
	real_time += time;
	loop_time = (int)(time * time_ratio_c * time_rate);
	game_time += loop_time;
	frame_counter = getRealTime() / ticks_per_frame_c;
}

int mouseTime = -1;

void resetMouseClick() {
	mouseTime = -1;
}

int getNClicks() {
	if( mouseTime == -1 )
		mouseTime = getRealTime();
	int time = getRealTime() - mouseTime;
	if( time < 2000 )
		return 1;
	else if( time < 5000 )
		return 2;
	return 3;
}

/* Return probability (as a proportion of RAND_MAX) that at least one poisson event
* occurred within the time_interval, given the mean number of time units per event.
*/
int poisson(int mean_ticks_per_event,int time_interval) {
	if( mean_ticks_per_event == 0 )
		return RAND_MAX;
	ASSERT( mean_ticks_per_event > 0 );
	int prob = (int)(RAND_MAX * ( 1.0 - exp( - ((double)time_interval) / mean_ticks_per_event ) ));
	return prob;
}

int n_digits(int number) {
	int num = 0;
	if( number < 0 ) {
		number = - number;
		num++;
	}
	while( number != 0 ) {
		num++;
		number /= 10;
	}
	return num;
}

void textLines(int *n_lines,int *max_wid,const char *text) {
	*n_lines = 0;
	*max_wid = 0;
	const char *ptr = text;
	const char *n_ptr = ptr;
	for(;;) {
		*n_lines = *n_lines + 1;

		while( *n_ptr != '\n' && *n_ptr != '\0' )
			n_ptr++;

		int wid = (int)(n_ptr - ptr);
		if( wid > *max_wid )
			*max_wid = wid;

		if( *n_ptr == '\0' )
			break;
		n_ptr++;
		ptr = n_ptr;
	}
}

char application_name[] = "Gigalomania";

FILE *logfile = NULL;
#ifdef _WIN32
char application_path[MAX_PATH] = "";
char logfilename[MAX_PATH] = "";
char oldlogfilename[MAX_PATH] = "";
#else
char application_path[] = "";
char logfilename[] = "log.txt";
char oldlogfilename[] = "log_old.txt";
#endif

char *getApplicationFilename(const char *name) {
#ifdef _WIN32
	char *filename = new char[MAX_PATH];
	strcpy(filename, application_path);
	PathAppendA(filename, name);
#else
	char *filename = new char[strlen(name)+1];
	strcpy(filename, name);
#endif
	return filename;
}

void initLogFile() {
	/*remove("log_old.txt");
	rename("log.txt","log_old.txt");
	remove("log.txt");*/

#ifdef _WIN32
	// first need to establish full path, and create folder if necessary
	bool ok = true;
    if ( SUCCEEDED( SHGetFolderPathA( NULL, CSIDL_APPDATA,
                                     NULL, 0, logfilename ) ) ) {
        PathAppendA(logfilename, application_name);

		if( access(logfilename, 0) != 0 ) {
			// folder doesn't seem to exist - try creating it
			int res = mkdir(logfilename);
			//int res = 1; // test
			if( res != 0 ) {
				printf("Failed to create folder for application data!\n");
				MessageBoxA(NULL, "Failed to create folder for application data - storing in local folder instead.\n", "Warning", MB_OK|MB_ICONEXCLAMATION);
				ok = false;
			}
		}
    }
	else {
		printf("Failed to obtain path for application folder!\n");
		MessageBoxA(NULL, "Failed to obtain path for application folder - storing in local folder instead.\n", "Warning", MB_OK|MB_ICONEXCLAMATION);
		ok = false;
	}

	if( ok ) {
		strcpy(application_path, logfilename);
		strcpy(oldlogfilename, logfilename);
		PathAppendA(logfilename, "log.txt");
		PathAppendA(oldlogfilename, "log_old.txt");
	}
	else {
		// just save in local directory and hope for the best!
		strcpy(application_path, "");
		strcpy(logfilename, "log.txt");
		strcpy(oldlogfilename, "log_old.txt");
	}
#endif

	remove(oldlogfilename);
	rename(logfilename, oldlogfilename);
	remove(logfilename);

	LOG("Initialising Log File...\n");
	LOG("Version %d.%d\n", majorVersion, minorVersion);

#ifdef _DEBUG
	LOG("Running in Debug mode\n");
#else
	LOG("Running in Release mode\n");
#endif

#ifdef _WIN32
	LOG("Platform: Windows\n");
#elif __linux
	LOG("Platform: Linux\n");
#else
	LOG("Platform: UNKNOWN\n");
#endif
}

bool log(const char *text,...) {
	//return true;
	logfile = fopen(logfilename,"at+");
	va_list vlist;
	char buffer[65536];
	va_start(vlist, text);
	vsprintf(buffer,text,vlist);
	if( logfile != NULL )
		fprintf(logfile,buffer);
	if( debugwindow )
		printf(buffer);
	va_end(vlist);
	if( logfile != NULL )
		fclose(logfile);
	return true;
}

